// Downloaded from https://repo.progsbase.com - Code Developed Using progsbase.

function LoadFont(bytes, fontReference, errorMessage){
  var f;
  var p;
  var success;

  f = {};
  fontReference.font = f;

  /* The font stores its data to avoid passing around an unnecessary parameter */
  f.data = bytes;

  success = true;

  p = CreateNumberReference(0);

  ReadOffsetSubtable(f, p);
  success = InitFont(f, p, errorMessage);

  if(success){
  }

  delete(p);

  return success;
}
function DrawText(x, y, image, font, text, fontSize, color){
  var xStart, scale, cpG;
  var i, xx, yy, kern;
  var unicode;
  var vm;
  var hm;
  var glyphBounds;
  var glyphBitmap;

  scale = GetScaleFactorForPixelHeight(font, fontSize);

  xStart = x;
  /* Save the starting x in case we advance to a new line */
  vm = GetFontVMetrics(font);
  vm.ascent = Math.round(vm.ascent*scale);
  vm.descent = Math.round(vm.descent*scale);
  vm.lineGap = Math.round(vm.lineGap*scale);

  unicode = StringToNumberArray(text);
  for(i = 0; i < unicode.length; i = i + 1){
    if(unicode[i] == 10){
      /* If we encounted a '\n' character, advance y to the next line. */
      y = y + vm.ascent - vm.descent + vm.lineGap;
      x = xStart;
    }else{
      cpG = GetGlyphIndex(font, unicode[i]);
      if(cpG != 0){
        hm = GetGlyphHMetrics(font, cpG);
        glyphBounds = GetGlyphBitmapBox(font, cpG, scale);

        yy = Math.round(y + glyphBounds.y0);
        xx = Math.round(x + hm.leftSideBearing*scale);

        glyphBitmap = GenerateGlyphBitmap(font, cpG, scale, 0.35, true);
        BlendGrayBitmapAndRGBABitmapWithColor(image, glyphBitmap, xx, yy, color);

        x = x + hm.advanceWidth*scale;

        /* Add kerning if there is another character after this one */
        if(i != unicode.length - 1){
          kern = GetGlyphKerningAdvance(font, cpG, GetGlyphIndex(font, unicode[i + 1]));
          x = x + Math.round(kern*scale);
        }
      }
    }
  }
}
function GetHeight(font, str, fontSize){
  var i, scale, yMin, yMax, lines, lineAdvance, glyphIndex;
  var unicode;
  var glyphBounds;
  var vm;

  unicode = StringToNumberArray(str);
  scale = GetScaleFactorForPixelHeight(font, fontSize);

  vm = GetFontVMetrics(font);
  vm.ascent = Math.round(vm.ascent*scale);
  vm.descent = Math.round(vm.descent*scale);
  vm.lineGap = Math.round(vm.lineGap*scale);

  lineAdvance = vm.ascent - vm.descent + vm.lineGap;

  lines = 0;

  yMin = 0;
  yMax = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    glyphIndex = GetGlyphIndex(font, unicode[i]);
    glyphBounds = GetGlyphBitmapBox(font, glyphIndex, scale);

    yMin = Math.min(yMin, glyphBounds.y0);
    yMax = Math.max(yMax, lines*lineAdvance + glyphBounds.y1);
  }

  return yMax - yMin;
}
function GetWidth(font, str, fontSize){
  var scale;
  var x;
  var i, glyphIndex;
  var kern;
  var unicode;
  var hm;

  unicode = StringToNumberArray(str);
  scale = GetScaleFactorForPixelHeight(font, fontSize);

  x = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    glyphIndex = GetGlyphIndex(font, unicode[i]);
    hm = GetGlyphHMetrics(font, glyphIndex);

    x = x + hm.advanceWidth*scale;

    /* Add kerning if there is another character after this one */
    if(i != unicode.length - 1){
      kern = GetGlyphKerningAdvance(font, glyphIndex, GetGlyphIndex(font, unicode[i + 1]));
      x = x + Math.round(kern*scale);
    }
  }

  return x;
}
function CreateLinkedListActiveEdges(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function CreateLinkedListActiveEdgesArray(lengthx){
  var lls;
  var i;

  lls = [];
  lls.length = lengthx;
  for(i = 0; i < lls.length; i = i + 1){
    lls[i] = CreateLinkedListActiveEdges();
  }

  return lls;
}
function LinkedListAddActiveEdge(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListActiveEdgesLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function LinkedListActiveEdgesIndex(ll, index){
  var i;
  var node;

  node = ll.first;
  for(i = 0; i < index; i = i + 1){
    node = node.next;
  }

  return node.value;
}
function LinkedListInsertActiveEdge(ll, index, value){
  var i;
  var node, tmp;

  if(index == 0){
    tmp = ll.first;
    ll.first = {};
    ll.first.next = tmp;
    ll.first.value = value;
    ll.first.end = false;
  }else{
    node = ll.first;
    for(i = 0; i < index - 1; i = i + 1){
      node = node.next;
    }

    tmp = node.next;
    node.next = {};
    node.next.next = tmp;
    node.next.value = value;
    node.next.end = false;
  }
}
function LinkedListRemoveActiveEdge(ll, index){
  var i;
  var node, prev;

  node = ll.first;
  prev = ll.first;

  for(i = 0; i < index; i = i + 1){
    prev = node;
    node = node.next;
  }

  if(index == 0){
    ll.first = prev.next;
  }
  if( !prev.next.end ){
    prev.next = prev.next.next;
  }
}
function FreeLinkedListActiveEdges(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function FreeLinkedListActiveEdgesArray(lls){
  var i;

  for(i = 0; i < lls.length; i = i + 1){
    FreeLinkedListActiveEdges(lls[i]);
  }
  delete(lls);
}
function LinkedListActiveEdgesToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListActiveEdgesLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = node.value;
    node = node.next;
  }

  return array;
}
function ArrayToLinkedListActiveEdges(array){
  var ll;
  var i;

  ll = CreateLinkedListActiveEdges();

  for(i = 0; i < array.length; i = i + 1){
    LinkedListAddActiveEdge(ll, array[i]);
  }

  return ll;
}
function LinkedListActiveEdgesEqual(a, b){
  var equal, done;
  var an, bn;

  an = a.first;
  bn = b.first;

  equal = true;
  done = false;
  for(; equal &&  !done ; ){
    if(an.end == bn.end){
      if(an.end){
        done = true;
      }else if(an.value == bn.value){
        an = an.next;
        bn = bn.next;
      }else{
        equal = false;
      }
    }else{
      equal = false;
    }
  }

  return equal;
}
function GetGlyphBitmapBox(f, glyphIndex, scale){
  var b, r;

  b = GetGlyphBox(f, glyphIndex);

  r = {};
  r.x0 = Math.floor(b.x0*scale);
  r.y0 = Math.floor( -b.y1*scale);
  r.x1 = Math.ceil(b.x1*scale);
  r.y1 = Math.ceil( -b.y0*scale);

  delete(b);

  return r;
}
function AddPoint(points, i, x, y){
  if(points.length != 0){
    points[i].x = x;
    points[i].y = y;
  }
}
function TesselateCurve(points, i, x0, y0, x1, y1, x2, y2, flatnessSq, n){
  var mx, my, dx, dy;

  /* Midpoint
 */
  mx = (x0 + 2*x1 + x2)/4;
  my = (y0 + 2*y1 + y2)/4;

  /* Versus directly drawn line
 */
  dx = (x0 + x2)/2 - mx;
  dy = (y0 + y2)/2 - my;

  if(n > 16){
  }else{

    if(dx*dx + dy*dy > flatnessSq){
      i = TesselateCurve(points, i, x0, y0, (x0 + x1)/2, (y0 + y1)/2, mx, my, flatnessSq, n + 1);
      i = TesselateCurve(points, i, mx, my, (x1 + x2)/2, (y1 + y2)/2, x2, y2, flatnessSq, n + 1);
    }else{
      AddPoint(points, i, x2, y2);
      i = i + 1;
    }
  }
  return i;
}
function TesselateCubic(points, i, x0, y0, x1, y1, x2, y2, x3, y3, objSpaceFlatnessSquared, n){
  var dx0, dy0, dx1, dy1, dx2, dy2, dx, dy, longlen, shortlen, flatnessSquared;
  var x01, y01, x12, y12, x23, y23, xa, ya, xb, yb, mx, my;
  dx0 = x1 - x0;
  dy0 = y1 - y0;
  dx1 = x2 - x1;
  dy1 = y2 - y1;
  dx2 = x3 - x2;
  dy2 = y3 - y2;
  dx = x3 - x0;
  dy = y3 - y0;
  longlen = (Math.sqrt(dx0*dx0 + dy0*dy0) + Math.sqrt(dx1*dx1 + dy1*dy1) + Math.sqrt(dx2*dx2 + dy2*dy2));
  shortlen = Math.sqrt(dx*dx + dy*dy);
  flatnessSquared = longlen*longlen - shortlen*shortlen;

  if(n > 16){
  }else if(flatnessSquared > objSpaceFlatnessSquared){
    x01 = (x0 + x1)/2;
    y01 = (y0 + y1)/2;
    x12 = (x1 + x2)/2;
    y12 = (y1 + y2)/2;
    x23 = (x2 + x3)/2;
    y23 = (y2 + y3)/2;

    xa = (x01 + x12)/2;
    ya = (y01 + y12)/2;
    xb = (x12 + x23)/2;
    yb = (y12 + y23)/2;

    mx = (xa + xb)/2;
    my = (ya + yb)/2;

    i = TesselateCubic(points, i, x0, y0, x01, y01, xa, ya, mx, my, objSpaceFlatnessSquared, n + 1);
    i = TesselateCubic(points, i, mx, my, xb, yb, x23, y23, x3, y3, objSpaceFlatnessSquared, n + 1);
  }else{
    AddPoint(points, i, x3, y3);
    i = i + 1;
  }

  return i;
}
function FlattenCurves(vertices, objSpaceFlatness, numContours, contourLengths){
  var result;
  var numPoints;
  var flatnessSq;
  var x, y, i, n, start, pass;
  var NEW_CONTOUR, LINE_TO, CURVE_TO, CUBIC_TO;
  /* Constants
 */
  NEW_CONTOUR = 1;
  LINE_TO = 2;
  CURVE_TO = 3;
  CUBIC_TO = 4;

  n = 0;

  /* Count how many "moves" there are to get the contour count
 */
  for(i = 0; i < vertices.length; i = i + 1){
    if(vertices[i].type == 1){
      n = n + 1;
    }
  }

  numPoints = 0;
  result = [];
  result.length = 0;

  numContours.numberValue = n;
  if(n != 0){
    flatnessSq = objSpaceFlatness*objSpaceFlatness;
    start = 0;

    contourLengths.numberArray = [];
    contourLengths.numberArray.length = n;

    /* make two passes through the points so we don't need to realloc
 */
    for(pass = 0; pass < 2; pass = pass + 1){
      x = 0;
      y = 0;

      if(pass == 1){
        /* We allocate a dummy zero size array in every case because Java
 */
        /* complains that the variable is not initialized, because the code
 */
        /* analyzer has problems with complex situations like these.
 */
        delete(result);
        result = [];
        result.length = numPoints;
        for(i = 0; i < numPoints; i = i + 1){
          result[i] = {};
        }
      }
      numPoints = 0;

      n =  -1;
      for(i = 0; i < vertices.length; i = i + 1){
        if(vertices[i].type == NEW_CONTOUR){

          if(n >= 0){
            contourLengths.numberArray[n] = numPoints - start;
          }
          n = n + 1;
          start = numPoints;

          x = vertices[i].x;
          y = vertices[i].y;
          AddPoint(result, numPoints, x, y);
          numPoints = numPoints + 1;
        }else if(vertices[i].type == LINE_TO){
          x = vertices[i].x;
          y = vertices[i].y;
          AddPoint(result, numPoints, x, y);
          numPoints = numPoints + 1;
        }else if(vertices[i].type == CURVE_TO){
          numPoints = TesselateCurve(result, numPoints, x, y, vertices[i].cx, vertices[i].cy, vertices[i].x, vertices[i].y, flatnessSq, 0);

          x = vertices[i].x;
          y = vertices[i].y;
        }else if(vertices[i].type == CUBIC_TO){
          numPoints = TesselateCubic(result, numPoints, x, y, vertices[i].cx, vertices[i].cy, vertices[i].cx1, vertices[i].cy1, vertices[i].x, vertices[i].y, flatnessSq, 0);

          x = vertices[i].x;
          y = vertices[i].y;
        }
      }
      contourLengths.numberArray[n] = numPoints - start;
    }
  }

  return result;
}
function SortEdgesQuickSort(e, s, n){
  var t;
  var m, i, j, z;
  var c, c01, c12, toBreak, toBreak2;

  /* Threshold for transitioning to insertion sort
 */
  for(; n > 12; ){
    /* Compute median of three
 */
    m = Math.floor(n/2);

    c01 = e[s].y0 < e[s + m].y0;
    c12 = e[s + m].y0 < e[s + n - 1].y0;

    /* If 0 >= mid >= end, or 0 < mid < end, then use mid
 */
    if(c01 != c12){
      /* Otherwise, we'll need to swap something else to middle
 */
      c = e[s].y0 < e[s + n - 1].y0;

      /* 0 > mid && mid < n: 0 > n => n; 0 < n => 0
 */
      /* 0 < mid && mid > n: 0 > n => 0; 0 < n => n
 */
      if(c == c12){
        z = 0;
      }else{
        z = n - 1;
      }

      t = e[s + z];
      e[s + z] = e[s + m];
      e[s + m] = t;
    }

    /* Now p[m] is the median-of-three.
 */
    /* Swap it to the beginning so it won't move around.
 */
    t = e[s];
    e[s] = e[s + m];
    e[s + m] = t;

    /* Partition loop
 */
    i = 1;
    j = n - 1;

    toBreak2 = false;
    for(;  !toBreak2 ; ){
      /* Handling of equality is crucial here.
 */
      /* or sentinels & efficiency with duplicates.
 */
      toBreak = false;
      for(;  !toBreak ; i = i + 1){
        if( !(e[s + i].y0 < e[s].y0) ){
          toBreak = true;
        }
      }

      toBreak = false;
      for(;  !toBreak ; j = j + 1){
        if( !(e[s].y0 < e[s + j].y0) ){
          toBreak = true;
        }
      }

      /* Make sure we haven't crossed.
 */
      if(i >= j){
        toBreak2 = true;
      }else{
        t = e[s + i];
        e[s + i] = e[s + j];
        e[s + j] = t;

        i = i + 1;
        j = j - 1;
      }
    }

    /* Recurse on smaller side, iterate on larger.
 */
    if(j < (n - i)){
      SortEdgesQuickSort(e, s, j);
      s = s + i;
      n = n - i;
    }else{
      SortEdgesQuickSort(e, s + i, n - i);
      n = j;
    }
  }
}
function SortEdgesInsSort(e, n){
  var i, j;
  var t, a, b;
  var toBreak;

  for(i = 1; i < n; i = i + 1){
    t = e[i];
    a = t;

    j = i;

    toBreak = false;
    for(; j > 0 &&  !toBreak ; ){
      b = e[j - 1];

      if( !(a.y0 < b.y0) ){
        toBreak = true;
      }else{
        e[j] = e[j - 1];
        j = j - 1;
      }
    }

    if(i != j){
      e[j] = t;
    }
  }
}
function SortEdges(e, n){
  SortEdgesInsSort(e, n);
}
function NewActiveEdge(e, xoff, startPoint){
  var z;
  var dxdy;

  z = {};

  dxdy = (e.x1 - e.x0)/(e.y1 - e.y0);

  z.fdx = dxdy;
  if(dxdy != 0){
    z.fdy = 1/dxdy;
  }else{
    z.fdy = 0;
  }

  z.fx = e.x0 + dxdy*(startPoint - e.y0);
  z.fx = z.fx - xoff;

  if(e.invert){
    z.direction = 1;
  }else{
    z.direction =  -1;
  }

  z.sy = e.y0;
  z.ey = e.y1;

  return z;
}
function HandleClippedEdge(scanline, s, x, e, x0, y0, x1, y1){
  var f;

  if(y0 != y1){
    /* assert (y0 < y1);
 */
    /* assert (e.sy <= e.ey);
 */
    if(y0 <= e.ey){
      if(y1 >= e.sy){
        if(y0 < e.sy){
          x0 = x0 + (x1 - x0)*(e.sy - y0)/(y1 - y0);
          y0 = e.sy;
        }
        if(y1 > e.ey){
          x1 = x1 + (x1 - x0)*(e.ey - y1)/(y1 - y0);
          y1 = e.ey;
        }

        if(x0 == x){
        }else if(x0 == x + 1){
        }else if(x0 <= x){
        }else if(x0 >= x + 1){
        }else{
        }

        if(x0 <= x && x1 <= x){
          f = e.direction*(y1 - y0);
          scanline[s + x] = scanline[s + x] + f;
        }else if(x0 >= x + 1 && x1 >= x + 1){
        }else{
          /* coverage = 1 - average x position
 */
          f = e.direction*(y1 - y0)*(1 - ((x0 - x) + (x1 - x))/2);
          scanline[s + x] = scanline[s + x] + f;
        }
      }
    }
  }
}
function FillActiveEdges(scanline, scanlineFill, active, yTop){
  var e;
  var l;
  var x, x0, x1, x2, x3, dx, xb, xTop, xBottom;
  var y0, y1, y2, y3, sy0, sy1, dy, yBottom, height;
  var y_crossing, step, sign, area;
  var t, f;

  yBottom = yTop + 1;

  l = active.first;

  for(;  !l.end ; ){
    e = l.value;

    /* Compute intersection points with top & bottom
 */
    /* assert (e.ey >= y_top);
 */
    if(e.fdx == 0){
      x0 = e.fx;
      if(x0 < scanline.length){
        if(x0 >= 0){
          HandleClippedEdge(scanline, 0, Math.floor(x0), e, x0, yTop, x0, yBottom);
          HandleClippedEdge(scanlineFill, 0, Math.floor(x0 + 1), e, x0, yTop, x0, yBottom);
        }else{
          HandleClippedEdge(scanlineFill, 0, 0, e, x0, yTop, x0, yBottom);
        }
      }
    }else{
      x0 = e.fx;
      dx = e.fdx;
      xb = x0 + dx;
      dy = e.fdy;

      /* assert (e.sy <= y_bottom && e.ey >= y_top);
 */
      /* Compute end points of line segment clipped to this scanline (if the
 */
      /* line segment starts on this scanline). x0 is the intersection of the
 */
      /* line with y_top, but that may be off the line segment.
 */
      if(e.sy > yTop){
        xTop = x0 + dx*(e.sy - yTop);
        sy0 = e.sy;
      }else{
        xTop = x0;
        sy0 = yTop;
      }
      if(e.ey < yBottom){
        xBottom = x0 + dx*(e.ey - yTop);
        sy1 = e.ey;
      }else{
        xBottom = xb;
        sy1 = yBottom;
      }

      if(xTop >= 0 && xBottom >= 0 && xTop < scanline.length && xBottom < scanline.length){
        /* From here on, we don't have to range check x values
 */
        if(Math.floor(xTop) == Math.floor(xBottom)){
          /* Simple case, only spans one pixel
 */
          x = Math.floor(xTop);
          height = sy1 - sy0;

          /* assert (x >= 0 && x < scanline.length);
 */
          f = e.direction*(1 - ((xTop - x) + (xBottom - x))/2)*height;
          scanline[x] = scanline[x] + f;
          /* Everything right of this pixel is filled
 */
          scanlineFill[x + 1] = scanlineFill[x + 1] + e.direction*height;
        }else{
          /* Covers 2+ pixels
 */
          if(xTop > xBottom){
            /* Flip scanline vertically; signed area is the same
 */
            sy0 = yBottom - (sy0 - yTop);
            sy1 = yBottom - (sy1 - yTop);
            t = sy0;
            sy0 = sy1;
            sy1 = t;
            t = xBottom;
            xBottom = xTop;
            xTop = t;
            dx =  -dx;
            dy =  -dy;
            t = x0;
            x0 = xb;
            xb = t;
          }

          x1 = Math.floor(xTop);
          x2 = Math.floor(xBottom);

          /* Compute intersection with y axis at x1+1
 */
          y_crossing = (x1 + 1 - x0)*dy + yTop;

          sign = e.direction;

          /* Area of the rectangle covered from y0..y_crossing
 */
          area = sign*(y_crossing - sy0);

          /* Area of the triangle (x_top,y0), (x+1,y0), (x+1,y_crossing)
 */
          f = area*(1 - ((xTop - x1) + (x1 + 1 - x1))/2);
          scanline[x1] = scanline[x1] + f;

          step = sign*dy;
          for(x = x1 + 1; x < x2; x = x + 1){
            scanline[x] = scanline[x] + area + step/2;
            area = area + step;
          }
          y_crossing = y_crossing + dy*(x2 - (x1 + 1));

          /* assert (abs(area) <= 1.01);
 */
          f = area + sign*(1 - ((x2 - x2) + (xBottom - x2))/2)*(sy1 - y_crossing);
          scanline[x2] = scanline[x2] + f;

          scanlineFill[x2 + 1] = scanlineFill[x2 + 1] + sign*(sy1 - sy0);
        }
      }else{
        /* If edge goes outside of box we're drawing, we require
 */
        /* clipping logic. Since this does not match the intended use
 */
        /* of this library, we use a different, very slow brute
 */
        /* force implementation.
 */
        for(x = 0; x < scanline.length; x = x + 1){
          /* Cases:
 */
          /* There can be up to two intersections with the pixel. Any intersection
 */
          /* with left or right edges can be handled by splitting into two (or three)
 */
          /* regions. Intersections with top & bottom do not necessitate case-wise logic.
 */
          /* The old way of doing this found the intersections with the left & right
 */
          /* edges, then used some simple logic to produce up to three segments in sorted
 */
          /* order from top-to-bottom. However, this had a problem: if an x edge was
 */
          /* epsilon across the x border, then the corresponding y position might not be
 */
          /* distinct from the other y segment, and it might ignored as an empty segment.
 */
          /* to avoid that, we need to explicitly produce segments based on x positions.
 */
          y0 = yTop;
          x1 = x;
          x2 = x + 1;
          x3 = xb;
          y3 = yBottom;

          /* x = e.x + e.dx * (y-y_top)
 */
          /* (y-y_top) = (x - e.x) / e.dx
 */
          /* y = (x - e.x) / e.dx + y_top
 */
          y1 = (x - x0)/dx + yTop;
          y2 = (x + 1 - x0)/dx + yTop;

          if(x0 < x1 && x3 > x2){
            /* Three segments descending down-right
 */
            HandleClippedEdge(scanline, 0, x, e, x0, y0, x1, y1);
            HandleClippedEdge(scanline, 0, x, e, x1, y1, x2, y2);
            HandleClippedEdge(scanline, 0, x, e, x2, y2, x3, y3);
          }else if(x3 < x1 && x0 > x2){
            /* Three segments descending down-left
 */
            HandleClippedEdge(scanline, 0, x, e, x0, y0, x2, y2);
            HandleClippedEdge(scanline, 0, x, e, x2, y2, x1, y1);
            HandleClippedEdge(scanline, 0, x, e, x1, y1, x3, y3);
          }else if(x0 < x1 && x3 > x1){
            /* Two segments across x, down-right
 */
            HandleClippedEdge(scanline, 0, x, e, x0, y0, x1, y1);
            HandleClippedEdge(scanline, 0, x, e, x1, y1, x3, y3);
          }else if(x3 < x1 && x0 > x1){
            /* Two segments across x, down-left
 */
            HandleClippedEdge(scanline, 0, x, e, x0, y0, x1, y1);
            HandleClippedEdge(scanline, 0, x, e, x1, y1, x3, y3);
          }else if(x0 < x2 && x3 > x2){
            /* Two segments across x+1, down-right
 */
            HandleClippedEdge(scanline, 0, x, e, x0, y0, x2, y2);
            HandleClippedEdge(scanline, 0, x, e, x2, y2, x3, y3);
          }else if(x3 < x2 && x0 > x2){
            /* Two segments across x+1, down-left
 */
            HandleClippedEdge(scanline, 0, x, e, x0, y0, x2, y2);
            HandleClippedEdge(scanline, 0, x, e, x2, y2, x3, y3);
          }else{
            /* One segment
 */
            HandleClippedEdge(scanline, 0, x, e, x0, y0, x3, y3);
          }
        }
      }
    }
    l = l.next;
  }
}
function RasterizeSortedEdges(out, edges, n, xoff, yoff){
  var active;
  var step;
  var z;
  var y, i, j, l, e, k, sum;
  var yScanTop, yScanBottom;
  var scanline, scanlineFill;

  /* boolean toBreak, changed, wasActive;
 */
  active = CreateLinkedListActiveEdges();

  j = 0;
  e = 0;

  scanline = [];
  scanline.length = out.w;
  scanlineFill = [];
  scanlineFill.length = out.w + 1;

  y = yoff;
  edges[n].y0 = (yoff + out.h) + 1;

  for(; j < out.h; ){
    /* Clear scanline
 */
    for(i = 0; i < scanline.length; i = i + 1){
      scanline[i] = 0;
    }
    for(i = 0; i < scanlineFill.length; i = i + 1){
      scanlineFill[i] = 0;
    }

    /* Find center of pixel for this scanline
 */
    yScanTop = y + 0;
    yScanBottom = y + 1;

    /* Update all active edges;
 */
    /* Remove all active edges that terminate before the center of this scanline
 */
    l = 0;
    step = active.first;
    for(;  !step.end ; ){
      if(step.value.ey <= yScanTop){
        /* Delete from list
 */
        step = step.next;
        LinkedListRemoveActiveEdge(active, l);
      }else{
        step = step.next;
        l = l + 1;
      }
    }

    /* Insert all edges that start before the bottom of this scanline
 */
    for(; edges[e].y0 <= yScanBottom; ){
      /* Skip horizontal
 */
      if(edges[e].y0 != edges[e].y1){
        z = NewActiveEdge(edges[e], xoff, yScanTop);

        if(j == 0 && yoff != 0){
          if(z.ey < yScanTop){
            /* This can happen due to subpixel positioning and some kind of
 */
            /* fp rounding error, so the author thinks.
 */
            z.ey = yScanTop;
          }
        }

        /* If we get really unlucky a tiny bit
 */
        /* of an edge can be out of bounds
 */
        /* assert (z.ey >= yScanTop);
 */
        /* Insert at front
 */
        LinkedListInsertActiveEdge(active, 0, z);
      }
      e = e + 1;
    }

    if(active.first != active.last){
      FillActiveEdges(scanline, scanlineFill, active, yScanTop);
    }

    /* Copy scanline to bitmap
 */
    sum = 0;
    for(i = 0; i < out.w; i = i + 1){
      sum = sum + scanlineFill[i];
      k = scanline[i] + sum;

      k = Math.abs(k)*255 + 0.5;
      if(k > 255){
        k = 255;
      }

      out.pixels[j*out.w + i] = k/255;
    }

    /* Advance all the edges
 */
    step = active.first;
    for(;  !step.end ; ){
      step.value.fx = step.value.fx + step.value.fdx;
      step = step.next;
    }

    y = y + 1;
    j = j + 1;
  }

  FreeLinkedListActiveEdges(active);
  delete(scanline);
}
function RasterizeGlyph(out, points, windingLengths, numWindings, scale, xoff, yoff, invert){
  var xScale, yScale;
  var a, b, n, m, i, j, k, s;
  var toInvertEdge;
  var edges;

  xScale = scale;
  if(invert){
    yScale =  -scale;
  }else{
    yScale = scale;
  }

  /* Now we have to blow out the windings into explicit edge lists
 */
  n = 0;
  for(i = 0; i < numWindings; i = i + 1){
    n = n + windingLengths[i];
  }

  edges = [];
  edges.length = n + 1;
  /* Add an extra one as a sentinel
 */
  for(i = 0; i < edges.length; i = i + 1){
    edges[i] = {};
  }
  n = 0;

  m = 0;
  for(i = 0; i < numWindings; i = i + 1){
    s = m;

    m = m + windingLengths[i];
    j = windingLengths[i] - 1;

    for(k = 0; k < windingLengths[i]; ){
      a = k;
      b = j;

      if(points[s + j].y == points[s + k].y){
      }else{
        /* Add edge from j to k to the list
 */
        toInvertEdge = false;

        if(invert){
          if(points[s + j].y > points[s + k].y){
            toInvertEdge = true;
          }
        }else if(points[s + j].y < points[s + k].y){
          toInvertEdge = true;
        }

        if(toInvertEdge){
          edges[n].invert = true;
          a = j;
          b = k;
        }

        edges[n].x0 = points[s + a].x*xScale;
        edges[n].y0 = points[s + a].y*yScale;
        edges[n].x1 = points[s + b].x*xScale;
        edges[n].y1 = points[s + b].y*yScale;
        n = n + 1;
      }

      j = k;
      k = k + 1;
    }
  }

  /* Now sort the edges by their highest point
 */
  /* (should snap to integer, and then by x)
 */
  SortEdges(edges, n);

  RasterizeSortedEdges(out, edges, n, xoff, yoff);

  for(i = 0; i < edges.length; i = i + 1){
    delete(edges[i]);
  }
  delete(edges);
}
function GenerateGlyphBitmap(f, glyphIndex, scale, flatnessInPixels, invert){
  var out;
  var vertices;
  var numContours;
  var contourLengths;
  var points;
  var glyphBounds;
  var i, xoff, yoff;

  vertices = GetGlyphShape(f, glyphIndex);

  numContours = {};
  contourLengths = {};

  points = FlattenCurves(vertices, flatnessInPixels/scale, numContours, contourLengths);

  out = {};

  if(points.length != 0){
    glyphBounds = GetGlyphBitmapBox(f, glyphIndex, scale);

    out.w = Math.abs(glyphBounds.x1 - glyphBounds.x0);
    out.h = Math.abs(glyphBounds.y1 - glyphBounds.y0);

    xoff = glyphBounds.x0;
    yoff = glyphBounds.y0;

    out.pixels = [];
    out.pixels.length = out.w*out.h;

    RasterizeGlyph(out, points, contourLengths.numberArray, numContours.numberValue, scale, xoff, yoff, invert);
  }

  delete(numContours);
  delete(contourLengths.numberArray);
  delete(contourLengths);
  for(i = 0; i < points.length; i = i + 1){
    delete(points[i]);
  }
  delete(points);

  return out;
}
function BlendGrayBitmapAndRGBABitmapWithColor(rgba, gray, x, y, color){
  var i, j, a, value, xStart, yStart, xEnd, yEnd;
  var c;

  if(rgba.x.length != 0){
    if(rgba.x[0].y.length != 0){
      if(gray.w != 0 && gray.h != 0){

        /* Do clipping to avoid out of bounds access
 */
        if(x < 0){
          xStart =  -x;
        }else{
          xStart = 0;
        }
        if(y < 0){
          yStart =  -y;
        }else{
          yStart = 0;
        }

        xEnd = Math.min(rgba.x.length - x, gray.w);
        yEnd = Math.min(rgba.x[0].y.length - y, gray.h);

        for(j = yStart; j < yEnd; j = j + 1){
          for(i = xStart; i < xEnd; i = i + 1){
            value = gray.pixels[j*gray.w + i];
            a = value*color.a;

            c = rgba.x[x + i].y[y + j];
            c.a = c.a + (1 - c.a)*a;
            c.r = color.r*(1 - c.a) + (color.r*a + c.r*(1 - a))*c.a;
            c.g = color.g*(1 - c.a) + (color.g*a + c.g*(1 - a))*c.a;
            c.b = color.b*(1 - c.a) + (color.b*a + c.b*(1 - a))*c.a;
          }
        }
      }
    }
  }
}
function CompareTagsEquals(tag, value){
  var result;
  var t;
  var str;
		
  str = StringToNumberArray(value);
		
  if(value.length != 4){
    result = false;
  }else{
    t = 0;
    t = t + Math.pow(2, 24)*str[0];
    t = t + Math.pow(2, 16)*str[1];
    t = t + Math.pow(2, 8)*str[2];
    t = t + str[3];
    result = t == tag;
  }
  return result;
}
function TruncToShort(x){
  var result;

  x = BitExtract(x, 0, 15);
  if(BitExtract(x, 15, 15) == 1){
    result =  -(Not2Byte(x) + 1);
  }else{
    result = x;
  }
  return result;
}
function TruncToShortUnsigned(x){
  x = BitExtract(x, 0, 15);
  return x;
}
function TruncToByteUnsigned(x){
  x = BitExtract(x, 0, 7);
  return x;
}
function FreeFont(f){
  var i;

  for(i = 0; i < f.tableDirectories.length; i = i + 1){
    delete(f.tableDirectories[i]);
  }
  delete(f.tableDirectories);

  for(i = 0; i < f.cmap.subtables.length; i = i + 1){
    delete(f.cmap.subtables[i]);
  }
  delete(f.cmap.subtables);
  delete(f.cmap);

  if(f.cffIndex != 0){
    delete(f.cff);
    delete(f.gsubrs);
    delete(f.subrs);
    delete(f.fontdicts);
    delete(f.fdselect);
    delete(f.charstrings);
  }
}
function GetGlyphIndex(f, codePoint){
  var result, i, format, segCountX2, entrySelector, rangeShift, endCount, search, searchRange, start, startGlyph, ngroups, end, mid, high, low, offset, item, first, count, bytes;
  var p;

  p = CreateNumberReference(0);
  p.numberValue = f.indexMap;

  format = Read2bytesBE(f.data, p);

  /* If not found
 */
  result = 0;

  /* Format 4 is used when you support the Unicode BMP but not all the
 */
  /* code points in the entire plane range (0x0000 to 0xFFFF) thus there are
 */
  /* holes in range.
 */
  /* The unicode standard is divided into 17 planes, each plane has a size
 */
  /* of 16 bits which means each plane contains 2^16 (65,536) code points.
 */
  /* The first plane (Plane 0) is called Basic Multilingual Plane (BMP).
 */
  /* It contains characters for almost all modern languages and most commonly
 */
  /* used symbols. Its range is 0x0000 -> 0xFFFF.
 */
  if(format == 4){
    p.numberValue = p.numberValue + 4;
    segCountX2 = Read2bytesBE(f.data, p);
    searchRange = Read2bytesBE(f.data, p);
    entrySelector = Read2bytesBE(f.data, p);
    rangeShift = Read2bytesBE(f.data, p);

    /* <= 0xffff
 */
    if(codePoint <= 65535){
      endCount = f.indexMap + 14;
      search = endCount;

      p.numberValue = search + rangeShift;

      /* They lie from endCount .. endCount + segCount
 */
      /* but searchRange is the nearest power of two, so...
 */
      if(codePoint >= Read2bytesBE(f.data, p)){
        search = search + rangeShift;
      }

      searchRange = Math.floor(searchRange/2);

      /* Now decrement to bias correctly to find smallest
 */
      search = search - 2;
      for(i = 0; i < entrySelector; i = i + 1){
        searchRange = Math.floor(searchRange/2);

        p.numberValue = search + searchRange*2;
        end = Read2bytesBE(f.data, p);

        if(codePoint > end){
          search = search + searchRange*2;
        }
      }
      search = search + 2;

      item = Math.floor((search - endCount)/2);

      p.numberValue = endCount + item*2;
      /* assert(codePoint < Read2bytesBE(f.data, p));
 */
      p.numberValue = endCount + segCountX2 + 2 + item*2;
      start = Read2bytesBE(f.data, p);

      if(codePoint < start){
        result = 0;
      }else{
        p.numberValue = endCount + segCountX2*3 + 2 + item*2;
        offset = Read2bytesBE(f.data, p);

        if(offset == 0){
          p.numberValue = endCount + segCountX2*2 + 2 + item*2;
          result = TruncToShortUnsigned(codePoint + Read2bytesBE(f.data, p));
        }else{
          p.numberValue = offset + (codePoint - start)*2 + endCount + segCountX2*3 + 2 + item*2;
          result = Read2bytesBE(f.data, p);
        }
      }
    }
  }else if(format == 0){
    /* Apple byte encoding
 */
    p.numberValue = f.indexMap + 2;
    bytes = Read2bytesBE(f.data, p);
    if(codePoint < bytes - 6){
      result = TruncToByteUnsigned(f.data[f.indexMap + 6] + codePoint);
    }
  }else if(format == 6){
    p.numberValue = f.indexMap + 6;
    first = Read2bytesBE(f.data, p);
    count = Read2bytesBE(f.data, p);

    if(codePoint >= first && codePoint < first + count){
      p.numberValue = f.indexMap + 10 + (codePoint - first)*2;
      result = Read2bytesBE(f.data, p);
    }
  }else if(format == 12 || format == 13){
    p.numberValue = f.indexMap + 12;
    ngroups = Read4bytesBE(f.data, p);
    low = 0;
    high = ngroups;

    /* Binary search the right group.
 */
    for(; low < high; ){
      /* Rounds down, so low <= mid < high
 */
      mid = low + (Math.floor((high - low)/2));

      p.numberValue = f.indexMap + 16 + mid*12;
      start = Read4bytesBE(f.data, p);
      end = Read4bytesBE(f.data, p);

      if(codePoint < start){
        high = mid;
      }else if(codePoint > end){
        low = mid + 1;
      }else{
        startGlyph = Read4bytesBE(f.data, p);
        if(format == 12){
          result = startGlyph + codePoint - start;
        }else{
          /* format == 13
 */
          result = startGlyph;
        }
      }
    }
  }
  delete(p);

  return result;
}
function GetScaleFactorForPixelHeight(f, pixels){
  var m;
  m = GetFontVMetrics(f);
  return pixels/(m.ascent - m.descent);
}
function GetFontVMetrics(f){
  var result;
  var p;

  result = {};

  p = CreateNumberReference(f.hhea + 4);
  result.ascent = TruncToShort(Read2bytesBE(f.data, p));
  result.descent = TruncToShort(Read2bytesBE(f.data, p));
  result.lineGap = TruncToShort(Read2bytesBE(f.data, p));

  delete(p);

  return result;
}
function GetFontBoundingBox(f){
  var result;
  var p;

  result = {};

  p = CreateNumberReference(f.hhea + 36);
  result.x0 = TruncToShort(Read2bytesBE(f.data, p));
  result.y0 = TruncToShort(Read2bytesBE(f.data, p));
  result.x1 = TruncToShort(Read2bytesBE(f.data, p));
  result.y1 = TruncToShort(Read2bytesBE(f.data, p));

  delete(p);

  return result;
}
function GetGlyphHMetrics(f, glyphIndex){
  var result;
  var p;
  var numOfLongHorMetrics;

  result = {};

  p = CreateNumberReference(f.hhea + 34);
  numOfLongHorMetrics = Read2bytesBE(f.data, p);

  if(glyphIndex < numOfLongHorMetrics){
    p = CreateNumberReference(f.hmtx + 4*glyphIndex);
    result.advanceWidth = TruncToShort(Read2bytesBE(f.data, p));
    result.leftSideBearing = TruncToShort(Read2bytesBE(f.data, p));
  }else{
    p = CreateNumberReference(f.hmtx + 4*(numOfLongHorMetrics - 1));
    result.advanceWidth = TruncToShort(Read2bytesBE(f.data, p));
    p.numberValue = f.hmtx + 4*numOfLongHorMetrics + 2*(glyphIndex - numOfLongHorMetrics);
    result.leftSideBearing = TruncToShort(Read2bytesBE(f.data, p));
  }

  delete(p);

  return result;
}
function GetGlyphKerningAdvance(f, glyphIndex1, glyphIndex2){
  var result;

  result = 0;
  if(f.kern != 0){
    result = GetGlyphKerningAdvanceKERN(f, glyphIndex1, glyphIndex2);
  }else if(f.gpos != 0){
    result = GetGlyphKerningAdvanceGPOS(f, glyphIndex1, glyphIndex2);
  }else{
  }

  return result;
}
function GetGlyphBox(f, glyphIndex){
  var result;
  var p;
  var c;
  var r;
  var g;

  result = {};

  if(f.cffIndex != 0){
    c = {};
    c.bounds = true;

    r = CffRunCharString(f, glyphIndex, c);
    if(r){
      result.x0 = c.xMin;
      result.y0 = c.yMin;
      result.x1 = c.xMax;
      result.y1 = c.yMax;
    }

    delete(c);
  }else{
    g = GetGlyfOffset(f, glyphIndex);
    if(g >= 0){
      p = CreateNumberReference(g + 2);
      result.x0 = TruncToShort(Read2bytesBE(f.data, p));
      result.y0 = TruncToShort(Read2bytesBE(f.data, p));
      result.x1 = TruncToShort(Read2bytesBE(f.data, p));
      result.y1 = TruncToShort(Read2bytesBE(f.data, p));

      delete(p);
    }
  }
  return result;
}
function ReadOffsetSubtable(f, p){
  f.offsetSubtable = {};
  f.offsetSubtable.scalerType = Read4bytesBE(f.data, p);
  f.offsetSubtable.numTables = Read2bytesBE(f.data, p);
  f.offsetSubtable.searchRange = Read2bytesBE(f.data, p);
  f.offsetSubtable.entrySelector = Read2bytesBE(f.data, p);
  f.offsetSubtable.rangeShift = Read2bytesBE(f.data, p);
}
function BufferPeekByte(b){
  var v;
  if(b.cursor < b.size){
    v = b.data[b.index + b.cursor];
  }else{
    v = 0;
  }
  return v;
}
function BufferGetByte(b){
  var v;
  if(b.cursor < b.size){
    v = b.data[b.index + b.cursor];
    b.cursor = b.cursor + 1;
  }else{
    v = 0;
  }
  return v;
}
function BufferGet(b, n){
  var v, i;
  v = 0;

  if(n >= 1 && n <= 4){
    for(i = 0; i < n; i = i + 1){
      v = v*Math.pow(2, 8) + BufferGetByte(b);
    }
  }
  return v;
}
function BufferSeek(b, c){
  if(c > b.size || c < 0){
    b.cursor = b.size;
  }else{
    b.cursor = c;
  }
}
function BufferSkip(b, c){
  BufferSeek(b, b.cursor + c);
}
function GetBufferRange(b, o, s){
  var result;

  result = {};
  result.data = b.data;
  result.index = b.index + o;
  result.size = s;
  result.cursor = 0;
  return result;
}
function CffGetIndex(b){
  var count, start, offsize;
  start = b.cursor;

  count = BufferGet(b, 2);
  if(count != 0){
    offsize = BufferGetByte(b);
    /* assert (offsize >= 1d && offsize <= 4d);
 */
    BufferSkip(b, offsize*count);
    BufferSkip(b, BufferGet(b, offsize) - 1);
  }
  return GetBufferRange(b, start, b.cursor - start);
}
function CffIndexGet(b, i){
  var count, offsize, start, end;

  BufferSeek(b, 0);
  count = BufferGet(b, 2);
  offsize = BufferGetByte(b);

  /* assert (i >= 0d && i < count);
 */
  /* assert (offsize >= 1d && offsize <= 4d);
 */
  BufferSkip(b, i*offsize);
  start = BufferGet(b, offsize);
  end = BufferGet(b, offsize);

  return GetBufferRange(b, 2 + (count + 1)*offsize + start, end - start);
}
function CffGetInt(b){
  var v, b0;
  b0 = BufferGetByte(b);
  if(b0 >= 32 && b0 <= 246){
    v = b0 - 139;
  }else if(b0 >= 247 && b0 <= 250){
    v = (b0 - 247)*256 + BufferGetByte(b) + 108;
  }else if(b0 >= 251 && b0 <= 254){
    v =  -(b0 - 251)*256 - BufferGetByte(b) - 108;
  }else if(b0 == 28){
    v = BufferGet(b, 2);
  }else if(b0 == 29){
    v = BufferGet(b, 4);
  }else{
    v = 0;
  }
  return v;
}
function CffSkipOperand(b){
  var v, b0;
  var toBreak;

  b0 = BufferPeekByte(b);
  toBreak = false;

  /* assert (b0 >= 28d);
 */
  if(b0 == 30){
    BufferSkip(b, 1);
    for(; b.cursor < b.size &&  !toBreak ; ){
      v = BufferGetByte(b);
      if(BitExtract(v, 0, 3) == 15 || Math.floor(v/Math.pow(2, 4)) == 15){
        toBreak = true;
      }
    }
  }else{
    CffGetInt(b);
  }
}
function CffDictGet(b, key){
  var start, end, op;
  var result;

  result = GetBufferRange(b, 0, 0);

  BufferSeek(b, 0);
  for(; b.cursor < b.size; ){
    start = b.cursor;
    for(; BufferPeekByte(b) >= 28; ){
      CffSkipOperand(b);
    }
    end = b.cursor;

    op = BufferGetByte(b);
    if(op == 12){
      op = BufferGetByte(b) + 256;
    }
    if(op == key){
      delete(result);
      result = GetBufferRange(b, start, end - start);
    }
  }

  return result;
}
function CffDictGetInts(b, key, out){
  var i;
  var operands;

  operands = CffDictGet(b, key);
  for(i = 0; i < out.length && operands.cursor < operands.size; i = i + 1){
    out[i] = CffGetInt(operands);
  }
  delete(operands);
}
function CffGetSubrs(b, fontDict){
  var subrsoff;
  var privateLoc;
  var pdict;
  var result;

  privateLoc = [];
  privateLoc.length = 2;
  CffDictGetInts(fontDict, 18, privateLoc);

  if(privateLoc[0] == 0 || privateLoc[0] == 0){
    result = {};
  }else{
    pdict = GetBufferRange(b, privateLoc[1], privateLoc[0]);

    subrsoff = [];
    subrsoff.length = 1;
    CffDictGetInts(pdict, 19, subrsoff);

    if(subrsoff[0] == 0){
      result = {};
    }else{
      BufferSeek(b, privateLoc[1] + subrsoff[0]);
      result = CffGetIndex(b);
    }
  }
  return result;
}
function InitFont(f, p, errorMessage){
  var rp;
  var td;
  var st;
  var cff, topdictidx, topdict;
  var i, j, maxp, cffSize;
  var charstrings, cstype, fdarrayoff, fdselectoff;
  var cmapFound, found, success;

  rp = CreateNumberReference(0);

  cffSize = 0;
  maxp = 0;

  success = true;
  cmapFound = false;

  f.tableDirectories = [];
  f.tableDirectories.length = f.offsetSubtable.numTables;
  for(i = 0; i < f.offsetSubtable.numTables; i = i + 1){
    td = {};

    td.tag = Read4bytesBE(f.data, p);
    td.checkSum = Read4bytesBE(f.data, p);
    /* @TODO: Verify checksum to check for file corruption!
 */
    td.offset = Read4bytesBE(f.data, p);
    td.lengthx = Read4bytesBE(f.data, p);

    f.tableDirectories[i] = td;

    if(CompareTagsEquals(td.tag, "cmap".split(''))){
      cmapFound = true;

      rp.numberValue = td.offset;
      f.cmap = ReadCmap(f, rp);

      /* Fix java crying about uninitialized variable,
 */
      /* although "st" will always be valid when used
 */
      st = f.cmap.subtables[0];

      found = false;
      for(j = 0; j < f.cmap.numberSubtables &&  !found ; j = j + 1){
        st = f.cmap.subtables[j];

        /* Unicode format.
 */
        /* Mac/iOS has these.
 */
        if(st.platformID == 0){
          found = true;
        }

        /* Microsoft (only unicode variants).
 */
        if(st.platformID == 3 && (st.platformSpecificID == 1 || st.platformSpecificID == 10)){
          found = true;
        }
      }

      if(found){
        f.indexMap = td.offset + st.offset;
      }
    }else if(CompareTagsEquals(td.tag, "glyf".split(''))){
      f.glyf = td.offset;
    }else if(CompareTagsEquals(td.tag, "loca".split(''))){
      f.loca = td.offset;
    }else if(CompareTagsEquals(td.tag, "head".split(''))){
      f.head = td.offset;
    }else if(CompareTagsEquals(td.tag, "hhea".split(''))){
      f.hhea = td.offset;
    }else if(CompareTagsEquals(td.tag, "hmtx".split(''))){
      f.hmtx = td.offset;
    }else if(CompareTagsEquals(td.tag, "kern".split(''))){
      f.kern = td.offset;
    }else if(CompareTagsEquals(td.tag, "GPOS".split(''))){
      f.gpos = td.offset;
    }else if(CompareTagsEquals(td.tag, "CFF ".split(''))){
      cffSize = td.lengthx;
      f.cffIndex = td.offset;
    }else if(CompareTagsEquals(td.tag, "maxp".split(''))){
      maxp = td.offset;
    }
  }

  if(f.glyf == 0){
    if(f.cffIndex == 0){
      success = false;
      errorMessage.string = "Invalid font file: file didn\'t contain neither a glyf (TrueType) nor CFF (OpenType) table.".split('');
    }
  }

  if(success && ( !cmapFound  || f.head == 0 || f.hhea == 0 || f.hmtx == 0)){
    success = false;
    errorMessage.string = "Invalid font file: one or more of these tables was missing - cmap, head, hhea, hmtx (if that helps you).".split('');
  }

  if(success && (f.indexMap == 0)){
    success = false;
    errorMessage.string = "Couldn\'t find encoding subtable in \"cmap\" which we support. We support only Unicode encoding formats, we don\'t support platform specific Apple/Microsoft tables.".split('');
  }

  if(success && (f.glyf != 0 && f.loca == 0)){
    success = false;
    errorMessage.string = "Invalid font file: glyf table was found, but loca table was not (if that helps you).".split('');
  }

  if(success){
    /* After reading the tables, continue with some
 */
    /* extra initialization steps.
 */
    rp.numberValue = f.head + 50;
    f.indexToLocFormat = Read2bytesBE(f.data, rp);

    if(maxp != 0){
      rp.numberValue = maxp + 4;
      f.numGlyphs = Read2bytesBE(f.data, rp);
    }else{
      f.numGlyphs = 0;
    }

    /* Initialization for CFF / Type2 fonts (OTF)
 */
    if(f.cffIndex != 0){
      cff = {};
      f.cff = cff;

      cff.data = f.data;
      cff.index = f.cffIndex;
      cff.cursor = 0;
      cff.size = cffSize;

      /* Read the header
 */
      BufferSkip(cff, 2);
      BufferSeek(cff, BufferGetByte(cff));
      /* hdrsize
 */
      /* "The name INDEX could list multiple fonts,
 */
      /* but we just use the first one." - Sean Barrett, stb_truetype.h
 */
      /* Actually by specification it's REQUIRED to only list
 */
      /* one font when the CFF is embeded in an OpenType font.
 */
      /* So it's all good.
 */
      CffGetIndex(cff);
      /* name INDEX
 */
      topdictidx = CffGetIndex(cff);
      topdict = CffIndexGet(topdictidx, 0);
      CffGetIndex(cff);
      /* string INDEX
 */
      f.gsubrs = CffGetIndex(cff);

      charstrings = [];
      charstrings.length = 1;
      cstype = [];
      cstype.length = 1;
      cstype[0] = 2;

      fdarrayoff = [];
      fdarrayoff.length = 1;
      fdselectoff = [];
      fdselectoff.length = 1;

      CffDictGetInts(topdict, 17, charstrings);
      CffDictGetInts(topdict, 262, cstype);
      CffDictGetInts(topdict, 292, fdarrayoff);
      CffDictGetInts(topdict, 293, fdselectoff);
      f.subrs = CffGetSubrs(cff, topdict);

      /* We only support Type 2 charstrings
 */
      if(cstype[0] != 2 || charstrings[0] == 0){
        success = false;
        errorMessage.string = "We only support Type 2 charstrings in OpenType fonts.".split('');
      }

      if(success){
        if(fdarrayoff[0] != 0){
          /* Looks like a CID font
 */
          if(fdselectoff[0] == 0){
            success = false;
            errorMessage.string = "Invalid OpenType font file.".split('');
          }else{
            BufferSeek(cff, fdarrayoff[0]);
            f.fontdicts = CffGetIndex(cff);
            f.fdselect = GetBufferRange(cff, fdselectoff[0], cff.size - fdselectoff[0]);
          }
        }else{
          /* Not CID... set dummy values.
 */
          f.fontdicts = {};
          f.fdselect = {};
        }

        BufferSeek(cff, charstrings[0]);
        f.charstrings = CffGetIndex(cff);
      }

      delete(topdictidx);
      delete(topdict);
      delete(charstrings);
      delete(cstype);
      delete(fdarrayoff);
      delete(fdselectoff);
    }
  }

  delete(rp);

  return success;
}
function TrackVertex(c, x, y){
  if((x > c.xMax) ||  !c.started ){
    c.xMax = x;
  }
  if((y > c.yMax) ||  !c.started ){
    c.yMax = y;
  }
  if((x < c.xMin) ||  !c.started ){
    c.xMin = x;
  }
  if((y < c.yMin) ||  !c.started ){
    c.yMin = y;
  }
  c.started = true;
}
function AddVertex(c, type, x, y, cx, cy, cx1, cy1){
  var CUBIC_TO;
  /* Constants
 */
  CUBIC_TO = 4;

  if(c.bounds){
    TrackVertex(c, x, y);
    if(type == CUBIC_TO){
      TrackVertex(c, cx, cy);
      TrackVertex(c, cx1, cy1);
    }
  }else{
    SetVertex(c.vertices[c.numVertices], type, x, y, cx, cy);
    c.vertices[c.numVertices].cx1 = cx1;
    c.vertices[c.numVertices].cy1 = cy1;
  }
  c.numVertices = c.numVertices + 1;
}
function CSContextCloseShape(c){
  var LINE_TO;
  /* Constants
 */
  LINE_TO = 2;

  if(c.xFirst != c.x || c.yFirst != c.y){
    AddVertex(c, LINE_TO, c.xFirst, c.yFirst, 0, 0, 0, 0);
  }
}
function CSContextRMoveTo(c, dx, dy){
  var NEW_CONTOUR;
  /* Constants
 */
  NEW_CONTOUR = 1;

  CSContextCloseShape(c);
  c.x = c.x + dx;
  c.xFirst = c.x;
  c.y = c.y + dy;
  c.yFirst = c.y;

  AddVertex(c, NEW_CONTOUR, c.x, c.y, 0, 0, 0, 0);
}
function CSContextRLineTo(c, dx, dy){
  var LINE_TO;
  /* Constants
 */
  LINE_TO = 2;

  c.x = c.x + dx;
  c.y = c.y + dy;
  AddVertex(c, LINE_TO, c.x, c.y, 0, 0, 0, 0);
}
function CSContextRCCurveTo(c, dx1, dy1, dx2, dy2, dx3, dy3){
  var cx1, cy1, cx2, cy2;
  var CUBIC_TO;
  /* Constants
 */
  CUBIC_TO = 4;

  cx1 = c.x + dx1;
  cy1 = c.y + dy1;
  cx2 = cx1 + dx2;
  cy2 = cy1 + dy2;
  c.x = cx2 + dx3;
  c.y = cy2 + dy3;
  AddVertex(c, CUBIC_TO, c.x, c.y, cx1, cy1, cx2, cy2);
}
function CidGetGlyphSubrs(f, glyphIndex){
  var result, fdselect;
  var nranges, start, end, v, fmt, fdselector, i;

  fdselect = f.fdselect;
  fdselector =  -1;

  BufferSeek(fdselect, 0);
  fmt = BufferGetByte(fdselect);
  if(fmt == 0){
    BufferSkip(fdselect, glyphIndex);
    fdselector = BufferGetByte(fdselect);
  }else if(fmt == 3){
    nranges = BufferGet(fdselect, 2);
    start = BufferGet(fdselect, 2);
    for(i = 0; i < nranges && fdselector ==  -1; i = i + 1){
      v = BufferGetByte(fdselect);
      end = BufferGet(fdselect, 2);
      if(glyphIndex >= start && glyphIndex < end){
        fdselector = v;
      }
      start = end;
    }
  }
  if(fdselector ==  -1){

    result = {};
  }else{
    result = CffGetSubrs(f.cff, CffIndexGet(f.fontdicts, fdselector));
  }
  return result;
}
function CffIndexCount(b){
  BufferSeek(b, 0);
  return BufferGet(b, 2);
}
function CffGetSubr(idx, n){
  var result;
  var count;
  var bias;

  count = CffIndexCount(idx);
  bias = 107;
  if(count >= 33900){
    bias = 32768;
  }else if(count >= 1240){
    bias = 1131;
  }
  n = n + bias;
  if(n < 0 || n >= count){
    result = {};
  }else{
    result = CffIndexGet(idx, n);
  }
  return result;
}
function CffRunCharString(f, glyphIndex, c){
  var maskbits, subrStackHeight, sp, v, i, b0, k, a, dx, dy, dx1, dx2, dx3, dx4, dx5, dx6, dy1, dy2, dy3, dy4, dy5, dy6, b1;
  var s;
  var success, error, toBreak, inHeader, hasSubrs, clearStack;
  var subrStack;
  var subrs, b;

  s = [];
  s.length = 48;

  inHeader = true;
  hasSubrs = false;
  maskbits = 0;
  subrStackHeight = 0;
  sp = 0;

  subrStack = [];
  subrStack.length = 10;
  subrs = f.subrs;

  success = false;
  error = false;

  /* This currently ignores the initial width value,
 */
  /* which isn't needed if we have hmtx.
 */
  b = CffIndexGet(f.charstrings, glyphIndex);
  for(; b.cursor < b.size &&  !error  &&  !success ; ){
    i = 0;
    clearStack = true;
    b0 = BufferGetByte(b);

    if(b0 == 0x13p0 || b0 == 0x14p0){
      /* hintmask, cntrmask
 */
      if(inHeader){
        maskbits = maskbits + Math.floor(sp/2);
      }

      inHeader = false;
      BufferSkip(b, Math.floor((maskbits + 7)/8));
    }else if(b0 == 0x01p0 || b0 == 0x03p0 || b0 == 0x12p0 || b0 == 0x17p0){
      /* hstem, vstem, hstemhm, vstemhm
 */
      maskbits = maskbits + Math.floor(sp/2);
    }else if(b0 == 0x15p0){
      /* rmoveto
 */
      inHeader = false;
      if(sp < 2){
        error = true;
      }else{
        CSContextRMoveTo(c, s[sp - 2], s[sp - 1]);
      }
    }else if(b0 == 0x04p0){
      /* vmoveto
 */
      inHeader = false;
      if(sp < 1){
        error = true;
      }else{
        CSContextRMoveTo(c, 0, s[sp - 1]);
      }
    }else if(b0 == 0x16p0){
      /* hmoveto
 */
      inHeader = false;
      if(sp < 1){
        error = true;
      }else{
        CSContextRMoveTo(c, s[sp - 1], 0);
      }
    }else if(b0 == 0x05p0){
      /* rlineto
 */
      if(sp < 2){
        error = true;
      }else{
        for(; i + 1 < sp; i = i + 2){
          CSContextRLineTo(c, s[i], s[i + 1]);
        }
      }
    }else if(b0 == 0x07p0){
      /* vlineto
 */
      if(sp < 1){
        error = true;
      }else{
        toBreak = false;
        for(;  !toBreak ; ){
          if(i >= sp){
            toBreak = true;
          }else{
            CSContextRLineTo(c, 0, s[i]);
            i = i + 1;

            if(i >= sp){
              toBreak = true;
            }else{
              CSContextRLineTo(c, s[i], 0);
              i = i + 1;
            }
          }
        }
      }
    }else if(b0 == 0x06p0){
      /* hlineto
 */
      if(sp < 1){
        error = true;
      }else{
        toBreak = false;
        for(;  !toBreak ; ){
          if(i >= sp){
            toBreak = true;
          }else{
            CSContextRLineTo(c, s[i], 0);
            i = i + 1;

            if(i >= sp){
              toBreak = true;
            }else{
              CSContextRLineTo(c, 0, s[i]);
              i = i + 1;
            }
          }
        }
      }
    }else if(b0 == 0x1Fp0){
      /* hvcurveto
 */
      if(sp < 4){
        error = true;
      }else{
        toBreak = false;
        for(;  !toBreak ; ){
          if(i + 3 >= sp){
            toBreak = true;
          }else{
if(sp - i == 5){
              a = s[i + 4];
            }else{
              a = 0;
            }

            CSContextRCCurveTo(c, s[i], 0, s[i + 1], s[i + 2], a, s[i + 3]);
            i = i + 4;

            if(i + 3 >= sp){
              toBreak = true;
            }else{
if(sp - i == 5){
                a = s[i + 4];
              }else{
                a = 0;
              }

              CSContextRCCurveTo(c, 0, s[i], s[i + 1], s[i + 2], s[i + 3], a);
              i = i + 4;
            }
          }
        }
      }
    }else if(b0 == 0x1Ep0){
      /* vhcurveto
 */
      if(sp < 4){
        error = true;
      }else{
        toBreak = false;
        for(;  !toBreak ; ){
          if(i + 3 >= sp){
            toBreak = true;
          }else{
if(sp - i == 5){
              a = s[i + 4];
            }else{
              a = 0;
            }

            CSContextRCCurveTo(c, 0, s[i], s[i + 1], s[i + 2], s[i + 3], a);
            i = i + 4;
            if(i + 3 >= sp){
              toBreak = true;
            }else{
if(sp - i == 5){
                a = s[i + 4];
              }else{
                a = 0;
              }

              CSContextRCCurveTo(c, s[i], 0, s[i + 1], s[i + 2], a, s[i + 3]);
              i = i + 4;
            }
          }
        }
      }
    }else if(b0 == 0x08p0){
      /* rrcurveto
 */
      if(sp < 6){
        error = true;
      }else{
        for(; i + 5 < sp; i = i + 6){
          CSContextRCCurveTo(c, s[i], s[i + 1], s[i + 2], s[i + 3], s[i + 4], s[i + 5]);
        }
      }
    }else if(b0 == 0x18p0){
      /* rcurveline
 */
      if(sp < 8){
        error = true;
      }else{
        for(; i + 5 < sp - 2; i = i + 6){
          CSContextRCCurveTo(c, s[i], s[i + 1], s[i + 2], s[i + 3], s[i + 4], s[i + 5]);
        }
        if(i + 1 >= sp){
          error = true;
        }else{
          CSContextRLineTo(c, s[i], s[i + 1]);
        }
      }
    }else if(b0 == 0x19p0){
      /* rlinecurve
 */
      if(sp < 8){
        error = true;
      }else{
        for(; i + 1 < sp - 6; i = i + 2){
          CSContextRLineTo(c, s[i], s[i + 1]);
        }
        if(i + 5 >= sp){
          error = true;
        }else{
          CSContextRCCurveTo(c, s[i], s[i + 1], s[i + 2], s[i + 3], s[i + 4], s[i + 5]);
        }
      }
    }else if(b0 == 0x1Ap0 || b0 == 0x1Bp0){
      /* vvcurveto, hhcurveto
 */
      if(sp < 4){
        error = true;
      }else{
        k = 0;
        if(sp%2 != 0){
          k = s[i];
          i = i + 1;
        }
        for(; i + 3 < sp; i = i + 4){
          if(b0 == 0x1Bp0){
            CSContextRCCurveTo(c, s[i], k, s[i + 1], s[i + 2], s[i + 3], 0);
          }else{
            CSContextRCCurveTo(c, k, s[i], s[i + 1], s[i + 2], 0, s[i + 3]);
          }
          k = 0;
        }
      }
    }else if(b0 == 0x0Ap0 || b0 == 0x1Dp0){
      if(b0 == 0x0Ap0){
        /* callsubr
 */
        if( !hasSubrs ){
          if(f.fdselect.size != 0){
            subrs = CidGetGlyphSubrs(f, glyphIndex);
          }
          hasSubrs = true;
        }
      }

      /* callgsubr
 */
      if(sp < 1){
        error = true;
      }else{
        sp = sp - 1;
        v = Math.floor(s[sp]);

        if(subrStackHeight >= 10){
          error = true;
        }else{
          subrStack[subrStackHeight] = b;
          subrStackHeight = subrStackHeight + 1;

          if(b0 == 0x0Ap0){
            b = CffGetSubr(subrs, v);
          }else{
            b = CffGetSubr(f.gsubrs, v);
          }

          if(b.size == 0){
            error = true;
          }else{
            b.cursor = 0;
            clearStack = false;
          }
        }
      }
    }else if(b0 == 0x0Bp0){
      /* return
 */
      if(subrStackHeight <= 0){
        error = true;
      }else{
        subrStackHeight = subrStackHeight - 1;
        b = subrStack[subrStackHeight];
        clearStack = false;
      }
    }else if(b0 == 0x0Ep0){
      /* endchar
 */
      CSContextCloseShape(c);
      success = true;
    }else if(b0 == 0x0Cp0){
      /* two-byte escape
 */
      b1 = BufferGetByte(b);

      /* @TODO These "flex" implementations ignore the flex-depth and resolution,
 */
      /* and always draw beziers.
 */
      if(b1 == 0x22p0){
        /* hflex
 */
        if(sp < 7){
          error = true;
        }else{
          dx1 = s[0];
          dx2 = s[1];
          dy2 = s[2];
          dx3 = s[3];
          dx4 = s[4];
          dx5 = s[5];
          dx6 = s[6];
          CSContextRCCurveTo(c, dx1, 0, dx2, dy2, dx3, 0);
          CSContextRCCurveTo(c, dx4, 0, dx5,  -dy2, dx6, 0);
        }
      }else if(b1 == 0x23p0){
        /* flex
 */
        if(sp < 13){
          error = true;
        }else{
          dx1 = s[0];
          dy1 = s[1];
          dx2 = s[2];
          dy2 = s[3];
          dx3 = s[4];
          dy3 = s[5];
          dx4 = s[6];
          dy4 = s[7];
          dx5 = s[8];
          dy5 = s[9];
          dx6 = s[10];
          dy6 = s[11];
          /* fd is s[12]
 */
          CSContextRCCurveTo(c, dx1, dy1, dx2, dy2, dx3, dy3);
          CSContextRCCurveTo(c, dx4, dy4, dx5, dy5, dx6, dy6);
        }
      }else if(b1 == 0x24p0){
        /* hflex1
 */
        if(sp < 9){
          error = true;
        }else{
          dx1 = s[0];
          dy1 = s[1];
          dx2 = s[2];
          dy2 = s[3];
          dx3 = s[4];
          dx4 = s[5];
          dx5 = s[6];
          dy5 = s[7];
          dx6 = s[8];
          CSContextRCCurveTo(c, dx1, dy1, dx2, dy2, dx3, 0);
          CSContextRCCurveTo(c, dx4, 0, dx5, dy5, dx6,  -(dy1 + dy2 + dy5));
        }
      }else if(b1 == 0x25p0){
        /* flex1
 */
        if(sp < 11){
          error = true;
        }else{
          dx1 = s[0];
          dy1 = s[1];
          dx2 = s[2];
          dy2 = s[3];
          dx3 = s[4];
          dy3 = s[5];
          dx4 = s[6];
          dy4 = s[7];
          dx5 = s[8];
          dy5 = s[9];
          dy6 = s[10];
          dx6 = dy6;

          dx = dx1 + dx2 + dx3 + dx4 + dx5;
          dy = dy1 + dy2 + dy3 + dy4 + dy5;

          if(Math.abs(dx) > Math.abs(dy)){
            dy6 =  -dy;
          }else{
            dx6 =  -dx;
          }
          CSContextRCCurveTo(c, dx1, dy1, dx2, dy2, dx3, dy3);
          CSContextRCCurveTo(c, dx4, dy4, dx5, dy5, dx6, dy6);
        }
      }else{
        error = true;
      }
    }else if(b0 != 255 && b0 != 28 && (b0 < 32 || b0 > 254)){
      error = true;
    }else{
      /* push immediate
 */
      if(b0 == 255){
        k = Math.floor(BufferGet(b, 4)/0x10000p0);
      }else{
        BufferSkip(b,  -1);
        k = TruncToShort(CffGetInt(b));
      }
      if(sp >= 48){
        error = true;
      }else{
        s[sp] = k;
        sp = sp + 1;

        clearStack = false;
      }
    }

    if(clearStack){
      sp = 0;
    }
  }

  return success;
}
function ReadCmap(f, p){
  var st;
  var result;
  var i;

  result = {};
  result.version = Read2bytesBE(f.data, p);
  result.numberSubtables = Read2bytesBE(f.data, p);

  result.subtables = [];
  result.subtables.length = result.numberSubtables;
  for(i = 0; i < result.numberSubtables; i = i + 1){
    st = {};
    st.platformID = Read2bytesBE(f.data, p);
    st.platformSpecificID = Read2bytesBE(f.data, p);
    st.offset = Read4bytesBE(f.data, p);

    result.subtables[i] = st;
  }

  return result;
}
function GetGlyfOffset(f, glyphIndex){
  var result, g1, g2;
  var p;

  p = CreateNumberReference(0);

  g2 = 0;
  if(glyphIndex >= f.numGlyphs){
    g1 =  -1;
  }else if(f.indexToLocFormat == 1){
    /* 32 bit
 */
    p.numberValue = f.loca + glyphIndex*4;
    g1 = Read4bytesBE(f.data, p);
    p.numberValue = f.loca + glyphIndex*4 + 4;
    g2 = Read4bytesBE(f.data, p);
  }else if(f.indexToLocFormat == 0){
    /* 16 bit
 */
    p.numberValue = f.loca + glyphIndex*2;
    g1 = Read2bytesBE(f.data, p)*2;
    p.numberValue = f.loca + glyphIndex*2 + 2;
    g2 = Read2bytesBE(f.data, p)*2;
  }else{
    /* Unknown f.indexToLocFormat
 */
    g1 =  -1;
  }

  if(g1 ==  -1 || g1 == g2){
    result =  -1;
  }else{
    result = f.glyf + g1;
  }

  delete(p);

  return result;
}
function SetVertex(v, type, x, y, cx, cy){
  v.type = type;
  v.x = x;
  v.y = y;
  v.cx = cx;
  v.cy = cy;
}
function CloseShape(vertices, numVertices, wasOff, startOff, sx, sy, scx, scy, cx, cy){
  var LINE_TO, CURVE_TO;
  /* Constants
 */
  LINE_TO = 2;
  CURVE_TO = 3;

  if(startOff == 1){
    if(wasOff == 1){
      SetVertex(vertices[numVertices], CURVE_TO, Math.floor((cx + scx)/2), Math.floor((cy + scy)/2), cx, cy);
      numVertices = numVertices + 1;
    }

    SetVertex(vertices[numVertices], CURVE_TO, sx, sy, scx, scy);
    numVertices = numVertices + 1;
  }else if(wasOff == 1){
    SetVertex(vertices[numVertices], CURVE_TO, sx, sy, cx, cy);
    numVertices = numVertices + 1;
  }else{
    SetVertex(vertices[numVertices], LINE_TO, sx, sy, 0, 0);
    numVertices = numVertices + 1;
  }
  return numVertices;
}
function GetGlyphShapeTT(f, glyphIndex){
  var mtx;
  var more, comp, gidx, wasOff, startOff, numVertices, sx, sy, cx, cy, scx, scy, x, dx, y, dy, flags, g, i, j, numberOfContours, endPtsOfContours, ins, points, m, n, nextMove, flagCount, off, d;
  var v;
  var vertices, compVerts, temp;
  var p;
  var NEW_CONTOUR, LINE_TO, CURVE_TO;
  /* Constants
 */
  NEW_CONTOUR = 1;
  LINE_TO = 2;
  CURVE_TO = 3;

  p = CreateNumberReference(0);

  numVertices = 0;

  if(glyphIndex == 0){
    /* No glyph
 */
    vertices = [];
    vertices.length = 0;
  }else{

    /* Start reading the data from the "glyf" table
 */
    g = GetGlyfOffset(f, glyphIndex);
    if(g ==  -1){
      /* Error when getting the index
 */
      vertices = [];
      vertices.length = 0;
    }else{
      /* Continue as normal..
 */
      p.numberValue = g;
      numberOfContours = Read2bytesBE(f.data, p);

      if(numberOfContours > 0){
        /* Handle simple shapes
 */
        flags = 0;
        wasOff = 0;
        startOff = 0;

        endPtsOfContours = (g + 10);
        p.numberValue = g + 10 + numberOfContours*2;
        ins = Read2bytesBE(f.data, p);
        points = g + 10 + numberOfContours*2 + 2 + ins;

        p.numberValue = endPtsOfContours + numberOfContours*2 - 2;
        n = 1 + Read2bytesBE(f.data, p);

        m = n + 2*numberOfContours;
        /* A loose bound on how many vertices we might need
 */
        vertices = [];
        vertices.length = m;
        for(i = 0; i < m; i = i + 1){
          vertices[i] = {};
        }

        nextMove = 0;
        flagCount = 0;

        /* In first pass, we load uninterpreted data into the allocated array
 */
        /* above, shifted to the end of the array so we won't overwrite it when
 */
        /* we create our final data starting from the front.
 */
        /* Starting offset for uninterpreted data,
 */
        /* regardless of how "m" ends up being calculated.
 */
        off = m - n;

        /* First load flags
 */
        for(i = 0; i < n; i = i + 1){
          if(flagCount == 0){
            flags = f.data[points];
            points = points + 1;

            if(BitExtract(flags, 3, 3) == 1){
              flagCount = f.data[points];
              points = points + 1;
            }
          }else{
            flagCount = flagCount - 1;
          }
          vertices[off + i].type = flags;
        }

        /* Load x coords
 */
        x = 0;
        for(i = 0; i < n; i = i + 1){
          flags = vertices[off + i].type;
          if(BitExtract(flags, 1, 1) == 1){
            dx = f.data[points];
            points = points + 1;

            if(BitExtract(flags, 4, 4) == 1){
              x = x + dx;
            }else{
              x = x - dx;
            }
          }else if(BitExtract(flags, 4, 4) != 1){
            x = x + TruncToShort(f.data[points]*256 + f.data[points + 1]);
            points = points + 2;
          }
          vertices[off + i].x = x;
        }

        /* Load y coords
 */
        y = 0;
        for(i = 0; i < n; i = i + 1){
          flags = vertices[off + i].type;
          if(BitExtract(flags, 2, 2) == 1){
            dy = f.data[points];
            points = points + 1;

            if(BitExtract(flags, 5, 5) == 1){
              y = y + dy;
            }else{
              y = y - dy;
            }
          }else if(BitExtract(flags, 5, 5) != 1){
            y = y + TruncToShort(f.data[points]*256 + f.data[points + 1]);
            points = points + 2;
          }
          vertices[off + i].y = y;
        }

        /* Now convert them to our format
 */
        sx = 0;
        sy = 0;
        cx = 0;
        cy = 0;
        scx = 0;
        scy = 0;

        j = 0;
        /* Contour counter
 */
        for(i = 0; i < n; i = i + 1){
          flags = vertices[off + i].type;
          x = vertices[off + i].x;
          y = vertices[off + i].y;

          if(nextMove == i){
            if(i != 0){
              numVertices = CloseShape(vertices, numVertices, wasOff, startOff, sx, sy, scx, scy, cx, cy);
            }

            /* Now start a new one
 */
            startOff = 1 - BitExtract(flags, 0, 0);
            if(startOff == 1){
              /* If we start off with an off-curve point, then when we need to find a point on
 */
              /* the curve
 */
              /* where we can start, and we need to save some state for when we wrap around.
 */
              scx = x;
              scy = y;
              if(BitExtract(vertices[off + i + 1].type, 0, 0) == 0){
                /* Next point is also a curve point, so interpolate an on-point curve
 */
                sx = Math.floor((x + vertices[off + i + 1].x)/2);
                sy = Math.floor((y + vertices[off + i + 1].y)/2);
              }else{
                /* Otherwise just use the next point as our start point
 */
                sx = vertices[off + i + 1].x;
                sy = vertices[off + i + 1].y;
                i = i + 1;
              }
            }else{
              sx = x;
              sy = y;
            }

            SetVertex(vertices[numVertices], NEW_CONTOUR, sx, sy, 0, 0);
            numVertices = numVertices + 1;

            wasOff = 0;

            p.numberValue = endPtsOfContours + j*2;
            nextMove = 1 + Read2bytesBE(f.data, p);

            j = j + 1;
          }else if(BitExtract(flags, 0, 0) == 0){
            /* if it's a curve
 */
            if(wasOff == 1){
              /* two off-curve control points in a row means interpolate an on-curve midpoint
 */
              SetVertex(vertices[numVertices], CURVE_TO, Math.floor((cx + x)/2), Math.floor((cy + y)/2), cx, cy);
              numVertices = numVertices + 1;
            }
            cx = x;
            cy = y;
            wasOff = 1;
          }else{
if(wasOff == 1){
              SetVertex(vertices[numVertices], CURVE_TO, x, y, cx, cy);
              numVertices = numVertices + 1;
            }else{
              SetVertex(vertices[numVertices], LINE_TO, x, y, 0, 0);
              numVertices = numVertices + 1;
            }
            wasOff = 0;
          }
        }
        numVertices = CloseShape(vertices, numVertices, wasOff, startOff, sx, sy, scx, scy, cx, cy);
      }else if(numberOfContours < 0){
        /* Handle compound shapes
 */
        more = 1;
        comp = g + 10;

        vertices = [];
        vertices.length = 0;

        for(; more == 1; ){

          mtx = [];
          mtx.length = 6;
          mtx[0] = 1;
          mtx[3] = 1;
          /* Everything else in mtx starts at 0 (2x2 identity matrix)
 */
          p.numberValue = comp;
          flags = Read2bytesBE(f.data, p);
          comp = comp + 2;

          p.numberValue = comp;
          gidx = Read2bytesBE(f.data, p);
          comp = comp + 2;

          if(BitExtract(flags, 1, 1) == 1){
            /* XY values
 */
            if(BitExtract(flags, 0, 0) == 1){
              /* shorts
 */
              p.numberValue = comp;
              mtx[4] = Read2bytesBE(f.data, p);
              comp = comp + 2;

              p.numberValue = comp;
              mtx[5] = Read2bytesBE(f.data, p);
              comp = comp + 2;
            }else{
              mtx[4] = f.data[comp];
              comp = comp + 1;
              mtx[5] = f.data[comp];
              comp = comp + 1;
            }
          }else{
          }

          if(BitExtract(flags, 3, 3) == 1){
            /* WE_HAVE_A_SCALE
 */
            p.numberValue = comp;
            d = Read2bytesBE(f.data, p)/16384.0;
            mtx[0] = d;
            mtx[3] = d;
            comp = comp + 2;
            mtx[1] = 0;
            mtx[2] = 0;
          }else if(BitExtract(flags, 6, 6) == 1){
            /* WE_HAVE_AN_X_AND_YSCALE
 */
            p.numberValue = comp;
            mtx[0] = Read2bytesBE(f.data, p)/16384.0;
            comp = comp + 2;
            mtx[1] = 0;
            mtx[2] = 0;
            p.numberValue = comp;
            mtx[3] = Read2bytesBE(f.data, p)/16384.0;
            comp = comp + 2;
          }else if(BitExtract(flags, 7, 7) == 1){
            /* WE_HAVE_A_TWO_BY_TWO
 */
            p.numberValue = comp;
            mtx[0] = Read2bytesBE(f.data, p)/16384.0;
            comp = comp + 2;
            p.numberValue = comp;
            mtx[1] = Read2bytesBE(f.data, p)/16384.0;
            comp = comp + 2;
            p.numberValue = comp;
            mtx[2] = Read2bytesBE(f.data, p)/16384.0;
            comp = comp + 2;
            p.numberValue = comp;
            mtx[3] = Read2bytesBE(f.data, p)/16384.0;
            comp = comp + 2;
          }

          /* Find transformation scales
 */
          m = Math.sqrt(mtx[0]*mtx[0] + mtx[1]*mtx[1]);
          n = Math.sqrt(mtx[2]*mtx[2] + mtx[3]*mtx[3]);

          compVerts = GetGlyphShape(f, gidx);
          if(compVerts.length > 0){
            /* Transform vertices.
 */
            for(i = 0; i < compVerts.length; i = i + 1){
              v = compVerts[i];

              x = v.x;
              y = v.y;

              v.x = m*(mtx[0]*x + mtx[2]*y + mtx[4]);
              v.y = m*(mtx[1]*x + mtx[3]*y + mtx[5]);

              x = v.cx;
              y = v.cy;

              v.cx = m*(mtx[0]*x + mtx[2]*y + mtx[4]);
              v.cy = m*(mtx[1]*x + mtx[3]*y + mtx[5]);
            }

            /* Append vertices
 */
            temp = [];
            temp.length = numVertices + compVerts.length;

            if(numVertices > 0){
              for(i = 0; i < numVertices; i = i + 1){
                temp[i] = vertices[i];
              }
            }

            for(i = 0; i < compVerts.length; i = i + 1){
              temp[i + numVertices] = compVerts[i];
            }

            delete(vertices);

            vertices = temp;

            numVertices = numVertices + compVerts.length;
            delete(compVerts);
          }

          /* More components?
 */
          more = BitExtract(flags, 5, 5);
        }
      }else{
        /* numberOfContours == 0, do nothing
 */
        vertices = [];
        vertices.length = 0;
      }
    }
  }

  /* Truncate the array to just the right size,
 */
  /* without extra vertices, so we don't have to
 */
  /* bookkeep a second variable (numVertices)
 */
  /* which may be different from vertices.length.
 */
  temp = [];
  temp.length = numVertices;
  for(i = 0; i < numVertices; i = i + 1){
    temp[i] = vertices[i];
  }

  delete(vertices);
  delete(p);

  return temp;
}
function GetGlyphShape(f, glyphIndex){
  var i;
  var count, output;
  var vertices;

  if(f.cffIndex != 0){
    count = {};
    count.bounds = true;

    output = {};

    /* R uns the charstring twice, once to count
 */
    /* and once to output (to avoid reallocating).
 */
    if(CffRunCharString(f, glyphIndex, count)){
      vertices = [];
      vertices.length = count.numVertices;
      for(i = 0; i < vertices.length; i = i + 1){
        vertices[i] = {};
      }

      output.vertices = vertices;

      CffRunCharString(f, glyphIndex, output);
    }else{
      vertices = [];
      vertices.length = 0;
    }
    delete(count);
    delete(output);
  }else{
    vertices = GetGlyphShapeTT(f, glyphIndex);
  }
  return vertices;
}
function GetGlyphKerningAdvanceGPOS(f, g1, g2){
  return 0;
}
function GetGlyphKerningAdvanceKERN(f, g1, g2){
  var p;
  var result;
  var l, r, m;
  var needle, straw;
  var toBreak;
  toBreak = false;

  l = 0;

  result = 0;
  if(f.kern != 0){
    p = CreateNumberReference(0);

    /* Number of tables.. need at least 1
 */
    p.numberValue = f.kern + 2;
    if(Read2bytesBE(f.data, p) >= 1){
      /* Horizontal flag must be set in format
 */
      p.numberValue = f.kern + 8;
      if(Read2bytesBE(f.data, p) == 1){
        p.numberValue = f.kern + 10;
        r = Read2bytesBE(f.data, p) - 1;
        needle = g1*Math.pow(2, 16) + g2;
        for(; l <= r &&  !toBreak ; ){
          m = Math.floor((l + r)/2);

          p.numberValue = f.kern + 18 + (m*6);
          straw = Read4bytesBE(f.data, p);
          if(needle < straw){
            r = m - 1;
          }else if(needle > straw){
            l = m + 1;
          }else{
            p.numberValue = f.kern + 22 + (m*6);
            result = TruncToShort(Read2bytesBE(f.data, p));
            toBreak = true;
          }
        }
      }
    }

    delete(p);
  }

  return result;
}
function test(){
  var failures;
  var fontData;

  failures = CreateNumberReference(0);

  return failures.numberValue;
}
function StringToNumberArray(string){
  var i;
  var array;

  array = [];
  array.length = string.length;

  for(i = 0; i < string.length; i = i + 1){
    array[i] = string[i].charCodeAt(0);
  }
  return array;
}
function NumberArrayToString(array){
  var i;
  var string;

  string = [];
  string.length = array.length;

  for(i = 0; i < array.length; i = i + 1){
    string[i] = String.fromCharCode(array[i]);
  }
  return string;
}
function NumberArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function BooleanArraysEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function StringsEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.length == b.length){
    for(i = 0; i < a.length && equal; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function FillNumberArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillString(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillBooleanArray(a, value){
  var i;

  for(i = 0; i < a.length; i = i + 1){
    a[i] = value;
  }
}
function FillNumberArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillBooleanArrayRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function FillStringRange(a, value, from, to){
  var i, lengthx;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    for(i = 0; i < lengthx; i = i + 1){
      a[from + i] = value;
    }

    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyNumberArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyBooleanArray(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyString(a){
  var i;
  var n;

  n = [];
  n.length = a.length;

  for(i = 0; i < a.length; i = i + 1){
    n[i] = a[i];
  }

  return n;
}
function CopyNumberArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.numberArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyBooleanArrayRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.booleanArray = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function CopyStringRange(a, from, to, copyReference){
  var i, lengthx;
  var n;
  var success;

  if(from >= 0 && from <= a.length && to >= 0 && to <= a.length && from <= to){
    lengthx = to - from;
    n = [];
    n.length = lengthx;

    for(i = 0; i < lengthx; i = i + 1){
      n[i] = a[from + i];
    }

    copyReference.string = n;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function IsLastElement(lengthx, index){
  return index + 1 == lengthx;
}
function CreateNumberArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  FillNumberArray(array, value);

  return array;
}
function CreateBooleanArray(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  FillBooleanArray(array, value);

  return array;
}
function CreateString(lengthx, value){
  var array;

  array = [];
  array.length = lengthx;
  FillString(array, value);

  return array;
}
function SwapElementsOfNumberArray(A, ai, bi){
  var tmp;

  tmp = A[ai];
  A[ai] = A[bi];
  A[bi] = tmp;
}
function SwapElementsOfStringArray(A, ai, bi){
  var tmp;

  tmp = A.stringArray[ai];
  A.stringArray[ai] = A.stringArray[bi];
  A.stringArray[bi] = tmp;
}
function ReverseNumberArray(array){
  var i;

  for(i = 0; i < array.length/2; i = i + 1){
    SwapElementsOfNumberArray(array, i, array.length - i - 1);
  }
}
function CreateBooleanReference(value){
  var ref;

  ref = {};
  ref.booleanValue = value;

  return ref;
}
function CreateBooleanArrayReference(value){
  var ref;

  ref = {};
  ref.booleanArray = value;

  return ref;
}
function CreateBooleanArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.booleanArray = [];
  ref.booleanArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.booleanArray[i] = value;
  }

  return ref;
}
function FreeBooleanArrayReference(booleanArrayReference){
  delete(booleanArrayReference.booleanArray);
  delete(booleanArrayReference);
}
function CreateCharacterReference(value){
  var ref;

  ref = {};
  ref.characterValue = value;

  return ref;
}
function CreateNumberReference(value){
  var ref;

  ref = {};
  ref.numberValue = value;

  return ref;
}
function CreateNumberArrayReference(value){
  var ref;

  ref = {};
  ref.numberArray = value;

  return ref;
}
function CreateNumberArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.numberArray = [];
  ref.numberArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.numberArray[i] = value;
  }

  return ref;
}
function FreeNumberArrayReference(numberArrayReference){
  delete(numberArrayReference.numberArray);
  delete(numberArrayReference);
}
function CreateStringReference(value){
  var ref;

  ref = {};
  ref.string = value;

  return ref;
}
function CreateStringReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.string = [];
  ref.string.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.string[i] = value;
  }

  return ref;
}
function FreeStringReference(stringReference){
  delete(stringReference.string);
  delete(stringReference);
}
function CreateStringArrayReference(strings){
  var ref;

  ref = {};
  ref.stringArray = strings;

  return ref;
}
function CreateStringArrayReferenceLengthValue(lengthx, value){
  var ref;
  var i;

  ref = {};
  ref.stringArray = [];
  ref.stringArray.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    ref.stringArray[i] = CreateStringReference(value);
  }

  return ref;
}
function FreeStringArrayReference(stringArrayReference){
  var i;

  for(i = 0; i < stringArrayReference.stringArray.length; i = i + 1){
    delete(stringArrayReference.stringArray[i]);
  }
  delete(stringArrayReference.stringArray);
  delete(stringArrayReference);
}
function And4Byte(n1, n2){
  return AndBytes(n1, n2, 4);
}
function And2Byte(n1, n2){
  return AndBytes(n1, n2, 2);
}
function AndByte(n1, n2){
  return AndBytes(n1, n2, 1);
}
function AndBytes(n1, n2, bytes){
  var byteVal, result, i;

  byteVal = 1;
  result = 0;

  if(n1 >= 0 && n1 < Math.pow(2, bytes*8) && n2 >= 0 && n2 < Math.pow(2, bytes*8)){
    n1 = Truncate(n1);
    n2 = Truncate(n2);
    bytes = Truncate(bytes);

    for(i = 0; i < bytes*8; i = i + 1){
      if(n1%2 == 1 && n2%2 == 1){
        result = result + byteVal;
      }
      n1 = Math.floor(n1/2);
      n2 = Math.floor(n2/2);
      byteVal = byteVal*2;
    }
  }

  return result;
}
function Or4Byte(n1, n2){
  return OrBytes(n1, n2, 4);
}
function Or2Byte(n1, n2){
  return OrBytes(n1, n2, 2);
}
function OrByte(n1, n2){
  return OrBytes(n1, n2, 1);
}
function OrBytes(n1, n2, bytes){
  var byteVal, result, i;

  byteVal = 1;
  result = 0;

  if(n1 >= 0 && n1 < Math.pow(2, bytes*8) && n2 >= 0 && n2 < Math.pow(2, bytes*8)){
    n1 = Truncate(n1);
    n2 = Truncate(n2);
    bytes = Truncate(bytes);

    for(i = 0; i < bytes*8; i = i + 1){
      if(n1%2 == 1 || n2%2 == 1){
        result = result + byteVal;
      }
      n1 = Math.floor(n1/2);
      n2 = Math.floor(n2/2);
      byteVal = byteVal*2;
    }
  }

  return result;
}
function Xor4Byte(n1, n2){
  return XorBytes(n1, n2, 4);
}
function Xor2Byte(n1, n2){
  return XorBytes(n1, n2, 2);
}
function XorByte(n1, n2){
  return XorBytes(n1, n2, 1);
}
function XorBytes(n1, n2, bytes){
  var byteVal, result, i;

  byteVal = 1;
  result = 0;

  if(n1 >= 0 && n1 < Math.pow(2, bytes*8) && n2 >= 0 && n2 < Math.pow(2, bytes*8)){
    n1 = Truncate(n1);
    n2 = Truncate(n2);
    bytes = Truncate(bytes);

    for(i = 0; i < bytes*8; i = i + 1){
      if(n1%2 != n2%2){
        result = result + byteVal;
      }
      n1 = Math.floor(n1/2);
      n2 = Math.floor(n2/2);
      byteVal = byteVal*2;
    }
  }

  return result;
}
function Not4Byte(b){
  return NotBytes(b, 4);
}
function Not2Byte(b){
  return NotBytes(b, 2);
}
function NotByte(b){
  return NotBytes(b, 1);
}
function NotBytes(b, lengthx){
  var result;

  result = 0;

  if(b >= 0 && b < Math.pow(2, lengthx*8)){
    b = Truncate(b);
    lengthx = Truncate(lengthx);

    result = Math.pow(2, lengthx*8) - b - 1;
  }

  return result;
}
function ShiftLeft4Byte(b, amount){
  return ShiftLeftBytes(b, amount, 4);
}
function ShiftLeft2Byte(b, amount){
  return ShiftLeftBytes(b, amount, 2);
}
function ShiftLeftByte(b, amount){
  return ShiftLeftBytes(b, amount, 1);
}
function ShiftLeftBytes(b, amount, lengthx){
  var result;

  result = 0;

  if(b >= 0 && b < Math.pow(2, lengthx*8) && amount >= 0 && amount <= lengthx*8){
    b = Truncate(b);
    amount = Truncate(amount);

    result = b*Math.pow(2, amount);
  }

  return result;
}
function ShiftRight4Byte(b, amount){
  return ShiftRightBytes(b, amount, 4);
}
function ShiftRight2Byte(b, amount){
  return ShiftRightBytes(b, amount, 2);
}
function ShiftRightByte(b, amount){
  return ShiftRightBytes(b, amount, 1);
}
function ShiftRightBytes(b, amount, lengthx){
  var result;

  result = 0;

  if(b >= 0 && b < Math.pow(2, lengthx*8) && amount >= 0 && amount <= lengthx*8){
    b = Truncate(b);
    amount = Truncate(amount);

    result = Truncate(b/Math.pow(2, amount));
  }

  return result;
}
function RotateLeft4Byte(w, n){
  var a;
  /*return (w << n) | (w >> (32 - n)); */
  /* Mask the upper bits first, then rotate. */
  a = And4Byte(w, Not4Byte(ShiftLeft4Byte(1, n) - 1));
  return Or4Byte(ShiftLeft4Byte(a, n), ShiftRight4Byte(w, (32 - n)));
}
function RotateRight4Bytes(w, n){
  var a;
  /* return (w >> d) | (w << (32 - n)); */
  /* Mask away the upper bits first, then perform the shift. */
  a = And4Byte(w, ShiftLeft4Byte(1, n) - 1);
  return Or4Byte(ShiftRight4Byte(w, n), ShiftLeft4Byte(a, 32 - n));
}
function CreateBooleanArrayFromNumber(w, size){
  var out;
  var p, j;

  out = CreateBooleanArray(size, false);

  j = 0;
  p = 1;
  for(; p < w; ){
    p = p*2;
    j = j + 1;
  }

  for(; j >= 0; j = j - 1){
    if(w >= p){
      w = w - p;
      if(j < size){
        out[size - 1 - j] = true;
      }
    }
    p = p/2;
  }

  return out;
}
function BooleanArrayToNumber(bits){
  var w, i, p;

  w = 0;
  p = 1;
  for(i = 31; i >= 0; i = i - 1){
    if(bits[i]){
      w = w + p;
    }
    p = p*2;
  }

  return w;
}
function BooleanAnd(a, b){
  var out;
  var i, lengthx;

  lengthx = a.length;

  out = [];
  out.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    out[i] = a[i] && b[i];
  }
  return out;
}
function BooleanXor(a, b){
  var out;
  var i, lengthx;

  lengthx = a.length;

  out = [];
  out.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    if(a[i] || b[i]){
      if( !(a[i] && b[i]) ){
        out[i] = true;
      }
    }
  }
  return out;
}
function BooleanNot(a){
  var out;
  var i, lengthx;

  lengthx = a.length;

  out = [];
  out.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    out[i] =  !a[i] ;
  }
  return out;
}
function ShiftBitsRight4Byte(w, n){
  var wb;
  var ob;
  var i, it;
  var f;
  f = false;

  if(n == 0){
    ob = w;
  }else{
    wb = w;
    ob = [];
    ob.length = 32;

    for(i = 0; i < 32; i = i + 1){
      it = i - n;

      if(it < 0){
        f = false;
      }else{
        f = wb[it];
      }

      ob[i] = f;
    }
  }

  return ob;
}
function ReadNextBit(data, nextbit){
  var bytenr, bitnumber, bit, b;

  bytenr = Math.floor(nextbit.numberValue/8);
  bitnumber = nextbit.numberValue%8;

  b = data[bytenr];

  bit = Math.floor(b/Math.pow(2, bitnumber))%2;

  nextbit.numberValue = nextbit.numberValue + 1;

  return bit;
}
function BitExtract(b, fromInc, toInc){
  return Math.floor(b/Math.pow(2, fromInc))%Math.pow(2, toInc + 1 - fromInc);
}
function ReadBitRange(data, nextbit, lengthx){
  var startbyte, endbyte;
  var startbit, endbit;
  var number, i;

  number = 0;

  startbyte = Math.floor(nextbit.numberValue/8);
  endbyte = Math.floor((nextbit.numberValue + lengthx)/8);

  startbit = nextbit.numberValue%8;
  endbit = (nextbit.numberValue + lengthx - 1)%8;

  if(startbyte == endbyte){
    number = BitExtract(data[startbyte], startbit, endbit);
  }

  nextbit.numberValue = nextbit.numberValue + lengthx;

  return number;
}
function SkipToBoundary(nextbit){
  var skip;

  skip = 8 - nextbit.numberValue%8;
  nextbit.numberValue = nextbit.numberValue + skip;
}
function ReadNextByteBoundary(data, nextbit){
  var bytenr, b;

  bytenr = Math.floor(nextbit.numberValue/8);
  b = data[bytenr];
  nextbit.numberValue = nextbit.numberValue + 8;

  return b;
}
function Read2bytesByteBoundary(data, nextbit){
  var r;

  r = 0;
  r = r + Math.pow(2, 8)*ReadNextByteBoundary(data, nextbit);
  r = r + ReadNextByteBoundary(data, nextbit);

  return r;
}
function ReadXbytes(data, position, lengthx){
  var r;
  var i;

  r = [];
  r.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    r[i] = ReadByte(data, position);
  }

  return r;
}
function Read4bytesBE(data, position){
  var r;

  r = 0;
  r = r + Math.pow(2, 24)*ReadByte(data, position);
  r = r + Math.pow(2, 16)*ReadByte(data, position);
  r = r + Math.pow(2, 8)*ReadByte(data, position);
  r = r + ReadByte(data, position);

  return r;
}
function Read2bytesBE(data, position){
  var r;

  r = 0;
  r = r + Math.pow(2, 8)*ReadByte(data, position);
  r = r + ReadByte(data, position);

  return r;
}
function ReadByte(data, position){
  var next;

  next = data[position.numberValue];
  position.numberValue = position.numberValue + 1;

  return next;
}
function Read4bytesLE(data, position){
  var r;

  r = 0;
  r = r + ReadByte(data, position);
  r = r + Math.pow(2, 8)*ReadByte(data, position);
  r = r + Math.pow(2, 16)*ReadByte(data, position);
  r = r + Math.pow(2, 24)*ReadByte(data, position);

  return r;
}
function WriteByte(data, b, position){
  data[position.numberValue] = b;
  position.numberValue = position.numberValue + 1;
}
function Write2BytesLE(data, b, position){
  data[position.numberValue] = Round(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
  data[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
}
function Write4BytesLE(data, b, position){
  data[position.numberValue] = Round(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
  data[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data[position.numberValue] = Math.floor(b/Math.pow(2, 16))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data[position.numberValue] = Math.floor(b/Math.pow(2, 24))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
}
function Write2BytesBE(data, b, position){
  data[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data[position.numberValue] = Round(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
}
function Write4BytesBE(data, b, position){
  data[position.numberValue] = Math.floor(b/Math.pow(2, 24))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data[position.numberValue] = Math.floor(b/Math.pow(2, 16))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data[position.numberValue] = Math.floor(b/Math.pow(2, 8))%Math.pow(2, 8);
  position.numberValue = position.numberValue + 1;
  data[position.numberValue] = Round(b%Math.pow(2, 8));
  position.numberValue = position.numberValue + 1;
}
function WriteStringBytes(data, cs, position){
  var i, v;

  for(i = 0; i < cs.length; i = i + 1){
    v = cs[i].charCodeAt(0);
    WriteByte(data, v, position);
  }
}
function AssertFalse(b, failures){
  if(b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertTrue(b, failures){
  if( !b ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertEquals(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleansEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertCharactersEqual(a, b, failures){
  if(a != b){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringEquals(a, b, failures){
  if( !StringsEqual(a, b) ){
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertNumberArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertEquals(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertBooleanArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertBooleansEqual(a[i], b[i], failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function AssertStringArraysEqual(a, b, failures){
  var i;

  if(a.length == b.length){
    for(i = 0; i < a.length; i = i + 1){
      AssertStringEquals(a[i].string, b[i].string, failures);
    }
  }else{
    failures.numberValue = failures.numberValue + 1;
  }
}
function GetBlack(){
  var black;
  black = {};
  black.a = 1;
  black.r = 0;
  black.g = 0;
  black.b = 0;
  return black;
}
function GetWhite(){
  var white;
  white = {};
  white.a = 1;
  white.r = 1;
  white.g = 1;
  white.b = 1;
  return white;
}
function GetTransparent(){
  var transparent;
  transparent = {};
  transparent.a = 0;
  transparent.r = 0;
  transparent.g = 0;
  transparent.b = 0;
  return transparent;
}
function GetGray(percentage){
  var black;
  black = {};
  black.a = 1;
  black.r = 1 - percentage;
  black.g = 1 - percentage;
  black.b = 1 - percentage;
  return black;
}
function CreateRGBColor(r, g, b){
  var color;
  color = {};
  color.a = 1;
  color.r = r;
  color.g = g;
  color.b = b;
  return color;
}
function CreateRGBAColor(r, g, b, a){
  var color;
  color = {};
  color.a = a;
  color.r = r;
  color.g = g;
  color.b = b;
  return color;
}
function CreateImage(w, h, color){
  var image;
  var i, j;

  image = {};
  image.x = [];
  image.x.length = w;
  for(i = 0; i < w; i = i + 1){
    image.x[i] = {};
    image.x[i].y = [];
    image.x[i].y.length = h;
    for(j = 0; j < h; j = j + 1){
      image.x[i].y[j] = {};
      SetPixel(image, i, j, color);
    }
  }

  return image;
}
function DeleteImage(image){
  var i, j, w, h;

  w = ImageWidth(image);
  h = ImageHeight(image);

  for(i = 0; i < w; i = i + 1){
    for(j = 0; j < h; j = j + 1){
      delete(image.x[i].y[j]);
    }
    delete(image.x[i]);
  }
  delete(image);
}
function ImageWidth(image){
  return image.x.length;
}
function ImageHeight(image){
  var height;

  if(ImageWidth(image) == 0){
    height = 0;
  }else{
    height = image.x[0].y.length;
  }

  return height;
}
function SetPixel(image, x, y, color){
  if(x >= 0 && x < ImageWidth(image) && y >= 0 && y < ImageHeight(image)){
    image.x[x].y[y].a = color.a;
    image.x[x].y[y].r = color.r;
    image.x[x].y[y].g = color.g;
    image.x[x].y[y].b = color.b;
  }
}
function DrawPixel(image, x, y, color){
  var ra, ga, ba, aa;
  var rb, gb, bb, ab;
  var ro, go, bo, ao;

  if(x >= 0 && x < ImageWidth(image) && y >= 0 && y < ImageHeight(image)){
    ra = color.r;
    ga = color.g;
    ba = color.b;
    aa = color.a;

    rb = image.x[x].y[y].r;
    gb = image.x[x].y[y].g;
    bb = image.x[x].y[y].b;
    ab = image.x[x].y[y].a;

    ao = CombineAlpha(aa, ab);

    ro = AlphaBlend(ra, aa, rb, ab, ao);
    go = AlphaBlend(ga, aa, gb, ab, ao);
    bo = AlphaBlend(ba, aa, bb, ab, ao);

    image.x[x].y[y].r = ro;
    image.x[x].y[y].g = go;
    image.x[x].y[y].b = bo;
    image.x[x].y[y].a = ao;
  }
}
function CombineAlpha(as, ad){
  return as + ad*(1 - as);
}
function AlphaBlend(cs, as, cd, ad, ao){
  return (cs*as + cd*ad*(1 - as))/ao;
}
function DrawHorizontalLine1px(image, x, y, lengthx, color){
  var i;

  for(i = 0; i < lengthx; i = i + 1){
    DrawPixel(image, x + i, y, color);
  }
}
function DrawVerticalLine1px(image, x, y, height, color){
  var i;

  for(i = 0; i < height; i = i + 1){
    DrawPixel(image, x, y + i, color);
  }
}
function DrawRectangle1px(image, x, y, width, height, color){
  DrawHorizontalLine1px(image, x, y, width + 1, color);
  DrawVerticalLine1px(image, x, y + 1, height + 1 - 1, color);
  DrawVerticalLine1px(image, x + width, y + 1, height + 1 - 1, color);
  DrawHorizontalLine1px(image, x + 1, y + height, width + 1 - 2, color);
}
function DrawImageOnImage(dst, src, topx, topy){
  var y, x;

  for(y = 0; y < ImageHeight(src); y = y + 1){
    for(x = 0; x < ImageWidth(src); x = x + 1){
      if(topx + x >= 0 && topx + x < ImageWidth(dst) && topy + y >= 0 && topy + y < ImageHeight(dst)){
        DrawPixel(dst, topx + x, topy + y, src.x[x].y[y]);
      }
    }
  }
}
function DrawLine1px(image, x0, y0, x1, y1, color){
  XiaolinWusLineAlgorithm(image, x0, y0, x1, y1, color);
}
function XiaolinWusLineAlgorithm(image, x0, y0, x1, y1, color){
  var steep;
  var x, t, dx, dy, g, xEnd, yEnd, xGap, xpxl1, ypxl1, intery, xpxl2, ypxl2, olda;

  olda = color.a;

  steep = Math.abs(y1 - y0) > Math.abs(x1 - x0);

  if(steep){
    t = x0;
    x0 = y0;
    y0 = t;

    t = x1;
    x1 = y1;
    y1 = t;
  }
  if(x0 > x1){
    t = x0;
    x0 = x1;
    x1 = t;

    t = y0;
    y0 = y1;
    y1 = t;
  }

  dx = x1 - x0;
  dy = y1 - y0;
  g = dy/dx;

  if(dx == 0){
    g = 1;
  }

  xEnd = Round(x0);
  yEnd = y0 + g*(xEnd - x0);
  xGap = OneMinusFractionalPart(x0 + 0.5);
  xpxl1 = xEnd;
  ypxl1 = Math.floor(yEnd);
  if(steep){
    DrawPixel(image, ypxl1, xpxl1, SetBrightness(color, OneMinusFractionalPart(yEnd)*xGap));
    DrawPixel(image, ypxl1 + 1, xpxl1, SetBrightness(color, FractionalPart(yEnd)*xGap));
  }else{
    DrawPixel(image, xpxl1, ypxl1, SetBrightness(color, OneMinusFractionalPart(yEnd)*xGap));
    DrawPixel(image, xpxl1, ypxl1 + 1, SetBrightness(color, FractionalPart(yEnd)*xGap));
  }
  intery = yEnd + g;

  xEnd = Round(x1);
  yEnd = y1 + g*(xEnd - x1);
  xGap = FractionalPart(x1 + 0.5);
  xpxl2 = xEnd;
  ypxl2 = Math.floor(yEnd);
  if(steep){
    DrawPixel(image, ypxl2, xpxl2, SetBrightness(color, OneMinusFractionalPart(yEnd)*xGap));
    DrawPixel(image, ypxl2 + 1, xpxl2, SetBrightness(color, FractionalPart(yEnd)*xGap));
  }else{
    DrawPixel(image, xpxl2, ypxl2, SetBrightness(color, OneMinusFractionalPart(yEnd)*xGap));
    DrawPixel(image, xpxl2, ypxl2 + 1, SetBrightness(color, FractionalPart(yEnd)*xGap));
  }

  if(steep){
    for(x = xpxl1 + 1; x <= xpxl2 - 1; x = x + 1){
      DrawPixel(image, Math.floor(intery), x, SetBrightness(color, OneMinusFractionalPart(intery)));
      DrawPixel(image, Math.floor(intery) + 1, x, SetBrightness(color, FractionalPart(intery)));
      intery = intery + g;
    }
  }else{
    for(x = xpxl1 + 1; x <= xpxl2 - 1; x = x + 1){
      DrawPixel(image, x, Math.floor(intery), SetBrightness(color, OneMinusFractionalPart(intery)));
      DrawPixel(image, x, Math.floor(intery) + 1, SetBrightness(color, FractionalPart(intery)));
      intery = intery + g;
    }
  }

  color.a = olda;
}
function OneMinusFractionalPart(x){
  return 1 - FractionalPart(x);
}
function FractionalPart(x){
  return x - Math.floor(x);
}
function SetBrightness(color, newBrightness){
  color.a = newBrightness;
  return color;
}
function DrawQuadraticBezierCurve(image, x0, y0, cx, cy, x1, y1, color){
  var t, dt, dx, dy;
  var xs, ys, xe, ye;

  dx = Math.abs(x0 - x1);
  dy = Math.abs(y0 - y1);

  dt = 1/Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));

  xs = {};
  ys = {};
  xe = {};
  ye = {};

  QuadraticBezierPoint(x0, y0, cx, cy, x1, y1, 0, xs, ys);
  for(t = dt; t <= 1; t = t + dt){
    QuadraticBezierPoint(x0, y0, cx, cy, x1, y1, t, xe, ye);
    DrawLine1px(image, xs.numberValue, ys.numberValue, xe.numberValue, ye.numberValue, color);
    xs.numberValue = xe.numberValue;
    ys.numberValue = ye.numberValue;
  }

  delete(xs);
  delete(ys);
  delete(xe);
  delete(ye);
}
function QuadraticBezierPoint(x0, y0, cx, cy, x1, y1, t, x, y){
  x.numberValue = Math.pow(1 - t, 2)*x0 + (1 - t)*2*t*cx + Math.pow(t, 2)*x1;
  y.numberValue = Math.pow(1 - t, 2)*y0 + (1 - t)*2*t*cy + Math.pow(t, 2)*y1;
}
function DrawCubicBezierCurve(image, x0, y0, c0x, c0y, c1x, c1y, x1, y1, color){
  var t, dt, dx, dy;
  var xs, ys, xe, ye;

  dx = Math.abs(x0 - x1);
  dy = Math.abs(y0 - y1);

  dt = 1/Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));

  xs = {};
  ys = {};
  xe = {};
  ye = {};

  CubicBezierPoint(x0, y0, c0x, c0y, c1x, c1y, x1, y1, 0, xs, ys);
  for(t = dt; t <= 1; t = t + dt){
    CubicBezierPoint(x0, y0, c0x, c0y, c1x, c1y, x1, y1, t, xe, ye);
    DrawLine1px(image, xs.numberValue, ys.numberValue, xe.numberValue, ye.numberValue, color);
    xs.numberValue = xe.numberValue;
    ys.numberValue = ye.numberValue;
  }

  delete(xs);
  delete(ys);
  delete(xe);
  delete(ye);
}
function CubicBezierPoint(x0, y0, c0x, c0y, c1x, c1y, x1, y1, t, x, y){
  x.numberValue = Math.pow(1 - t, 3)*x0 + Math.pow(1 - t, 2)*3*t*c0x + (1 - t)*3*Math.pow(t, 2)*c1x + Math.pow(t, 3)*x1;

  y.numberValue = Math.pow(1 - t, 3)*y0 + Math.pow(1 - t, 2)*3*t*c0y + (1 - t)*3*Math.pow(t, 2)*c1y + Math.pow(t, 3)*y1;
}
function CopyImage(image){
  var copy;
  var i, j;

  copy = CreateImage(ImageWidth(image), ImageHeight(image), GetTransparent());

  for(i = 0; i < ImageWidth(image); i = i + 1){
    for(j = 0; j < ImageHeight(image); j = j + 1){
      SetPixel(copy, i, j, image.x[i].y[j]);
    }
  }

  return copy;
}
function GetImagePixel(image, x, y){
  return image.x[x].y[y];
}
function HorizontalFlip(img){
  var y, x;
  var tmp;
  var c1, c2;

  for(y = 0; y < ImageHeight(img); y = y + 1){
    for(x = 0; x < ImageWidth(img)/2; x = x + 1){
      c1 = img.x[x].y[y];
      c2 = img.x[ImageWidth(img) - 1 - x].y[y];

      tmp = c1.a;
      c1.a = c2.a;
      c2.a = tmp;

      tmp = c1.r;
      c1.r = c2.r;
      c2.r = tmp;

      tmp = c1.g;
      c1.g = c2.g;
      c2.g = tmp;

      tmp = c1.b;
      c1.b = c2.b;
      c2.b = tmp;
    }
  }
}
function DrawFilledRectangle(image, x, y, w, h, color){
  var i, j;

  for(i = 0; i < w; i = i + 1){
    for(j = 0; j < h; j = j + 1){
      SetPixel(image, x + i, y + j, color);
    }
  }
}
function RotateAntiClockwise90Degrees(image){
  var rotated;
  var x, y;

  rotated = CreateImage(ImageHeight(image), ImageWidth(image), GetBlack());

  for(y = 0; y < ImageHeight(image); y = y + 1){
    for(x = 0; x < ImageWidth(image); x = x + 1){
      SetPixel(rotated, y, ImageWidth(image) - 1 - x, GetImagePixel(image, x, y));
    }
  }

  return rotated;
}
function DrawCircle(canvas, xCenter, yCenter, radius, color){
  DrawCircleBasicAlgorithm(canvas, xCenter, yCenter, radius, color);
}
function BresenhamsCircleDrawingAlgorithm(canvas, xCenter, yCenter, radius, color){
  var x, y, delta;

  y = radius;
  x = 0;

  delta = 3 - 2*radius;
  for(; y >= x; x = x + 1){
    DrawLine1px(canvas, xCenter + x, yCenter + y, xCenter + x, yCenter + y, color);
    DrawLine1px(canvas, xCenter + x, yCenter - y, xCenter + x, yCenter - y, color);
    DrawLine1px(canvas, xCenter - x, yCenter + y, xCenter - x, yCenter + y, color);
    DrawLine1px(canvas, xCenter - x, yCenter - y, xCenter - x, yCenter - y, color);

    DrawLine1px(canvas, xCenter - y, yCenter + x, xCenter - y, yCenter + x, color);
    DrawLine1px(canvas, xCenter - y, yCenter - x, xCenter - y, yCenter - x, color);
    DrawLine1px(canvas, xCenter + y, yCenter + x, xCenter + y, yCenter + x, color);
    DrawLine1px(canvas, xCenter + y, yCenter - x, xCenter + y, yCenter - x, color);

    if(delta < 0){
      delta = delta + 4*x + 6;
    }else{
      delta = delta + 4*(x - y) + 10;
      y = y - 1;
    }
  }
}
function DrawCircleMidpointAlgorithm(canvas, xCenter, yCenter, radius, color){
  var d, x, y;

  d = Math.floor((5 - radius*4)/4);
  x = 0;
  y = radius;

  for(; x <= y; x = x + 1){
    DrawPixel(canvas, xCenter + x, yCenter + y, color);
    DrawPixel(canvas, xCenter + x, yCenter - y, color);
    DrawPixel(canvas, xCenter - x, yCenter + y, color);
    DrawPixel(canvas, xCenter - x, yCenter - y, color);
    DrawPixel(canvas, xCenter + y, yCenter + x, color);
    DrawPixel(canvas, xCenter + y, yCenter - x, color);
    DrawPixel(canvas, xCenter - y, yCenter + x, color);
    DrawPixel(canvas, xCenter - y, yCenter - x, color);

    if(d < 0){
      d = d + 2*x + 1;
    }else{
      d = d + 2*(x - y) + 1;
      y = y - 1;
    }
  }
}
function DrawCircleBasicAlgorithm(canvas, xCenter, yCenter, radius, color){
  var pixels, a, da, dx, dy;

  /* Place the circle in the center of the pixel. */
  xCenter = Math.floor(xCenter) + 0.5;
  yCenter = Math.floor(yCenter) + 0.5;

  pixels = 2*Math.PI*radius;

  /* Below a radius of 10 pixels, over-compensate to get a smoother circle. */
  if(radius < 10){
    pixels = pixels*10;
  }

  da = 2*Math.PI/pixels;

  for(a = 0; a < 2*Math.PI; a = a + da){
    dx = Math.cos(a)*radius;
    dy = Math.sin(a)*radius;

    /* Floor to get the pixel coordinate. */
    DrawPixel(canvas, Math.floor(xCenter + dx), Math.floor(yCenter + dy), color);
  }
}
function DrawFilledCircle(canvas, x, y, r, color){
  DrawFilledCircleBasicAlgorithm(canvas, x, y, r, color);
}
function DrawFilledCircleMidpointAlgorithm(canvas, xCenter, yCenter, radius, color){
  var d, x, y;

  d = Math.floor((5 - radius*4)/4);
  x = 0;
  y = radius;

  for(; x <= y; x = x + 1){
    DrawLineBresenhamsAlgorithm(canvas, xCenter + x, yCenter + y, xCenter - x, yCenter + y, color);
    DrawLineBresenhamsAlgorithm(canvas, xCenter + x, yCenter - y, xCenter - x, yCenter - y, color);
    DrawLineBresenhamsAlgorithm(canvas, xCenter + y, yCenter + x, xCenter - y, yCenter + x, color);
    DrawLineBresenhamsAlgorithm(canvas, xCenter + y, yCenter - x, xCenter - y, yCenter - x, color);

    if(d < 0){
      d = d + 2*x + 1;
    }else{
      d = d + 2*(x - y) + 1;
      y = y - 1;
    }
  }
}
function DrawFilledCircleBasicAlgorithm(canvas, xCenter, yCenter, radius, color){
  var pixels, a, da, dx, dy;

  /* Place the circle in the center of the pixel. */
  xCenter = Math.floor(xCenter) + 0.5;
  yCenter = Math.floor(yCenter) + 0.5;

  pixels = 2*Math.PI*radius;

  /* Below a radius of 10 pixels, over-compensate to get a smoother circle. */
  if(radius < 10){
    pixels = pixels*10;
  }

  da = 2*Math.PI/pixels;

  /* Draw lines for a half-circle to fill an entire circle. */
  for(a = 0; a < Math.PI; a = a + da){
    dx = Math.cos(a)*radius;
    dy = Math.sin(a)*radius;

    /* Floor to get the pixel coordinate. */
    DrawVerticalLine1px(canvas, Math.floor(xCenter - dx), Math.floor(yCenter - dy), Math.floor(2*dy) + 1, color);
  }
}
function DrawTriangle(canvas, xCenter, yCenter, height, color){
  var x1, y1, x2, y2, x3, y3;

  x1 = Math.floor(xCenter + 0.5);
  y1 = Math.floor(Math.floor(yCenter + 0.5) - height);
  x2 = x1 - 2*height*Math.tan(Math.PI/6);
  y2 = Math.floor(y1 + 2*height);
  x3 = x1 + 2*height*Math.tan(Math.PI/6);
  y3 = Math.floor(y1 + 2*height);

  DrawLine1px(canvas, x1, y1, x2, y2, color);
  DrawLine1px(canvas, x1, y1, x3, y3, color);
  DrawLine1px(canvas, x2, y2, x3, y3, color);
}
function DrawFilledTriangle(canvas, xCenter, yCenter, height, color){
  var i, offset, x1, y1;

  x1 = Math.floor(xCenter + 0.5);
  y1 = Math.floor(Math.floor(yCenter + 0.5) - height);

  for(i = 0; i <= 2*height; i = i + 1){
    offset = Math.floor(i*Math.tan(Math.PI/6));
    DrawHorizontalLine1px(canvas, x1 - offset, y1 + i, 2*offset, color);
  }
}
function DrawLine(canvas, x1, y1, x2, y2, thickness, color){
  DrawLineBresenhamsAlgorithmThick(canvas, x1, y1, x2, y2, thickness, color);
}
function DrawLineBresenhamsAlgorithmThick(canvas, x1, y1, x2, y2, thickness, color){
  var x, y, dx, dy, incX, incY, pdx, pdy, es, el, err, t, r;

  dx = x2 - x1;
  dy = y2 - y1;

  incX = Sign(dx);
  incY = Sign(dy);

  dx = Math.abs(dx);
  dy = Math.abs(dy);

  if(dx > dy){
    pdx = incX;
    pdy = 0;
    es = dy;
    el = dx;
  }else{
    pdx = 0;
    pdy = incY;
    es = dx;
    el = dy;
  }

  x = x1;
  y = y1;
  err = el/2;

  if(thickness >= 3){
    r = thickness/2;
    DrawCircle(canvas, x, y, r, color);
  }else if(Math.floor(thickness) == 2){
    DrawFilledRectangle(canvas, x, y, 2, 2, color);
  }else if(Math.floor(thickness) == 1){
    DrawPixel(canvas, x, y, color);
  }

  for(t = 0; t < el; t = t + 1){
    err = err - es;
    if(err < 0){
      err = err + el;
      x = x + incX;
      y = y + incY;
    }else{
      x = x + pdx;
      y = y + pdy;
    }

    if(thickness >= 3){
      r = thickness/2;
      DrawCircle(canvas, x, y, r, color);
    }else if(Math.floor(thickness) == 2){
      DrawFilledRectangle(canvas, x, y, 2, 2, color);
    }else if(Math.floor(thickness) == 1){
      DrawPixel(canvas, x, y, color);
    }
  }
}
function DrawLineBresenhamsAlgorithm(canvas, x1, y1, x2, y2, color){
  var x, y, dx, dy, incX, incY, pdx, pdy, es, el, err, t;

  dx = x2 - x1;
  dy = y2 - y1;

  incX = Sign(dx);
  incY = Sign(dy);

  dx = Math.abs(dx);
  dy = Math.abs(dy);

  if(dx > dy){
    pdx = incX;
    pdy = 0;
    es = dy;
    el = dx;
  }else{
    pdx = 0;
    pdy = incY;
    es = dx;
    el = dy;
  }

  x = x1;
  y = y1;
  err = el/2;
  DrawPixel(canvas, x, y, color);

  for(t = 0; t < el; t = t + 1){
    err = err - es;
    if(err < 0){
      err = err + el;
      x = x + incX;
      y = y + incY;
    }else{
      x = x + pdx;
      y = y + pdy;
    }

    DrawPixel(canvas, x, y, color);
  }
}
function DrawLineBresenhamsAlgorithmThickPatterned(canvas, x1, y1, x2, y2, thickness, pattern, offset, color){
  var x, y, dx, dy, incX, incY, pdx, pdy, es, el, err, t, r;

  dx = x2 - x1;
  dy = y2 - y1;

  incX = Sign(dx);
  incY = Sign(dy);

  dx = Math.abs(dx);
  dy = Math.abs(dy);

  if(dx > dy){
    pdx = incX;
    pdy = 0;
    es = dy;
    el = dx;
  }else{
    pdx = 0;
    pdy = incY;
    es = dx;
    el = dy;
  }

  x = x1;
  y = y1;
  err = el/2;

  offset.numberValue = (offset.numberValue + 1)%(pattern.length*thickness);

  if(pattern[Math.floor(offset.numberValue/thickness)]){
    if(thickness >= 3){
      r = thickness/2;
      DrawCircle(canvas, x, y, r, color);
    }else if(Math.floor(thickness) == 2){
      DrawFilledRectangle(canvas, x, y, 2, 2, color);
    }else if(Math.floor(thickness) == 1){
      DrawPixel(canvas, x, y, color);
    }
  }

  for(t = 0; t < el; t = t + 1){
    err = err - es;
    if(err < 0){
      err = err + el;
      x = x + incX;
      y = y + incY;
    }else{
      x = x + pdx;
      y = y + pdy;
    }

    offset.numberValue = (offset.numberValue + 1)%(pattern.length*thickness);

    if(pattern[Math.floor(offset.numberValue/thickness)]){
      if(thickness >= 3){
        r = thickness/2;
        DrawCircle(canvas, x, y, r, color);
      }else if(Math.floor(thickness) == 2){
        DrawFilledRectangle(canvas, x, y, 2, 2, color);
      }else if(Math.floor(thickness) == 1){
        DrawPixel(canvas, x, y, color);
      }
    }
  }
}
function GetLinePattern5(){
  var pattern;

  pattern = [];
  pattern.length = 19;

  pattern[0] = true;
  pattern[1] = true;
  pattern[2] = true;
  pattern[3] = true;
  pattern[4] = true;
  pattern[5] = true;
  pattern[6] = true;
  pattern[7] = true;
  pattern[8] = true;
  pattern[9] = true;
  pattern[10] = false;
  pattern[11] = false;
  pattern[12] = false;
  pattern[13] = true;
  pattern[14] = true;
  pattern[15] = true;
  pattern[16] = false;
  pattern[17] = false;
  pattern[18] = false;

  return pattern;
}
function GetLinePattern4(){
  var pattern;

  pattern = [];
  pattern.length = 13;

  pattern[0] = true;
  pattern[1] = true;
  pattern[2] = true;
  pattern[3] = true;
  pattern[4] = true;
  pattern[5] = true;
  pattern[6] = true;
  pattern[7] = true;
  pattern[8] = true;
  pattern[9] = true;
  pattern[10] = false;
  pattern[11] = false;
  pattern[12] = false;

  return pattern;
}
function GetLinePattern3(){
  var pattern;

  pattern = [];
  pattern.length = 13;

  pattern[0] = true;
  pattern[1] = true;
  pattern[2] = true;
  pattern[3] = true;
  pattern[4] = true;
  pattern[5] = true;
  pattern[6] = false;
  pattern[7] = false;
  pattern[8] = false;
  pattern[9] = true;
  pattern[10] = true;
  pattern[11] = false;
  pattern[12] = false;

  return pattern;
}
function GetLinePattern2(){
  var pattern;

  pattern = [];
  pattern.length = 4;

  pattern[0] = true;
  pattern[1] = true;
  pattern[2] = false;
  pattern[3] = false;

  return pattern;
}
function GetLinePattern1(){
  var pattern;

  pattern = [];
  pattern.length = 8;

  pattern[0] = true;
  pattern[1] = true;
  pattern[2] = true;
  pattern[3] = true;
  pattern[4] = true;
  pattern[5] = false;
  pattern[6] = false;
  pattern[7] = false;

  return pattern;
}
function Blur(src, pixels){
  var dst;
  var x, y, w, h;

  w = ImageWidth(src);
  h = ImageHeight(src);
  dst = CreateImage(w, h, GetTransparent());

  for(x = 0; x < w; x = x + 1){
    for(y = 0; y < h; y = y + 1){
      SetPixel(dst, x, y, CreateBlurForPoint(src, x, y, pixels));
    }
  }

  return dst;
}
function CreateBlurForPoint(src, x, y, pixels){
  var rgba;
  var i, j, countColor, countTransparent;
  var fromx, tox, fromy, toy;
  var w, h;
  var alpha;

  w = src.x.length;
  h = src.x[0].y.length;

  rgba = {};
  rgba.r = 0;
  rgba.g = 0;
  rgba.b = 0;
  rgba.a = 0;

  fromx = x - pixels;
  fromx = Math.max(fromx, 0);

  tox = x + pixels;
  tox = Math.min(tox, w - 1);

  fromy = y - pixels;
  fromy = Math.max(fromy, 0);

  toy = y + pixels;
  toy = Math.min(toy, h - 1);

  countColor = 0;
  countTransparent = 0;
  for(i = fromx; i < tox; i = i + 1){
    for(j = fromy; j < toy; j = j + 1){
      alpha = src.x[i].y[j].a;
      if(alpha > 0){
        rgba.r = rgba.r + src.x[i].y[j].r;
        rgba.g = rgba.g + src.x[i].y[j].g;
        rgba.b = rgba.b + src.x[i].y[j].b;
        countColor = countColor + 1;
      }
      rgba.a = rgba.a + alpha;
      countTransparent = countTransparent + 1;
    }
  }

  if(countColor > 0){
    rgba.r = rgba.r/countColor;
    rgba.g = rgba.g/countColor;
    rgba.b = rgba.b/countColor;
  }else{
    rgba.r = 0;
    rgba.g = 0;
    rgba.b = 0;
  }

  if(countTransparent > 0){
    rgba.a = rgba.a/countTransparent;
  }else{
    rgba.a = 0;
  }

  return rgba;
}
function GetBase64Table(){
  return "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".split('');
}
function GetBase64ReverseTable(){
  var table;
  var i, max;
  var revTable;

  table = GetBase64Table();
  max = GetBase64Maximum(table) + 1;
  revTable = [];
  revTable.length = max + 1;

  for(i = 0; i < table.length; i = i + 1){
    revTable[table[i].charCodeAt(0)] = i;
  }

  return revTable;
}
function GetBase64ValidTable(){
  var table;
  var max, i;
  var validTable;

  table = GetBase64Table();
  max = GetBase64Maximum(table) + 1;
  validTable = [];
  validTable.length = max;

  for(i = 0; i < max; i = i + 1){
    validTable[i] = IsValidBase64Character(String.fromCharCode(i));
  }

  return validTable;
}
function GetBase64Maximum(table){
  var maxValue, v, i;

  maxValue = 0;

  for(i = 0; i < table.length; i = i + 1){
    v = table[i].charCodeAt(0);
    maxValue = Math.max(maxValue, v);
  }
  return maxValue;
}
function IsValidBase64Character(c){
  var table;
  var isValid;
  var i;

  table = GetBase64Table();
  isValid = false;

  for(i = 0; i < table.length; i = i + 1){
    if(table[i] == c){
      isValid = true;
    }
  }

  return isValid;
}
function Base64Encode(data, encoded, errorMessage){
  var padding, triples, i, currentTriple, currentQuad;
  var data3;
  var done, success;
  var encoded4;

  padding = 0;
  if((data.length%3) == 1){
    padding = 2;
  }
  if((data.length%3) == 2){
    padding = 1;
  }
  triples = Math.ceil(data.length/3);

  /* Init */
  encoded.string = [];
  encoded.string.length = triples*4;
  errorMessage.string = "".split('');
  success = true;
  encoded4 = CreateStringReferenceLengthValue(4, 'A');
  done = false;
  data3 = [];
  data3.length = 3;

  for(i = 0; i < triples &&  !done ; i = i + 1){
    currentTriple = i*3;
    currentQuad = i*4;

    /* Fetch triplet. */
    if(padding > 0 && i + 1 == triples){
      if(padding == 2){
        data3[0] = data[currentTriple + 0];
        data3[1] = 0;
        data3[2] = 0;
      }else if(padding == 1){
        data3[0] = data[currentTriple + 0];
        data3[1] = data[currentTriple + 1];
        data3[2] = 0;
      }
    }else{
      data3[0] = data[currentTriple + 0];
      data3[1] = data[currentTriple + 1];
      data3[2] = data[currentTriple + 2];
    }

    success = Base64Encode3Numbers(data3, encoded4, errorMessage);

    if(success){
      /* Write quad. */
      encoded.string[currentQuad + 0] = encoded4.string[0];
      encoded.string[currentQuad + 1] = encoded4.string[1];

      if(padding > 0 && i + 1 == triples){
        if(padding == 2){
          encoded.string[currentQuad + 2] = '=';
          encoded.string[currentQuad + 3] = '=';
        }else if(padding == 1){
          encoded.string[currentQuad + 2] = encoded4.string[2];
          encoded.string[currentQuad + 3] = '=';
        }
      }else{
        encoded.string[currentQuad + 2] = encoded4.string[2];
        encoded.string[currentQuad + 3] = encoded4.string[3];
      }
    }else{
      success = false;
      done = true;
    }
  }

  delete(encoded4);
  delete(data3);

  return success;
}
function Base64Encode3Numbers(data, encoded4, errorMessage){
  var i, e;
  var isWithinBounds, isWhole;
  var success;

  /* Init */
  encoded4.string[0] = 'A';
  encoded4.string[1] = 'A';
  encoded4.string[2] = 'A';
  encoded4.string[3] = 'A';
  errorMessage.string = "".split('');

  /* Check. */
  if(data.length == 3){
    success = true;

    for(i = 0; i < data.length && success; i = i + 1){
      e = data[i];

      isWithinBounds = (e >= 0) && (e < Math.pow(2, 8));
      isWhole = IsInteger(e);
      if(isWithinBounds && isWhole){
      }else{
        errorMessage.string = "Input number is too high, too low or is not a whole number.".split('');
        success = false;
      }
    }

    if(success){
      Base64Encode3NumbersNoChecks(data, encoded4);
    }
  }else{
    errorMessage.string = "Input must contain 3 numbers.".split('');
    success = false;
  }

  return success;
}
function Base64Encode3NumbersNoChecks(data, encoded){
  var total, bit6_1, bit6_2, bit6_3, bit6_4;
  var d0, d1, d2, t1, t2, t3, t4;

  d2 = data[2];
  d1 = data[1];
  d0 = data[0];

  /* total = d2 | d1 << 8 | d0 << 16 */
  t1 = bShiftLeft4Byte(d1, 8);
  t2 = bOr4Byte(d2, t1);
  t3 = bShiftLeft4Byte(d0, 16);
  t4 = bOr4Byte(t2, t3);
  total = t4;

  /* bit6_1 = total & 0b111111 */
  bit6_1 = bAnd4Byte(total, 63);

  /* bit6_2 = total >> 6 & 0b111111 */
  t1 = bShiftRight4Byte(total, 6);
  t2 = bAnd4Byte(t1, 63);
  bit6_2 = t2;

  /* bit6_3 = total >> 12 & 0b111111 */
  t1 = bShiftRight4Byte(total, 12);
  t2 = bAnd4Byte(t1, 63);
  bit6_3 = t2;

  /* bit6_4 = total >> 18 & 0b111111 */
  t1 = bShiftRight4Byte(total, 18);
  t2 = bAnd4Byte(t1, 63);
  bit6_4 = t2;

  encoded.string[3] = GetBase64CharacterForNumber(bit6_1);
  encoded.string[2] = GetBase64CharacterForNumber(bit6_2);
  encoded.string[1] = GetBase64CharacterForNumber(bit6_3);
  encoded.string[0] = GetBase64CharacterForNumber(bit6_4);
}
function GetBase64CharacterForNumber(bit6){
  var table;

  table = GetBase64Table();

  return table[bit6];
}
function Base64EncodeFromText(data){
  var dataBytes;
  var encoded, errorMessage;
  var result;
  var success;

  dataBytes = UTF16StringToUTF8(data);

  encoded = {};
  errorMessage = {};
  success = Base64Encode(dataBytes, encoded, errorMessage);

  if(success){
    result = encoded.string;
  }else{
    result = "".split('');
  }

  FreeStringReference(encoded);
  delete(dataBytes);
  delete(errorMessage);

  return result;
}
function Base64Decode(data, decodedReference, errorMessage){
  var done, success;
  var data4;
  var quads, i, currentQuad, currentTriple, padding;
  var table, decoded;
  var decoded3;
  var validTable;

  table = GetBase64ReverseTable();
  validTable = GetBase64ValidTable();
  decoded3 = CreateNumberArrayReferenceLengthValue(3, 0);

  padding = 0;
  if(data.length > 1){
    if(data[data.length - 1] == '='){
      padding = padding + 1;
    }
  }
  if(data.length > 2){
    if(data[data.length - 2] == '='){
      padding = padding + 1;
    }
  }
  quads = data.length/4;

  /* Init */
  decoded = [];
  decoded.length = quads*3 - padding;
  errorMessage.string = "".split('');
  success = true;

  done = false;
  for(i = 0; i < quads &&  !done ; i = i + 1){
    data4 = [];
    data4.length = 4;
    currentQuad = i*4;
    currentTriple = i*3;

    if(padding > 0 && (i + 1) == quads){
      if(padding == 2){
        data4[0] = data[currentQuad + 0];
        data4[1] = data[currentQuad + 1];
        data4[2] = 'A';
        data4[3] = 'A';
      }else if(padding == 1){
        data4[0] = data[currentQuad + 0];
        data4[1] = data[currentQuad + 1];
        data4[2] = data[currentQuad + 2];
        data4[3] = 'A';
      }
    }else{
      data4[0] = data[currentQuad + 0];
      data4[1] = data[currentQuad + 1];
      data4[2] = data[currentQuad + 2];
      data4[3] = data[currentQuad + 3];
    }

    success = Base64Decode4WithTable(data4, table, validTable, decoded3, errorMessage);

    if(success){
      if(padding > 0 && i + 1 == quads){
        if(padding == 2){
          decoded[currentTriple + 0] = decoded3.numberArray[0];
        }else if(padding == 1){
          decoded[currentTriple + 0] = decoded3.numberArray[0];
          decoded[currentTriple + 1] = decoded3.numberArray[1];
        }
      }else{
        decoded[currentTriple + 0] = decoded3.numberArray[0];
        decoded[currentTriple + 1] = decoded3.numberArray[1];
        decoded[currentTriple + 2] = decoded3.numberArray[2];
      }
    }else{
      success = false;
      done = true;
    }
  }

  decodedReference.numberArray = decoded;
  delete(table);
  delete(validTable);
  FreeNumberArrayReference(decoded3);

  return success;
}
function Base64Decode4Chars(data, decoded3, errorMessage){
  var table;
  var validTable;
  var success;

  table = GetBase64ReverseTable();
  validTable = GetBase64ValidTable();

  success = Base64Decode4WithTable(data, table, validTable, decoded3, errorMessage);

  delete(validTable);
  delete(table);

  return success;
}
function Base64Decode4WithTable(data, table, validTable, decoded3, errorMessage){
  var i, valid;
  var success;

  if(data.length == 4){
    valid = 0;

    for(i = 0; i < 4; i = i + 1){
      if(IsValidBase64CharacterTable(data[i], validTable)){
        valid = valid + 1;
      }
    }

    if(valid == 4){
      Base64Decode4WithTableNoChecks(decoded3, data, table);
      success = true;
    }else{
      errorMessage.string = "Invalid character in input string.".split('');
      success = false;
    }
  }else{
    errorMessage.string = "There must be exactly four characters in the input string.".split('');
    success = false;
  }

  return success;
}
function IsValidBase64CharacterTable(c, validTable){
  return validTable[c.charCodeAt(0)];
}
function Base64Decode4NoChecks(decoded, data){
  var table;

  table = GetBase64ReverseTable();
  Base64Decode4WithTableNoChecks(decoded, data, table);
  delete(table);
}
function Base64Decode4WithTableNoChecks(decoded, data, table){
  var total, bit6_1, bit6_2, bit6_3, bit6_4, t1, t2, t3, t4, t5, t6, byte0, byte1, byte2;

  bit6_1 = GetBase64NumberForCharacter(data[3], table);
  bit6_2 = GetBase64NumberForCharacter(data[2], table);
  bit6_3 = GetBase64NumberForCharacter(data[1], table);
  bit6_4 = GetBase64NumberForCharacter(data[0], table);

  /* total = bit6_1 | bit6_2 << 6 | bit6_3 << 12 | bit6_4 << 18 */
  t1 = bShiftLeft4Byte(bit6_2, 6);
  t2 = bOr4Byte(bit6_1, t1);
  t3 = bShiftLeft4Byte(bit6_3, 12);
  t4 = bOr4Byte(t2, t3);
  t5 = bShiftLeft4Byte(bit6_4, 18);
  t6 = bOr4Byte(t4, t5);
  total = t6;

  /* byte2 = total & 255 */
  t1 = bAnd4Byte(total, 255);
  byte2 = t1;

  /* byte1 = total >> 8 & 255 */
  t1 = bShiftRight4Byte(total, 8);
  t2 = bAnd4Byte(t1, 255);
  byte1 = t2;

  /* byte0 = total >> 16 & 255 */
  t1 = bShiftRight4Byte(total, 16);
  t2 = bAnd4Byte(t1, 255);
  byte0 = t2;

  decoded.numberArray[2] = byte2;
  decoded.numberArray[1] = byte1;
  decoded.numberArray[0] = byte0;
}
function GetBase64NumberForCharacter(c, table){
  return table[c.charCodeAt(0)];
}
function Base64DecodeToText(data){
  var decoded;
  var errorMessage;
  var result;
  var success;

  decoded = {};
  errorMessage = {};
  success = Base64Decode(data, decoded, errorMessage);

  if(success){
    result = UTF8ToUTF16String(decoded.numberArray);
    FreeNumberArrayReference(decoded);
  }else{
    result = "".split('');
  }

  return result;
}
function WriteStringToStingStream(stream, index, src){
  var i;

  for(i = 0; i < src.length; i = i + 1){
    stream[index.numberValue + i] = src[i];
  }
  index.numberValue = index.numberValue + src.length;
}
function WriteCharacterToStingStream(stream, index, src){
  stream[index.numberValue] = src;
  index.numberValue = index.numberValue + 1;
}
function WriteBooleanToStingStream(stream, index, src){
  if(src){
    WriteStringToStingStream(stream, index, "true".split(''));
  }else{
    WriteStringToStingStream(stream, index, "false".split(''));
  }
}
function SubstringWithCheck(string, from, to, stringReference){
  var success;

  if(from >= 0 && from <= string.length && to >= 0 && to <= string.length && from <= to){
    stringReference.string = Substring(string, from, to);
    success = true;
  }else{
    success = false;
  }

  return success;
}
function Substring(string, from, to){
  var n;
  var i, lengthx;

  lengthx = to - from;

  n = [];
  n.length = lengthx;

  for(i = from; i < to; i = i + 1){
    n[i - from] = string[i];
  }

  return n;
}
function AppendString(s1, s2){
  var newString;

  newString = ConcatenateString(s1, s2);

  delete(s1);

  return newString;
}
function ConcatenateString(s1, s2){
  var newString;
  var i;

  newString = [];
  newString.length = s1.length + s2.length;

  for(i = 0; i < s1.length; i = i + 1){
    newString[i] = s1[i];
  }

  for(i = 0; i < s2.length; i = i + 1){
    newString[s1.length + i] = s2[i];
  }

  return newString;
}
function AppendCharacter(string, c){
  var newString;

  newString = ConcatenateCharacter(string, c);

  delete(string);

  return newString;
}
function ConcatenateCharacter(string, c){
  var newString;
  var i;
  newString = [];
  newString.length = string.length + 1;

  for(i = 0; i < string.length; i = i + 1){
    newString[i] = string[i];
  }

  newString[string.length] = c;

  return newString;
}
function SplitByCharacter(toSplit, splitBy){
  var split;
  var stringToSplitBy;

  stringToSplitBy = [];
  stringToSplitBy.length = 1;
  stringToSplitBy[0] = splitBy;

  split = SplitByString(toSplit, stringToSplitBy);

  delete(stringToSplitBy);

  return split;
}
function IndexOfCharacter(string, character, indexReference){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length &&  !found ; i = i + 1){
    if(string[i] == character){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function SubstringEqualsWithCheck(string, from, substring, equalsReference){
  var success;

  if(from < string.length){
    success = true;
    equalsReference.booleanValue = SubstringEquals(string, from, substring);
  }else{
    success = false;
  }

  return success;
}
function SubstringEquals(string, from, substring){
  var i;
  var equal;

  equal = true;
  if(string.length - from >= substring.length){
    for(i = 0; i < substring.length && equal; i = i + 1){
      if(string[from + i] != substring[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function IndexOfString(string, substring, indexReference){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length - substring.length + 1 &&  !found ; i = i + 1){
    if(SubstringEquals(string, i, substring)){
      found = true;
      indexReference.numberValue = i;
    }
  }

  return found;
}
function ContainsCharacter(string, character){
  var i;
  var found;

  found = false;
  for(i = 0; i < string.length &&  !found ; i = i + 1){
    if(string[i] == character){
      found = true;
    }
  }

  return found;
}
function ContainsString(string, substring){
  return IndexOfString(string, substring, {});
}
function ToUpperCase(string){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToUpperCase(string[i]);
  }
}
function ToLowerCase(string){
  var i;

  for(i = 0; i < string.length; i = i + 1){
    string[i] = charToLowerCase(string[i]);
  }
}
function EqualsIgnoreCase(a, b){
  var equal;
  var i;

  if(a.length == b.length){
    equal = true;
    for(i = 0; i < a.length && equal; i = i + 1){
      if(charToLowerCase(a[i]) != charToLowerCase(b[i])){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function ReplaceString(string, toReplace, replaceWith){
  var result;
  var i;
  var equalsReference;
  var success;

  equalsReference = {};
  result = [];
  result.length = 0;

  for(i = 0; i < string.length; ){
    success = SubstringEqualsWithCheck(string, i, toReplace, equalsReference);
    if(success){
      success = equalsReference.booleanValue;
    }

    if(success && toReplace.length > 0){
      result = ConcatenateString(result, replaceWith);
      i = i + toReplace.length;
    }else{
      result = ConcatenateCharacter(result, string[i]);
      i = i + 1;
    }
  }

  return result;
}
function ReplaceCharacter(string, toReplace, replaceWith){
  var result;
  var i;

  result = [];
  result.length = 0;

  for(i = 0; i < string.length; i = i + 1){
    if(string[i] == toReplace){
      result = ConcatenateCharacter(result, replaceWith);
    }else{
      result = ConcatenateCharacter(result, string[i]);
    }
  }

  return result;
}
function Trim(string){
  var result;
  var i, lastWhitespaceLocationStart, lastWhitespaceLocationEnd;
  var firstNonWhitespaceFound;

  /* Find whitepaces at the start. */
  lastWhitespaceLocationStart =  -1;
  firstNonWhitespaceFound = false;
  for(i = 0; i < string.length &&  !firstNonWhitespaceFound ; i = i + 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationStart = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  /* Find whitepaces at the end. */
  lastWhitespaceLocationEnd = string.length;
  firstNonWhitespaceFound = false;
  for(i = string.length - 1; i >= 0 &&  !firstNonWhitespaceFound ; i = i - 1){
    if(charIsWhiteSpace(string[i])){
      lastWhitespaceLocationEnd = i;
    }else{
      firstNonWhitespaceFound = true;
    }
  }

  if(lastWhitespaceLocationStart < lastWhitespaceLocationEnd){
    result = Substring(string, lastWhitespaceLocationStart + 1, lastWhitespaceLocationEnd);
  }else{
    result = [];
    result.length = 0;
  }

  return result;
}
function StartsWith(string, start){
  var startsWithString;

  startsWithString = false;
  if(string.length >= start.length){
    startsWithString = SubstringEquals(string, 0, start);
  }

  return startsWithString;
}
function EndsWith(string, end){
  var endsWithString;

  endsWithString = false;
  if(string.length >= end.length){
    endsWithString = SubstringEquals(string, string.length - end.length, end);
  }

  return endsWithString;
}
function SplitByString(toSplit, splitBy){
  var split;
  var next;
  var i;
  var c;
  var n;

  split = [];
  split.length = 0;

  next = [];
  next.length = 0;
  for(i = 0; i < toSplit.length; ){
    c = toSplit[i];

    if(SubstringEquals(toSplit, i, splitBy)){
      n = {};
      n.string = next;
      split = AddString(split, n);
      next = [];
      next.length = 0;
      i = i + splitBy.length;
    }else{
      next = AppendCharacter(next, c);
      i = i + 1;
    }
  }

  n = {};
  n.string = next;
  split = AddString(split, n);

  return split;
}
function StringIsBefore(a, b){
  var before, equal, done;
  var i;

  before = false;
  equal = true;
  done = false;

  if(a.length == 0 && b.length > 0){
    before = true;
  }else{
    for(i = 0; i < a.length && i < b.length &&  !done ; i = i + 1){
      if(a[i] != b[i]){
        equal = false;
      }
      if(charCharacterIsBefore(a[i], b[i])){
        before = true;
      }
      if(charCharacterIsBefore(b[i], a[i])){
        done = true;
      }
    }

    if(equal){
      if(a.length < b.length){
        before = true;
      }
    }
  }

  return before;
}
function Negate(x){
  return  -x;
}
function Positive(x){
  return  +x;
}
function Factorial(x){
  var i, f;

  f = 1;

  for(i = 2; i <= x; i = i + 1){
    f = f*i;
  }

  return f;
}
function Round(x){
  return Math.floor(x + 0.5);
}
function BankersRound(x){
  var r;

  if(Absolute(x - Truncate(x)) == 0.5){
    if( !DivisibleBy(Round(x), 2) ){
      r = Round(x) - 1;
    }else{
      r = Round(x);
    }
  }else{
    r = Round(x);
  }

  return r;
}
function Ceil(x){
  return Math.ceil(x);
}
function Floor(x){
  return Math.floor(x);
}
function Truncate(x){
  var t;

  if(x >= 0){
    t = Math.floor(x);
  }else{
    t = Math.ceil(x);
  }

  return t;
}
function Absolute(x){
  return Math.abs(x);
}
function Logarithm(x){
  return Math.log10(x);
}
function NaturalLogarithm(x){
  return Math.log(x);
}
function Sin(x){
  return Math.sin(x);
}
function Cos(x){
  return Math.cos(x);
}
function Tan(x){
  return Math.tan(x);
}
function Asin(x){
  return Math.asin(x);
}
function Acos(x){
  return Math.acos(x);
}
function Atan(x){
  return Math.atan(x);
}
function Atan2(y, x){
  var a;

  /* Atan2 is an invalid operation when x = 0 and y = 0, but this method does not return errors. */
  a = 0;

  if(x > 0){
    a = Atan(y/x);
  }else if(x < 0 && y >= 0){
    a = Atan(y/x) + Math.PI;
  }else if(x < 0 && y < 0){
    a = Atan(y/x) - Math.PI;
  }else if(x == 0 && y > 0){
    a = Math.PI/2;
  }else if(x == 0 && y < 0){
    a =  -Math.PI/2;
  }

  return a;
}
function Squareroot(x){
  return Math.sqrt(x);
}
function Exp(x){
  return Math.exp(x);
}
function DivisibleBy(a, b){
  return ((a%b) == 0);
}
function Combinations(n, k){
  var i, j, c;

  c = 1;
  j = 1;
  i = n - k + 1;

  for(; i <= n; ){
    c = c*i;
    c = c/j;

    i = i + 1;
    j = j + 1;
  }

  return c;
}
function Permutations(n, k){
  var i, c;

  c = 1;

  for(i = n - k + 1; i <= n; i = i + 1){
    c = c*i;
  }

  return c;
}
function EpsilonCompare(a, b, epsilon){
  return Math.abs(a - b) < epsilon;
}
function GreatestCommonDivisor(a, b){
  var t;

  for(; b != 0; ){
    t = b;
    b = a%b;
    a = t;
  }

  return a;
}
function GCDWithSubtraction(a, b){
  var g;

  if(a == 0){
    g = b;
  }else{
    for(; b != 0; ){
      if(a > b){
        a = a - b;
      }else{
        b = b - a;
      }
    }

    g = a;
  }

  return g;
}
function IsInteger(a){
  return (a - Math.floor(a)) == 0;
}
function GreatestCommonDivisorWithCheck(a, b, gcdReference){
  var success;
  var gcd;

  if(IsInteger(a) && IsInteger(b)){
    gcd = GreatestCommonDivisor(a, b);
    gcdReference.numberValue = gcd;
    success = true;
  }else{
    success = false;
  }

  return success;
}
function LeastCommonMultiple(a, b){
  var lcm;

  if(a > 0 && b > 0){
    lcm = Math.abs(a*b)/GreatestCommonDivisor(a, b);
  }else{
    lcm = 0;
  }

  return lcm;
}
function Sign(a){
  var s;

  if(a > 0){
    s = 1;
  }else if(a < 0){
    s =  -1;
  }else{
    s = 0;
  }

  return s;
}
function Max(a, b){
  return Math.max(a, b);
}
function Min(a, b){
  return Math.min(a, b);
}
function Power(a, b){
  return Math.pow(a, b);
}
function Gamma(x){
  return LanczosApproximation(x);
}
function LogGamma(x){
  return Math.log(Gamma(x));
}
function LanczosApproximation(z){
  var p;
  var i, y, t, x;

  p = [];
  p.length = 8;
  p[0] = 676.5203681218851;
  p[1] =  -1259.1392167224028;
  p[2] = 771.32342877765313;
  p[3] =  -176.61502916214059;
  p[4] = 12.507343278686905;
  p[5] =  -0.13857109526572012;
  p[6] = 9.9843695780195716e-6;
  p[7] = 1.5056327351493116e-7;

  if(z < 0.5){
    y = Math.PI/(Math.sin(Math.PI*z)*LanczosApproximation(1 - z));
  }else{
    z = z - 1;
    x = 0.99999999999980993;
    for(i = 0; i < p.length; i = i + 1){
      x = x + p[i]/(z + i + 1);
    }
    t = z + p.length - 0.5;
    y = Math.sqrt(2*Math.PI)*Math.pow(t, z + 0.5)*Math.exp( -t)*x;
  }

  return y;
}
function Beta(x, y){
  return Gamma(x)*Gamma(y)/Gamma(x + y);
}
function Sinh(x){
  return (Math.exp(x) - Math.exp( -x))/2;
}
function Cosh(x){
  return (Math.exp(x) + Math.exp( -x))/2;
}
function Tanh(x){
  return Sinh(x)/Cosh(x);
}
function Cot(x){
  return 1/Math.tan(x);
}
function Sec(x){
  return 1/Math.cos(x);
}
function Csc(x){
  return 1/Math.sin(x);
}
function Coth(x){
  return Cosh(x)/Sinh(x);
}
function Sech(x){
  return 1/Cosh(x);
}
function Csch(x){
  return 1/Sinh(x);
}
function Error(x){
  var y, t, tau, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10;

  if(x == 0){
    y = 0;
  }else if(x < 0){
    y =  -Error( -x);
  }else{
    c1 =  -1.26551223;
    c2 =  +1.00002368;
    c3 =  +0.37409196;
    c4 =  +0.09678418;
    c5 =  -0.18628806;
    c6 =  +0.27886807;
    c7 =  -1.13520398;
    c8 =  +1.48851587;
    c9 =  -0.82215223;
    c10 =  +0.17087277;

    t = 1/(1 + 0.5*Math.abs(x));

    tau = t*Math.exp( -(Math.pow(x, 2)) + c1 + t*(c2 + t*(c3 + t*(c4 + t*(c5 + t*(c6 + t*(c7 + t*(c8 + t*(c9 + t*c10)))))))));

    y = 1 - tau;
  }

  return y;
}
function ErrorInverse(x){
  var y, a, t;

  a = (8*(Math.PI - 3))/(3*Math.PI*(4 - Math.PI));

  t = 2/(Math.PI*a) + Math.log(1 - Math.pow(x, 2))/2;
  y = Sign(x)*Math.sqrt(Math.sqrt(Math.pow(t, 2) - Math.log(1 - Math.pow(x, 2))/a) - t);

  return y;
}
function FallingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x - k);
  }

  return y;
}
function RisingFactorial(x, n){
  var k, y;

  y = 1;

  for(k = 0; k <= n - 1; k = k + 1){
    y = y*(x + k);
  }

  return y;
}
function Hypergeometric(a, b, c, z, maxIterations, precision){
  var y;

  if(Math.abs(z) >= 0.5){
    y = Math.pow(1 - z,  -a)*HypergeometricDirect(a, c - b, c, z/(z - 1), maxIterations, precision);
  }else{
    y = HypergeometricDirect(a, b, c, z, maxIterations, precision);
  }

  return y;
}
function HypergeometricDirect(a, b, c, z, maxIterations, precision){
  var y, yp, n;
  var done;

  y = 0;
  done = false;

  for(n = 0; n < maxIterations &&  !done ; n = n + 1){
    yp = RisingFactorial(a, n)*RisingFactorial(b, n)/RisingFactorial(c, n)*Math.pow(z, n)/Factorial(n);
    if(Math.abs(yp) < precision){
      done = true;
    }
    y = y + yp;
  }

  return y;
}
function BernouilliNumber(n){
  return AkiyamaTanigawaAlgorithm(n);
}
function AkiyamaTanigawaAlgorithm(n){
  var m, j, B;
  var A;

  A = [];
  A.length = n + 1;

  for(m = 0; m <= n; m = m + 1){
    A[m] = 1/(m + 1);
    for(j = m; j >= 1; j = j - 1){
      A[j - 1] = j*(A[j - 1] - A[j]);
    }
  }

  B = A[0];

  delete(A);

  return B;
}
function bAnd4Byte(n1, n2){
  return bAndBytes(n1, n2, 4);
}
function bAnd2Byte(n1, n2){
  return bAndBytes(n1, n2, 2);
}
function bAndByte(n1, n2){
  return bAndBytes(n1, n2, 1);
}
function bAndBytes(n1, n2, bytes){
  var byteVal, result, i;

  byteVal = 1;
  result = 0;

  if(n1 >= 0 && n1 < Math.pow(2, bytes*8) && n2 >= 0 && n2 < Math.pow(2, bytes*8)){
    n1 = Truncate(n1);
    n2 = Truncate(n2);
    bytes = Truncate(bytes);

    for(i = 0; i < bytes*8; i = i + 1){
      if(n1%2 == 1 && n2%2 == 1){
        result = result + byteVal;
      }
      n1 = Math.floor(n1/2);
      n2 = Math.floor(n2/2);
      byteVal = byteVal*2;
    }
  }

  return result;
}
function bOr4Byte(n1, n2){
  return bOrBytes(n1, n2, 4);
}
function bOr2Byte(n1, n2){
  return bOrBytes(n1, n2, 2);
}
function bOrByte(n1, n2){
  return bOrBytes(n1, n2, 1);
}
function bOrBytes(n1, n2, bytes){
  var byteVal, result, i;

  byteVal = 1;
  result = 0;

  if(n1 >= 0 && n1 < Math.pow(2, bytes*8) && n2 >= 0 && n2 < Math.pow(2, bytes*8)){
    n1 = Truncate(n1);
    n2 = Truncate(n2);
    bytes = Truncate(bytes);

    for(i = 0; i < bytes*8; i = i + 1){
      if(n1%2 == 1 || n2%2 == 1){
        result = result + byteVal;
      }
      n1 = Math.floor(n1/2);
      n2 = Math.floor(n2/2);
      byteVal = byteVal*2;
    }
  }

  return result;
}
function bXor4Byte(n1, n2){
  return bXorBytes(n1, n2, 4);
}
function bXor2Byte(n1, n2){
  return bXorBytes(n1, n2, 2);
}
function bXorByte(n1, n2){
  return bXorBytes(n1, n2, 1);
}
function bXorBytes(n1, n2, bytes){
  var byteVal, result, i;

  byteVal = 1;
  result = 0;

  if(n1 >= 0 && n1 < Math.pow(2, bytes*8) && n2 >= 0 && n2 < Math.pow(2, bytes*8)){
    n1 = Truncate(n1);
    n2 = Truncate(n2);
    bytes = Truncate(bytes);

    for(i = 0; i < bytes*8; i = i + 1){
      if(n1%2 != n2%2){
        result = result + byteVal;
      }
      n1 = Math.floor(n1/2);
      n2 = Math.floor(n2/2);
      byteVal = byteVal*2;
    }
  }

  return result;
}
function bNot4Byte(b){
  return bNotBytes(b, 4);
}
function bNot2Byte(b){
  return bNotBytes(b, 2);
}
function bNotByte(b){
  return bNotBytes(b, 1);
}
function bNotBytes(b, lengthx){
  var result;

  result = 0;

  if(b >= 0 && b < Math.pow(2, lengthx*8)){
    b = Truncate(b);
    lengthx = Truncate(lengthx);

    result = Math.pow(2, lengthx*8) - b - 1;
  }

  return result;
}
function bShiftLeft4Byte(b, amount){
  return bShiftLeftBytes(b, amount, 4);
}
function bShiftLeft2Byte(b, amount){
  return bShiftLeftBytes(b, amount, 2);
}
function bShiftLeftByte(b, amount){
  return bShiftLeftBytes(b, amount, 1);
}
function bShiftLeftBytes(b, amount, lengthx){
  var result;

  result = 0;

  if(b >= 0 && b < Math.pow(2, lengthx*8) && amount >= 0 && amount <= lengthx*8){
    b = Truncate(b);
    amount = Truncate(amount);

    result = b*Math.pow(2, amount);
  }

  return result;
}
function bShiftRight4Byte(b, amount){
  return bShiftRightBytes(b, amount, 4);
}
function bShiftRight2Byte(b, amount){
  return bShiftRightBytes(b, amount, 2);
}
function bShiftRightByte(b, amount){
  return bShiftRightBytes(b, amount, 1);
}
function bShiftRightBytes(b, amount, lengthx){
  var result;

  result = 0;

  if(b >= 0 && b < Math.pow(2, lengthx*8) && amount >= 0 && amount <= lengthx*8){
    b = Truncate(b);
    amount = Truncate(amount);

    result = Truncate(b/Math.pow(2, amount));
  }

  return result;
}
function bReadNextBit(data, nextbit){
  var bytenr, bitnumber, bit, b;

  bytenr = Math.floor(nextbit.numberValue/8);
  bitnumber = nextbit.numberValue%8;

  b = data[bytenr];

  bit = Math.floor(b/Math.pow(2, bitnumber))%2;

  nextbit.numberValue = nextbit.numberValue + 1;

  return bit;
}
function bBitExtract(b, fromInc, toInc){
  return Math.floor(b/Math.pow(2, fromInc))%Math.pow(2, toInc + 1 - fromInc);
}
function bReadBitRange(data, nextbit, lengthx){
  var startbyte, endbyte;
  var startbit, endbit;
  var number, i;

  number = 0;

  startbyte = Math.floor(nextbit.numberValue/8);
  endbyte = Math.floor((nextbit.numberValue + lengthx)/8);

  startbit = nextbit.numberValue%8;
  endbit = (nextbit.numberValue + lengthx - 1)%8;

  if(startbyte == endbyte){
    number = bBitExtract(data[startbyte], startbit, endbit);
  }

  nextbit.numberValue = nextbit.numberValue + lengthx;

  return number;
}
function bSkipToBoundary(nextbit){
  var skip;

  skip = 8 - nextbit.numberValue%8;
  nextbit.numberValue = nextbit.numberValue + skip;
}
function bReadNextByteBoundary(data, nextbit){
  var bytenr, b;

  bytenr = Math.floor(nextbit.numberValue/8);
  b = data[bytenr];
  nextbit.numberValue = nextbit.numberValue + 8;

  return b;
}
function bRead2bytesByteBoundary(data, nextbit){
  var r;

  r = 0;
  r = r + Math.pow(2, 8)*bReadNextByteBoundary(data, nextbit);
  r = r + bReadNextByteBoundary(data, nextbit);

  return r;
}
function UnicodeToUTF8(unicode){
  var utf8;
  var lengthx, i, pos, p1, p2, p3, p4, code;

  lengthx = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    code = unicode[i];

    lengthx = lengthx + 1;
    if(code >= 128){
      lengthx = lengthx + 1;
    }
    if(code >= 2048){
      lengthx = lengthx + 1;
    }
    if(code >= 65536){
      lengthx = lengthx + 1;
    }
  }

  utf8 = [];
  utf8.length = lengthx;

  pos = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    code = unicode[i];

    if(code < 128){
      utf8[pos] = code;
      pos = pos + 1;
    }else if(code < 2048){
      p1 = code%64;
      p2 = Math.floor(code/64)%64;

      utf8[pos] = 128 + 64 + p2;
      pos = pos + 1;
      utf8[pos] = 128 + p1;
      pos = pos + 1;
    }else if(code < 65536){
      p1 = code%64;
      p2 = Math.floor(code/64)%64;
      p3 = Math.floor(code/4096);

      utf8[pos] = 128 + 64 + 32 + p3;
      pos = pos + 1;
      utf8[pos] = 128 + p2;
      pos = pos + 1;
      utf8[pos] = 128 + p1;
      pos = pos + 1;
    }else{
      p1 = code%64;
      p2 = Math.floor(code/64)%64;
      p3 = Math.floor(code/4096)%64;
      p4 = Math.floor(code/262144);

      utf8[pos] = 128 + 64 + 32 + 16 + p4;
      pos = pos + 1;
      utf8[pos] = 128 + p3;
      pos = pos + 1;
      utf8[pos] = 128 + p2;
      pos = pos + 1;
      utf8[pos] = 128 + p1;
      pos = pos + 1;
    }
  }

  return utf8;
}
function UTF8ToUnicode(utf8){
  var utf;
  var lengthx, i, pos, p1, p2, p3, p4, code;

  lengthx = 0;
  for(i = 0; i < utf8.length; ){
    code = Math.floor(utf8[i]/8)*8;

    if(code < 128){
      lengthx = lengthx + 1;
      i = i + 1;
    }else if(code == 128 + 64){
      lengthx = lengthx + 1;
      i = i + 2;
    }else if(code == 128 + 64 + 32){
      lengthx = lengthx + 1;
      i = i + 3;
    }else if(code == 128 + 64 + 32 + 16){
      lengthx = lengthx + 1;
      i = i + 4;
    }
  }

  utf = [];
  utf.length = lengthx;

  pos = 0;
  for(i = 0; i < utf8.length; ){
    code = Math.floor(utf8[i]/8)*8;

    if(code < 128){
      utf[pos] = utf8[i];
      i = i + 1;
      pos = pos + 1;
    }else if(code == 128 + 64){
      p1 = utf8[i + 1] - 128;
      p2 = utf8[i] - 128 - 64;

      utf[pos] = p2*64 + p1;

      i = i + 2;
      pos = pos + 1;
    }else if(code == 128 + 64 + 32){
      p1 = utf8[i + 2] - 128;
      p2 = utf8[i + 1] - 128;
      p3 = utf8[i] - 128 - 64 - 32;

      utf[pos] = p3*4096 + p2*64 + p1;

      i = i + 3;
      pos = pos + 1;
    }else if(code == 128 + 64 + 32 + 16){
      p1 = utf8[i + 3] - 128;
      p2 = utf8[i + 2] - 128;
      p3 = utf8[i + 1] - 128;
      p4 = utf8[i] - 128 - 64 - 32 - 16;

      utf[pos] = p4*262144 + p3*4096 + p2*64 + p1;

      i = i + 4;
      pos = pos + 1;
    }
  }

  return utf;
}
function UTF16ToUnicode(utf16){
  var unicode;
  var lengthx, i, pos, hs, ls, code;
  var surrogateLowerBound, surrogateUpperBound, highSurrogateBase, lowSurrogateBase, highMultiplier, surrogateBase;

  surrogateLowerBound = 55296;
  surrogateUpperBound = 57344;
  highSurrogateBase = 55296;
  lowSurrogateBase = 56320;
  highMultiplier = 1024;
  surrogateBase = 65536;

  lengthx = 0;
  for(i = 0; i < utf16.length; ){
    code = utf16[i];

    if(code < surrogateLowerBound || code >= surrogateUpperBound){
      lengthx = lengthx + 1;
      i = i + 1;
    }else{
      lengthx = lengthx + 1;
      i = i + 2;
    }
  }

  unicode = [];
  unicode.length = lengthx;

  pos = 0;
  for(i = 0; i < utf16.length; ){
    code = utf16[i];

    if(code < surrogateLowerBound || code >= surrogateUpperBound){
      unicode[pos] = utf16[i];
      i = i + 1;
      pos = pos + 1;
    }else{
      hs = utf16[i];
      ls = utf16[i + 1];

      unicode[pos] = (hs - highSurrogateBase)*highMultiplier + (ls - lowSurrogateBase) + surrogateBase;

      i = i + 2;
      pos = pos + 1;
    }
  }

  return unicode;
}
function UTF16StringToUnicode(str){
  var utf;
  var lengthx, i, pos, hs, ls, code;
  var surrogateLowerBound, surrogateUpperBound, highSurrogateBase, lowSurrogateBase, highMultiplier, surrogateBase;

  surrogateLowerBound = 55296;
  surrogateUpperBound = 57344;
  highSurrogateBase = 55296;
  lowSurrogateBase = 56320;
  highMultiplier = 1024;
  surrogateBase = 65536;

  lengthx = 0;
  for(i = 0; i < str.length; ){
    code = str[i].charCodeAt(0);

    if(code < surrogateLowerBound || code >= surrogateUpperBound){
      lengthx = lengthx + 1;
      i = i + 1;
    }else{
      lengthx = lengthx + 1;
      i = i + 2;
    }
  }

  utf = [];
  utf.length = lengthx;

  pos = 0;
  for(i = 0; i < str.length; ){
    code = str[i].charCodeAt(0);

    if(code < surrogateLowerBound || code >= surrogateUpperBound){
      utf[pos] = str[i].charCodeAt(0);
      i = i + 1;
      pos = pos + 1;
    }else{
      hs = str[i].charCodeAt(0);
      ls = str[i + 1].charCodeAt(0);

      utf[pos] = (hs - highSurrogateBase)*highMultiplier + (ls - lowSurrogateBase) + surrogateBase;

      i = i + 2;
      pos = pos + 1;
    }
  }

  return utf;
}
function UnicodeToUTF16(unicode){
  var utf16;
  var lengthx, code, i, pos, hs, ls;
  var highSurrogateBase, lowSurrogateBase, highMultiplier, surrogateBase;

  highSurrogateBase = 55296;
  lowSurrogateBase = 56320;
  highMultiplier = 1024;
  surrogateBase = 65536;

  lengthx = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    if(unicode[i] < surrogateBase){
      lengthx = lengthx + 1;
    }else{
      lengthx = lengthx + 2;
    }
  }

  utf16 = [];
  utf16.length = lengthx;

  pos = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    if(unicode[i] < surrogateBase){
      utf16[pos] = unicode[i];
      pos = pos + 1;
    }else{
      code = unicode[i] - surrogateBase;
      hs = Math.floor(code/highMultiplier) + highSurrogateBase;
      ls = code%highMultiplier + lowSurrogateBase;

      utf16[pos] = hs;
      pos = pos + 1;
      utf16[pos] = ls;
      pos = pos + 1;
    }
  }

  return utf16;
}
function UnicodeToUTF16String(unicode){
  var utf16;
  var lengthx, code, i, pos, hs, ls;
  var highSurrogateBase, lowSurrogateBase, highMultiplier, surrogateBase;

  highSurrogateBase = 55296;
  lowSurrogateBase = 56320;
  highMultiplier = 1024;
  surrogateBase = 65536;

  lengthx = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    if(unicode[i] < surrogateBase){
      lengthx = lengthx + 1;
    }else{
      lengthx = lengthx + 2;
    }
  }

  utf16 = [];
  utf16.length = lengthx;

  pos = 0;
  for(i = 0; i < unicode.length; i = i + 1){
    if(unicode[i] < surrogateBase){
      utf16[pos] = String.fromCharCode(unicode[i]);
      pos = pos + 1;
    }else{
      code = unicode[i] - surrogateBase;
      hs = Math.floor(code/highMultiplier) + highSurrogateBase;
      ls = code%highMultiplier + lowSurrogateBase;

      utf16[pos] = String.fromCharCode(hs);
      pos = pos + 1;
      utf16[pos] = String.fromCharCode(ls);
      pos = pos + 1;
    }
  }

  return utf16;
}
function UTF8ToUTF16(utf8){
  var unicode, utf16;

  unicode = UTF8ToUnicode(utf8);
  utf16 = UnicodeToUTF16(unicode);

  delete(unicode);

  return utf16;
}
function UTF16ToUTF8(utf16){
  var unicode, utf8;

  unicode = UTF16ToUnicode(utf16);
  utf8 = UnicodeToUTF8(unicode);

  delete(unicode);

  return utf8;
}
function UTF8ToUTF16String(utf8){
  var unicode;
  var utf16;

  unicode = UTF8ToUnicode(utf8);
  utf16 = UnicodeToUTF16String(unicode);

  delete(unicode);

  return utf16;
}
function UTF16StringToUTF8(utf16){
  var unicode, utf8;

  unicode = UTF16StringToUnicode(utf16);
  utf8 = UnicodeToUTF8(unicode);

  delete(unicode);

  return utf8;
}
function IsValidUnicode(unicode){
  var lower, upper, i, code;
  var valid;

  lower = 0;
  upper = 1114112;

  valid = true;
  for(i = 0; i < unicode.length && valid; i = i + 1){
    code = unicode[i];

    if(code == Math.floor(code)){
      if(code >= lower || code < upper){
      }else{
        valid = false;
      }
    }else{
      valid = false;
    }
  }

  return valid;
}
function IsValidUtf16(utf16){
  var i, code;
  var valid;
  var surrogateLowerBound, surrogateUpperBound;

  surrogateLowerBound = 55296;
  surrogateUpperBound = 57344;

  valid = true;
  for(i = 0; i < utf16.length && valid; i = i + 1){
    code = utf16[i];

    if(code == Math.floor(code)){
      if(code < surrogateLowerBound || code >= surrogateUpperBound){
      }else if(i + 1 < utf16.length){
        i = i + 1;
        code = utf16[i];

        if(code < surrogateLowerBound || code >= surrogateUpperBound){
          valid = false;
        }
      }else{
        valid = false;
      }
    }else{
      valid = false;
    }
  }

  return valid;
}
function IsValidUtf8(utf8){
  var i, code1, code2, code3, code4;
  var valid;

  valid = true;
  for(i = 0; i < utf8.length && valid; i = i + 1){
    code1 = utf8[i];

    if(code1 == Math.floor(code1)){
      if(code1 < 128){
      }else if(code1 < 128 + 64){
        valid = false;
      }else if(code1 < 128 + 64 + 32){
        if(i + 1 < utf8.length){
          i = i + 1;
          code2 = utf8[i];

          if(code2 >= 128 && code2 < 128 + 64){
          }else{
            valid = false;
          }
        }else{
          valid = false;
        }
      }else if(code1 < 128 + 64 + 32 + 16){
        if(i + 2 < utf8.length){
          i = i + 1;
          code2 = utf8[i];
          i = i + 1;
          code3 = utf8[i];

          if(code2 >= 128 && code2 < 128 + 64 && code3 >= 128 && code3 < 128 + 64){
          }else{
            valid = false;
          }
        }else{
          valid = false;
        }
      }else if(i + 3 < utf8.length){
        i = i + 1;
        code2 = utf8[i];
        i = i + 1;
        code3 = utf8[i];
        i = i + 1;
        code4 = utf8[i];

        if(code2 >= 128 && code2 < 128 + 64 && code3 >= 128 && code3 < 128 + 64 && code4 >= 128 && code4 < 128 + 64){
        }else{
          valid = false;
        }
      }else{
        valid = false;
      }
    }else{
      valid = false;
    }
  }

  return valid;
}
function AddNumber(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddNumberRef(list, i){
  list.numberArray = AddNumber(list.numberArray, i);
}
function RemoveNumber(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetNumberRef(list, i){
  return list.numberArray[i];
}
function RemoveNumberRef(list, i){
  list.numberArray = RemoveNumber(list.numberArray, i);
}
function AddString(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;

  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddStringRef(list, i){
  list.stringArray = AddString(list.stringArray, i);
}
function RemoveString(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetStringRef(list, i){
  return list.stringArray[i];
}
function RemoveStringRef(list, i){
  list.stringArray = RemoveString(list.stringArray, i);
}
function AddBoolean(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddBooleanRef(list, i){
  list.booleanArray = AddBoolean(list.booleanArray, i);
}
function RemoveBoolean(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }
		
  return newlist;
}
function GetBooleanRef(list, i){
  return list.booleanArray[i];
}
function RemoveDecimalRef(list, i){
  list.booleanArray = RemoveBoolean(list.booleanArray, i);
}
function CreateLinkedListString(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function LinkedListAddString(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListStringsToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListStringsLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = {};
    array[i].string = node.value;
    node = node.next;
  }

  return array;
}
function LinkedListStringsLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function FreeLinkedListString(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function CreateLinkedListNumbers(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function CreateLinkedListNumbersArray(lengthx){
  var lls;
  var i;

  lls = [];
  lls.length = lengthx;
  for(i = 0; i < lls.length; i = i + 1){
    lls[i] = CreateLinkedListNumbers();
  }

  return lls;
}
function LinkedListAddNumber(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListNumbersLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function LinkedListNumbersIndex(ll, index){
  var i;
  var node;

  node = ll.first;
  for(i = 0; i < index; i = i + 1){
    node = node.next;
  }

  return node.value;
}
function LinkedListInsertNumber(ll, index, value){
  var i;
  var node, tmp;

  if(index == 0){
    tmp = ll.first;
    ll.first = {};
    ll.first.next = tmp;
    ll.first.value = value;
    ll.first.end = false;
  }else{
    node = ll.first;
    for(i = 0; i < index - 1; i = i + 1){
      node = node.next;
    }

    tmp = node.next;
    node.next = {};
    node.next.next = tmp;
    node.next.value = value;
    node.next.end = false;
  }
}
function LinkedListSet(ll, index, value){
  var i;
  var node;

  node = ll.first;
  for(i = 0; i < index; i = i + 1){
    node = node.next;
  }

  node.next.value = value;
}
function LinkedListRemoveNumber(ll, index){
  var i;
  var node, prev;

  node = ll.first;
  prev = ll.first;

  for(i = 0; i < index; i = i + 1){
    prev = node;
    node = node.next;
  }

  if(index == 0){
    ll.first = prev.next;
  }
  if( !prev.next.end ){
    prev.next = prev.next.next;
  }
}
function FreeLinkedListNumbers(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function FreeLinkedListNumbersArray(lls){
  var i;

  for(i = 0; i < lls.length; i = i + 1){
    FreeLinkedListNumbers(lls[i]);
  }
  delete(lls);
}
function LinkedListNumbersToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListNumbersLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = node.value;
    node = node.next;
  }

  return array;
}
function ArrayToLinkedListNumbers(array){
  var ll;
  var i;

  ll = CreateLinkedListNumbers();

  for(i = 0; i < array.length; i = i + 1){
    LinkedListAddNumber(ll, array[i]);
  }

  return ll;
}
function LinkedListNumbersEqual(a, b){
  var equal, done;
  var an, bn;

  an = a.first;
  bn = b.first;

  equal = true;
  done = false;
  for(; equal &&  !done ; ){
    if(an.end == bn.end){
      if(an.end){
        done = true;
      }else if(an.value == bn.value){
        an = an.next;
        bn = bn.next;
      }else{
        equal = false;
      }
    }else{
      equal = false;
    }
  }

  return equal;
}
function CreateLinkedListCharacter(){
  var ll;

  ll = {};
  ll.first = {};
  ll.last = ll.first;
  ll.last.end = true;

  return ll;
}
function LinkedListAddCharacter(ll, value){
  ll.last.end = false;
  ll.last.value = value;
  ll.last.next = {};
  ll.last.next.end = true;
  ll.last = ll.last.next;
}
function LinkedListCharactersToArray(ll){
  var array;
  var lengthx, i;
  var node;

  node = ll.first;

  lengthx = LinkedListCharactersLength(ll);

  array = [];
  array.length = lengthx;

  for(i = 0; i < lengthx; i = i + 1){
    array[i] = node.value;
    node = node.next;
  }

  return array;
}
function LinkedListCharactersLength(ll){
  var l;
  var node;

  l = 0;
  node = ll.first;
  for(;  !node.end ; ){
    node = node.next;
    l = l + 1;
  }

  return l;
}
function FreeLinkedListCharacter(ll){
  var node, prev;

  node = ll.first;

  for(;  !node.end ; ){
    prev = node;
    node = node.next;
    delete(prev);
  }

  delete(node);
}
function CreateDynamicArrayNumbers(){
  var da;

  da = {};
  da.array = [];
  da.array.length = 10;
  da.lengthx = 0;

  return da;
}
function CreateDynamicArrayNumbersWithInitialCapacity(capacity){
  var da;

  da = {};
  da.array = [];
  da.array.length = capacity;
  da.lengthx = 0;

  return da;
}
function DynamicArrayAddNumber(da, value){
  if(da.lengthx == da.array.length){
    DynamicArrayNumbersIncreaseSize(da);
  }

  da.array[da.lengthx] = value;
  da.lengthx = da.lengthx + 1;
}
function DynamicArrayNumbersIncreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*3/2);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < da.array.length; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayNumbersDecreaseSizeNecessary(da){
  var needsDecrease;

  needsDecrease = false;

  if(da.lengthx > 10){
    needsDecrease = da.lengthx <= Math.round(da.array.length*2/3);
  }

  return needsDecrease;
}
function DynamicArrayNumbersDecreaseSize(da){
  var newLength, i;
  var newArray;

  newLength = Math.round(da.array.length*2/3);
  newArray = [];
  newArray.length = newLength;

  for(i = 0; i < newLength; i = i + 1){
    newArray[i] = da.array[i];
  }

  delete(da.array);

  da.array = newArray;
}
function DynamicArrayNumbersIndex(da, index){
  return da.array[index];
}
function DynamicArrayNumbersLength(da){
  return da.lengthx;
}
function DynamicArrayInsertNumber(da, index, value){
  var i;

  if(da.lengthx == da.array.length){
    DynamicArrayNumbersIncreaseSize(da);
  }

  for(i = da.lengthx; i > index; i = i - 1){
    da.array[i] = da.array[i - 1];
  }

  da.array[index] = value;

  da.lengthx = da.lengthx + 1;
}
function DynamicArraySet(da, index, value){
  da.array[index] = value;
}
function DynamicArrayRemoveNumber(da, index){
  var i;

  for(i = index; i < da.lengthx - 1; i = i + 1){
    da.array[i] = da.array[i + 1];
  }

  da.lengthx = da.lengthx - 1;

  if(DynamicArrayNumbersDecreaseSizeNecessary(da)){
    DynamicArrayNumbersDecreaseSize(da);
  }
}
function FreeDynamicArrayNumbers(da){
  delete(da.array);
  delete(da);
}
function DynamicArrayNumbersToArray(da){
  var array;
  var i;

  array = [];
  array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    array[i] = da.array[i];
  }

  return array;
}
function ArrayToDynamicArrayNumbersWithOptimalSize(array){
  var da;
  var i;
  var c, n, newCapacity;

  /*
         c = 10*(3/2)^n
         log(c) = log(10*(3/2)^n)
         log(c) = log(10) + log((3/2)^n)
         log(c) = 1 + log((3/2)^n)
         log(c) - 1 = log((3/2)^n)
         log(c) - 1 = n*log(3/2)
         n = (log(c) - 1)/log(3/2)
         */
  c = array.length;
  n = (Math.log(c) - 1)/Math.log(3/2);
  newCapacity = Math.floor(n) + 1;

  da = CreateDynamicArrayNumbersWithInitialCapacity(newCapacity);

  for(i = 0; i < array.length; i = i + 1){
    da.array[i] = array[i];
  }

  return da;
}
function ArrayToDynamicArrayNumbers(array){
  var da;

  da = {};
  da.array = CopyNumberArray(array);
  da.lengthx = array.length;

  return da;
}
function DynamicArrayNumbersEqual(a, b){
  var equal;
  var i;

  equal = true;
  if(a.lengthx == b.lengthx){
    for(i = 0; i < a.lengthx && equal; i = i + 1){
      if(a.array[i] != b.array[i]){
        equal = false;
      }
    }
  }else{
    equal = false;
  }

  return equal;
}
function DynamicArrayNumbersToLinkedList(da){
  var ll;
  var i;

  ll = CreateLinkedListNumbers();

  for(i = 0; i < da.lengthx; i = i + 1){
    LinkedListAddNumber(ll, da.array[i]);
  }

  return ll;
}
function LinkedListToDynamicArrayNumbers(ll){
  var da;
  var i;
  var node;

  node = ll.first;

  da = {};
  da.lengthx = LinkedListNumbersLength(ll);

  da.array = [];
  da.array.length = da.lengthx;

  for(i = 0; i < da.lengthx; i = i + 1){
    da.array[i] = node.value;
    node = node.next;
  }

  return da;
}
function AddCharacter(list, a){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length + 1;
  for(i = 0; i < list.length; i = i + 1){
    newlist[i] = list[i];
  }
  newlist[list.length] = a;
		
  delete(list);
		
  return newlist;
}
function AddCharacterRef(list, i){
  list.string = AddCharacter(list.string, i);
}
function RemoveCharacter(list, n){
  var newlist;
  var i;

  newlist = [];
  newlist.length = list.length - 1;

  if(n >= 0 && n < list.length){
    for(i = 0; i < list.length; i = i + 1){
      if(i < n){
        newlist[i] = list[i];
      }
      if(i > n){
        newlist[i - 1] = list[i];
      }
    }

    delete(list);
  }else{
    delete(newlist);
  }

  return newlist;
}
function GetCharacterRef(list, i){
  return list.string[i];
}
function RemoveCharacterRef(list, i){
  list.string = RemoveCharacter(list.string, i);
}
function charToLowerCase(character){
  var toReturn;

  toReturn = character;
  if(character == 'A'){
    toReturn = 'a';
  }else if(character == 'B'){
    toReturn = 'b';
  }else if(character == 'C'){
    toReturn = 'c';
  }else if(character == 'D'){
    toReturn = 'd';
  }else if(character == 'E'){
    toReturn = 'e';
  }else if(character == 'F'){
    toReturn = 'f';
  }else if(character == 'G'){
    toReturn = 'g';
  }else if(character == 'H'){
    toReturn = 'h';
  }else if(character == 'I'){
    toReturn = 'i';
  }else if(character == 'J'){
    toReturn = 'j';
  }else if(character == 'K'){
    toReturn = 'k';
  }else if(character == 'L'){
    toReturn = 'l';
  }else if(character == 'M'){
    toReturn = 'm';
  }else if(character == 'N'){
    toReturn = 'n';
  }else if(character == 'O'){
    toReturn = 'o';
  }else if(character == 'P'){
    toReturn = 'p';
  }else if(character == 'Q'){
    toReturn = 'q';
  }else if(character == 'R'){
    toReturn = 'r';
  }else if(character == 'S'){
    toReturn = 's';
  }else if(character == 'T'){
    toReturn = 't';
  }else if(character == 'U'){
    toReturn = 'u';
  }else if(character == 'V'){
    toReturn = 'v';
  }else if(character == 'W'){
    toReturn = 'w';
  }else if(character == 'X'){
    toReturn = 'x';
  }else if(character == 'Y'){
    toReturn = 'y';
  }else if(character == 'Z'){
    toReturn = 'z';
  }

  return toReturn;
}
function charToUpperCase(character){
  var toReturn;

  toReturn = character;
  if(character == 'a'){
    toReturn = 'A';
  }else if(character == 'b'){
    toReturn = 'B';
  }else if(character == 'c'){
    toReturn = 'C';
  }else if(character == 'd'){
    toReturn = 'D';
  }else if(character == 'e'){
    toReturn = 'E';
  }else if(character == 'f'){
    toReturn = 'F';
  }else if(character == 'g'){
    toReturn = 'G';
  }else if(character == 'h'){
    toReturn = 'H';
  }else if(character == 'i'){
    toReturn = 'I';
  }else if(character == 'j'){
    toReturn = 'J';
  }else if(character == 'k'){
    toReturn = 'K';
  }else if(character == 'l'){
    toReturn = 'L';
  }else if(character == 'm'){
    toReturn = 'M';
  }else if(character == 'n'){
    toReturn = 'N';
  }else if(character == 'o'){
    toReturn = 'O';
  }else if(character == 'p'){
    toReturn = 'P';
  }else if(character == 'q'){
    toReturn = 'Q';
  }else if(character == 'r'){
    toReturn = 'R';
  }else if(character == 's'){
    toReturn = 'S';
  }else if(character == 't'){
    toReturn = 'T';
  }else if(character == 'u'){
    toReturn = 'U';
  }else if(character == 'v'){
    toReturn = 'V';
  }else if(character == 'w'){
    toReturn = 'W';
  }else if(character == 'x'){
    toReturn = 'X';
  }else if(character == 'y'){
    toReturn = 'Y';
  }else if(character == 'z'){
    toReturn = 'Z';
  }

  return toReturn;
}
function charIsUpperCase(character){
  var isUpper;

  isUpper = false;
  if(character == 'A'){
    isUpper = true;
  }else if(character == 'B'){
    isUpper = true;
  }else if(character == 'C'){
    isUpper = true;
  }else if(character == 'D'){
    isUpper = true;
  }else if(character == 'E'){
    isUpper = true;
  }else if(character == 'F'){
    isUpper = true;
  }else if(character == 'G'){
    isUpper = true;
  }else if(character == 'H'){
    isUpper = true;
  }else if(character == 'I'){
    isUpper = true;
  }else if(character == 'J'){
    isUpper = true;
  }else if(character == 'K'){
    isUpper = true;
  }else if(character == 'L'){
    isUpper = true;
  }else if(character == 'M'){
    isUpper = true;
  }else if(character == 'N'){
    isUpper = true;
  }else if(character == 'O'){
    isUpper = true;
  }else if(character == 'P'){
    isUpper = true;
  }else if(character == 'Q'){
    isUpper = true;
  }else if(character == 'R'){
    isUpper = true;
  }else if(character == 'S'){
    isUpper = true;
  }else if(character == 'T'){
    isUpper = true;
  }else if(character == 'U'){
    isUpper = true;
  }else if(character == 'V'){
    isUpper = true;
  }else if(character == 'W'){
    isUpper = true;
  }else if(character == 'X'){
    isUpper = true;
  }else if(character == 'Y'){
    isUpper = true;
  }else if(character == 'Z'){
    isUpper = true;
  }

  return isUpper;
}
function charIsLowerCase(character){
  var isLower;

  isLower = false;
  if(character == 'a'){
    isLower = true;
  }else if(character == 'b'){
    isLower = true;
  }else if(character == 'c'){
    isLower = true;
  }else if(character == 'd'){
    isLower = true;
  }else if(character == 'e'){
    isLower = true;
  }else if(character == 'f'){
    isLower = true;
  }else if(character == 'g'){
    isLower = true;
  }else if(character == 'h'){
    isLower = true;
  }else if(character == 'i'){
    isLower = true;
  }else if(character == 'j'){
    isLower = true;
  }else if(character == 'k'){
    isLower = true;
  }else if(character == 'l'){
    isLower = true;
  }else if(character == 'm'){
    isLower = true;
  }else if(character == 'n'){
    isLower = true;
  }else if(character == 'o'){
    isLower = true;
  }else if(character == 'p'){
    isLower = true;
  }else if(character == 'q'){
    isLower = true;
  }else if(character == 'r'){
    isLower = true;
  }else if(character == 's'){
    isLower = true;
  }else if(character == 't'){
    isLower = true;
  }else if(character == 'u'){
    isLower = true;
  }else if(character == 'v'){
    isLower = true;
  }else if(character == 'w'){
    isLower = true;
  }else if(character == 'x'){
    isLower = true;
  }else if(character == 'y'){
    isLower = true;
  }else if(character == 'z'){
    isLower = true;
  }

  return isLower;
}
function charIsLetter(character){
  return charIsUpperCase(character) || charIsLowerCase(character);
}
function charIsNumber(character){
  var isNumberx;

  isNumberx = false;
  if(character == '0'){
    isNumberx = true;
  }else if(character == '1'){
    isNumberx = true;
  }else if(character == '2'){
    isNumberx = true;
  }else if(character == '3'){
    isNumberx = true;
  }else if(character == '4'){
    isNumberx = true;
  }else if(character == '5'){
    isNumberx = true;
  }else if(character == '6'){
    isNumberx = true;
  }else if(character == '7'){
    isNumberx = true;
  }else if(character == '8'){
    isNumberx = true;
  }else if(character == '9'){
    isNumberx = true;
  }

  return isNumberx;
}
function charIsWhiteSpace(character){
  var isWhiteSpacex;

  isWhiteSpacex = false;
  if(character == ' '){
    isWhiteSpacex = true;
  }else if(character == '\t'){
    isWhiteSpacex = true;
  }else if(character == '\n'){
    isWhiteSpacex = true;
  }else if(character == '\r'){
    isWhiteSpacex = true;
  }

  return isWhiteSpacex;
}
function charIsSymbol(character){
  var isSymbolx;

  isSymbolx = false;
  if(character == '!'){
    isSymbolx = true;
  }else if(character == '\"'){
    isSymbolx = true;
  }else if(character == '#'){
    isSymbolx = true;
  }else if(character == '$'){
    isSymbolx = true;
  }else if(character == '%'){
    isSymbolx = true;
  }else if(character == '&'){
    isSymbolx = true;
  }else if(character == '\''){
    isSymbolx = true;
  }else if(character == '('){
    isSymbolx = true;
  }else if(character == ')'){
    isSymbolx = true;
  }else if(character == '*'){
    isSymbolx = true;
  }else if(character == '+'){
    isSymbolx = true;
  }else if(character == ','){
    isSymbolx = true;
  }else if(character == '-'){
    isSymbolx = true;
  }else if(character == '.'){
    isSymbolx = true;
  }else if(character == '/'){
    isSymbolx = true;
  }else if(character == ':'){
    isSymbolx = true;
  }else if(character == ';'){
    isSymbolx = true;
  }else if(character == '<'){
    isSymbolx = true;
  }else if(character == '='){
    isSymbolx = true;
  }else if(character == '>'){
    isSymbolx = true;
  }else if(character == '?'){
    isSymbolx = true;
  }else if(character == '@'){
    isSymbolx = true;
  }else if(character == '['){
    isSymbolx = true;
  }else if(character == '\\'){
    isSymbolx = true;
  }else if(character == ']'){
    isSymbolx = true;
  }else if(character == '^'){
    isSymbolx = true;
  }else if(character == '_'){
    isSymbolx = true;
  }else if(character == '`'){
    isSymbolx = true;
  }else if(character == '{'){
    isSymbolx = true;
  }else if(character == '|'){
    isSymbolx = true;
  }else if(character == '}'){
    isSymbolx = true;
  }else if(character == '~'){
    isSymbolx = true;
  }

  return isSymbolx;
}
function charCharacterIsBefore(a, b){
  var ad, bd;

  ad = a.charCodeAt(0);
  bd = b.charCodeAt(0);

  return ad < bd;
}


module.exports = {
	LoadFont: LoadFont,
	DrawText: DrawText,
	GetHeight: GetHeight,
	GetWidth: GetWidth,
	CreateLinkedListActiveEdges: CreateLinkedListActiveEdges,
	CreateLinkedListActiveEdgesArray: CreateLinkedListActiveEdgesArray,
	LinkedListAddActiveEdge: LinkedListAddActiveEdge,
	LinkedListActiveEdgesLength: LinkedListActiveEdgesLength,
	LinkedListActiveEdgesIndex: LinkedListActiveEdgesIndex,
	LinkedListInsertActiveEdge: LinkedListInsertActiveEdge,
	LinkedListRemoveActiveEdge: LinkedListRemoveActiveEdge,
	FreeLinkedListActiveEdges: FreeLinkedListActiveEdges,
	FreeLinkedListActiveEdgesArray: FreeLinkedListActiveEdgesArray,
	LinkedListActiveEdgesToArray: LinkedListActiveEdgesToArray,
	ArrayToLinkedListActiveEdges: ArrayToLinkedListActiveEdges,
	LinkedListActiveEdgesEqual: LinkedListActiveEdgesEqual,
	GetGlyphBitmapBox: GetGlyphBitmapBox,
	AddPoint: AddPoint,
	TesselateCurve: TesselateCurve,
	TesselateCubic: TesselateCubic,
	FlattenCurves: FlattenCurves,
	SortEdgesQuickSort: SortEdgesQuickSort,
	SortEdgesInsSort: SortEdgesInsSort,
	SortEdges: SortEdges,
	NewActiveEdge: NewActiveEdge,
	HandleClippedEdge: HandleClippedEdge,
	FillActiveEdges: FillActiveEdges,
	RasterizeSortedEdges: RasterizeSortedEdges,
	RasterizeGlyph: RasterizeGlyph,
	GenerateGlyphBitmap: GenerateGlyphBitmap,
	BlendGrayBitmapAndRGBABitmapWithColor: BlendGrayBitmapAndRGBABitmapWithColor,
	CompareTagsEquals: CompareTagsEquals,
	TruncToShort: TruncToShort,
	TruncToShortUnsigned: TruncToShortUnsigned,
	TruncToByteUnsigned: TruncToByteUnsigned,
	FreeFont: FreeFont,
	GetGlyphIndex: GetGlyphIndex,
	GetScaleFactorForPixelHeight: GetScaleFactorForPixelHeight,
	GetFontVMetrics: GetFontVMetrics,
	GetFontBoundingBox: GetFontBoundingBox,
	GetGlyphHMetrics: GetGlyphHMetrics,
	GetGlyphKerningAdvance: GetGlyphKerningAdvance,
	GetGlyphBox: GetGlyphBox,
	ReadOffsetSubtable: ReadOffsetSubtable,
	BufferPeekByte: BufferPeekByte,
	BufferGetByte: BufferGetByte,
	BufferGet: BufferGet,
	BufferSeek: BufferSeek,
	BufferSkip: BufferSkip,
	GetBufferRange: GetBufferRange,
	CffGetIndex: CffGetIndex,
	CffIndexGet: CffIndexGet,
	CffGetInt: CffGetInt,
	CffSkipOperand: CffSkipOperand,
	CffDictGet: CffDictGet,
	CffDictGetInts: CffDictGetInts,
	CffGetSubrs: CffGetSubrs,
	InitFont: InitFont,
	TrackVertex: TrackVertex,
	AddVertex: AddVertex,
	CSContextCloseShape: CSContextCloseShape,
	CSContextRMoveTo: CSContextRMoveTo,
	CSContextRLineTo: CSContextRLineTo,
	CSContextRCCurveTo: CSContextRCCurveTo,
	CidGetGlyphSubrs: CidGetGlyphSubrs,
	CffIndexCount: CffIndexCount,
	CffGetSubr: CffGetSubr,
	CffRunCharString: CffRunCharString,
	ReadCmap: ReadCmap,
	GetGlyfOffset: GetGlyfOffset,
	SetVertex: SetVertex,
	CloseShape: CloseShape,
	GetGlyphShapeTT: GetGlyphShapeTT,
	GetGlyphShape: GetGlyphShape,
	GetGlyphKerningAdvanceGPOS: GetGlyphKerningAdvanceGPOS,
	GetGlyphKerningAdvanceKERN: GetGlyphKerningAdvanceKERN,
	StringToNumberArray: StringToNumberArray,
	NumberArrayToString: NumberArrayToString,
	NumberArraysEqual: NumberArraysEqual,
	BooleanArraysEqual: BooleanArraysEqual,
	StringsEqual: StringsEqual,
	FillNumberArray: FillNumberArray,
	FillString: FillString,
	FillBooleanArray: FillBooleanArray,
	FillNumberArrayRange: FillNumberArrayRange,
	FillBooleanArrayRange: FillBooleanArrayRange,
	FillStringRange: FillStringRange,
	CopyNumberArray: CopyNumberArray,
	CopyBooleanArray: CopyBooleanArray,
	CopyString: CopyString,
	CopyNumberArrayRange: CopyNumberArrayRange,
	CopyBooleanArrayRange: CopyBooleanArrayRange,
	CopyStringRange: CopyStringRange,
	IsLastElement: IsLastElement,
	CreateNumberArray: CreateNumberArray,
	CreateBooleanArray: CreateBooleanArray,
	CreateString: CreateString,
	SwapElementsOfNumberArray: SwapElementsOfNumberArray,
	SwapElementsOfStringArray: SwapElementsOfStringArray,
	ReverseNumberArray: ReverseNumberArray,
	CreateBooleanReference: CreateBooleanReference,
	CreateBooleanArrayReference: CreateBooleanArrayReference,
	CreateBooleanArrayReferenceLengthValue: CreateBooleanArrayReferenceLengthValue,
	FreeBooleanArrayReference: FreeBooleanArrayReference,
	CreateCharacterReference: CreateCharacterReference,
	CreateNumberReference: CreateNumberReference,
	CreateNumberArrayReference: CreateNumberArrayReference,
	CreateNumberArrayReferenceLengthValue: CreateNumberArrayReferenceLengthValue,
	FreeNumberArrayReference: FreeNumberArrayReference,
	CreateStringReference: CreateStringReference,
	CreateStringReferenceLengthValue: CreateStringReferenceLengthValue,
	FreeStringReference: FreeStringReference,
	CreateStringArrayReference: CreateStringArrayReference,
	CreateStringArrayReferenceLengthValue: CreateStringArrayReferenceLengthValue,
	FreeStringArrayReference: FreeStringArrayReference,
	And4Byte: And4Byte,
	And2Byte: And2Byte,
	AndByte: AndByte,
	AndBytes: AndBytes,
	Or4Byte: Or4Byte,
	Or2Byte: Or2Byte,
	OrByte: OrByte,
	OrBytes: OrBytes,
	Xor4Byte: Xor4Byte,
	Xor2Byte: Xor2Byte,
	XorByte: XorByte,
	XorBytes: XorBytes,
	Not4Byte: Not4Byte,
	Not2Byte: Not2Byte,
	NotByte: NotByte,
	NotBytes: NotBytes,
	ShiftLeft4Byte: ShiftLeft4Byte,
	ShiftLeft2Byte: ShiftLeft2Byte,
	ShiftLeftByte: ShiftLeftByte,
	ShiftLeftBytes: ShiftLeftBytes,
	ShiftRight4Byte: ShiftRight4Byte,
	ShiftRight2Byte: ShiftRight2Byte,
	ShiftRightByte: ShiftRightByte,
	ShiftRightBytes: ShiftRightBytes,
	RotateLeft4Byte: RotateLeft4Byte,
	RotateRight4Bytes: RotateRight4Bytes,
	CreateBooleanArrayFromNumber: CreateBooleanArrayFromNumber,
	BooleanArrayToNumber: BooleanArrayToNumber,
	BooleanAnd: BooleanAnd,
	BooleanXor: BooleanXor,
	BooleanNot: BooleanNot,
	ShiftBitsRight4Byte: ShiftBitsRight4Byte,
	ReadNextBit: ReadNextBit,
	BitExtract: BitExtract,
	ReadBitRange: ReadBitRange,
	SkipToBoundary: SkipToBoundary,
	ReadNextByteBoundary: ReadNextByteBoundary,
	Read2bytesByteBoundary: Read2bytesByteBoundary,
	ReadXbytes: ReadXbytes,
	Read4bytesBE: Read4bytesBE,
	Read2bytesBE: Read2bytesBE,
	ReadByte: ReadByte,
	Read4bytesLE: Read4bytesLE,
	WriteByte: WriteByte,
	Write2BytesLE: Write2BytesLE,
	Write4BytesLE: Write4BytesLE,
	Write2BytesBE: Write2BytesBE,
	Write4BytesBE: Write4BytesBE,
	WriteStringBytes: WriteStringBytes,
	AssertFalse: AssertFalse,
	AssertTrue: AssertTrue,
	AssertEquals: AssertEquals,
	AssertBooleansEqual: AssertBooleansEqual,
	AssertCharactersEqual: AssertCharactersEqual,
	AssertStringEquals: AssertStringEquals,
	AssertNumberArraysEqual: AssertNumberArraysEqual,
	AssertBooleanArraysEqual: AssertBooleanArraysEqual,
	AssertStringArraysEqual: AssertStringArraysEqual,
	GetBlack: GetBlack,
	GetWhite: GetWhite,
	GetTransparent: GetTransparent,
	GetGray: GetGray,
	CreateRGBColor: CreateRGBColor,
	CreateRGBAColor: CreateRGBAColor,
	CreateImage: CreateImage,
	DeleteImage: DeleteImage,
	ImageWidth: ImageWidth,
	ImageHeight: ImageHeight,
	SetPixel: SetPixel,
	DrawPixel: DrawPixel,
	CombineAlpha: CombineAlpha,
	AlphaBlend: AlphaBlend,
	DrawHorizontalLine1px: DrawHorizontalLine1px,
	DrawVerticalLine1px: DrawVerticalLine1px,
	DrawRectangle1px: DrawRectangle1px,
	DrawImageOnImage: DrawImageOnImage,
	DrawLine1px: DrawLine1px,
	XiaolinWusLineAlgorithm: XiaolinWusLineAlgorithm,
	OneMinusFractionalPart: OneMinusFractionalPart,
	FractionalPart: FractionalPart,
	SetBrightness: SetBrightness,
	DrawQuadraticBezierCurve: DrawQuadraticBezierCurve,
	QuadraticBezierPoint: QuadraticBezierPoint,
	DrawCubicBezierCurve: DrawCubicBezierCurve,
	CubicBezierPoint: CubicBezierPoint,
	CopyImage: CopyImage,
	GetImagePixel: GetImagePixel,
	HorizontalFlip: HorizontalFlip,
	DrawFilledRectangle: DrawFilledRectangle,
	RotateAntiClockwise90Degrees: RotateAntiClockwise90Degrees,
	DrawCircle: DrawCircle,
	BresenhamsCircleDrawingAlgorithm: BresenhamsCircleDrawingAlgorithm,
	DrawCircleMidpointAlgorithm: DrawCircleMidpointAlgorithm,
	DrawCircleBasicAlgorithm: DrawCircleBasicAlgorithm,
	DrawFilledCircle: DrawFilledCircle,
	DrawFilledCircleMidpointAlgorithm: DrawFilledCircleMidpointAlgorithm,
	DrawFilledCircleBasicAlgorithm: DrawFilledCircleBasicAlgorithm,
	DrawTriangle: DrawTriangle,
	DrawFilledTriangle: DrawFilledTriangle,
	DrawLine: DrawLine,
	DrawLineBresenhamsAlgorithmThick: DrawLineBresenhamsAlgorithmThick,
	DrawLineBresenhamsAlgorithm: DrawLineBresenhamsAlgorithm,
	DrawLineBresenhamsAlgorithmThickPatterned: DrawLineBresenhamsAlgorithmThickPatterned,
	GetLinePattern5: GetLinePattern5,
	GetLinePattern4: GetLinePattern4,
	GetLinePattern3: GetLinePattern3,
	GetLinePattern2: GetLinePattern2,
	GetLinePattern1: GetLinePattern1,
	Blur: Blur,
	CreateBlurForPoint: CreateBlurForPoint,
	GetBase64Table: GetBase64Table,
	GetBase64ReverseTable: GetBase64ReverseTable,
	GetBase64ValidTable: GetBase64ValidTable,
	GetBase64Maximum: GetBase64Maximum,
	IsValidBase64Character: IsValidBase64Character,
	Base64Encode: Base64Encode,
	Base64Encode3Numbers: Base64Encode3Numbers,
	Base64Encode3NumbersNoChecks: Base64Encode3NumbersNoChecks,
	GetBase64CharacterForNumber: GetBase64CharacterForNumber,
	Base64EncodeFromText: Base64EncodeFromText,
	Base64Decode: Base64Decode,
	Base64Decode4Chars: Base64Decode4Chars,
	Base64Decode4WithTable: Base64Decode4WithTable,
	IsValidBase64CharacterTable: IsValidBase64CharacterTable,
	Base64Decode4NoChecks: Base64Decode4NoChecks,
	Base64Decode4WithTableNoChecks: Base64Decode4WithTableNoChecks,
	GetBase64NumberForCharacter: GetBase64NumberForCharacter,
	Base64DecodeToText: Base64DecodeToText,
	WriteStringToStingStream: WriteStringToStingStream,
	WriteCharacterToStingStream: WriteCharacterToStingStream,
	WriteBooleanToStingStream: WriteBooleanToStingStream,
	SubstringWithCheck: SubstringWithCheck,
	Substring: Substring,
	AppendString: AppendString,
	ConcatenateString: ConcatenateString,
	AppendCharacter: AppendCharacter,
	ConcatenateCharacter: ConcatenateCharacter,
	SplitByCharacter: SplitByCharacter,
	IndexOfCharacter: IndexOfCharacter,
	SubstringEqualsWithCheck: SubstringEqualsWithCheck,
	SubstringEquals: SubstringEquals,
	IndexOfString: IndexOfString,
	ContainsCharacter: ContainsCharacter,
	ContainsString: ContainsString,
	ToUpperCase: ToUpperCase,
	ToLowerCase: ToLowerCase,
	EqualsIgnoreCase: EqualsIgnoreCase,
	ReplaceString: ReplaceString,
	ReplaceCharacter: ReplaceCharacter,
	Trim: Trim,
	StartsWith: StartsWith,
	EndsWith: EndsWith,
	SplitByString: SplitByString,
	StringIsBefore: StringIsBefore,
	Negate: Negate,
	Positive: Positive,
	Factorial: Factorial,
	Round: Round,
	BankersRound: BankersRound,
	Ceil: Ceil,
	Floor: Floor,
	Truncate: Truncate,
	Absolute: Absolute,
	Logarithm: Logarithm,
	NaturalLogarithm: NaturalLogarithm,
	Sin: Sin,
	Cos: Cos,
	Tan: Tan,
	Asin: Asin,
	Acos: Acos,
	Atan: Atan,
	Atan2: Atan2,
	Squareroot: Squareroot,
	Exp: Exp,
	DivisibleBy: DivisibleBy,
	Combinations: Combinations,
	Permutations: Permutations,
	EpsilonCompare: EpsilonCompare,
	GreatestCommonDivisor: GreatestCommonDivisor,
	GCDWithSubtraction: GCDWithSubtraction,
	IsInteger: IsInteger,
	GreatestCommonDivisorWithCheck: GreatestCommonDivisorWithCheck,
	LeastCommonMultiple: LeastCommonMultiple,
	Sign: Sign,
	Max: Max,
	Min: Min,
	Power: Power,
	Gamma: Gamma,
	LogGamma: LogGamma,
	LanczosApproximation: LanczosApproximation,
	Beta: Beta,
	Sinh: Sinh,
	Cosh: Cosh,
	Tanh: Tanh,
	Cot: Cot,
	Sec: Sec,
	Csc: Csc,
	Coth: Coth,
	Sech: Sech,
	Csch: Csch,
	Error: Error,
	ErrorInverse: ErrorInverse,
	FallingFactorial: FallingFactorial,
	RisingFactorial: RisingFactorial,
	Hypergeometric: Hypergeometric,
	HypergeometricDirect: HypergeometricDirect,
	BernouilliNumber: BernouilliNumber,
	AkiyamaTanigawaAlgorithm: AkiyamaTanigawaAlgorithm,
	bAnd4Byte: bAnd4Byte,
	bAnd2Byte: bAnd2Byte,
	bAndByte: bAndByte,
	bAndBytes: bAndBytes,
	bOr4Byte: bOr4Byte,
	bOr2Byte: bOr2Byte,
	bOrByte: bOrByte,
	bOrBytes: bOrBytes,
	bXor4Byte: bXor4Byte,
	bXor2Byte: bXor2Byte,
	bXorByte: bXorByte,
	bXorBytes: bXorBytes,
	bNot4Byte: bNot4Byte,
	bNot2Byte: bNot2Byte,
	bNotByte: bNotByte,
	bNotBytes: bNotBytes,
	bShiftLeft4Byte: bShiftLeft4Byte,
	bShiftLeft2Byte: bShiftLeft2Byte,
	bShiftLeftByte: bShiftLeftByte,
	bShiftLeftBytes: bShiftLeftBytes,
	bShiftRight4Byte: bShiftRight4Byte,
	bShiftRight2Byte: bShiftRight2Byte,
	bShiftRightByte: bShiftRightByte,
	bShiftRightBytes: bShiftRightBytes,
	bReadNextBit: bReadNextBit,
	bBitExtract: bBitExtract,
	bReadBitRange: bReadBitRange,
	bSkipToBoundary: bSkipToBoundary,
	bReadNextByteBoundary: bReadNextByteBoundary,
	bRead2bytesByteBoundary: bRead2bytesByteBoundary,
	UnicodeToUTF8: UnicodeToUTF8,
	UTF8ToUnicode: UTF8ToUnicode,
	UTF16ToUnicode: UTF16ToUnicode,
	UTF16StringToUnicode: UTF16StringToUnicode,
	UnicodeToUTF16: UnicodeToUTF16,
	UnicodeToUTF16String: UnicodeToUTF16String,
	UTF8ToUTF16: UTF8ToUTF16,
	UTF16ToUTF8: UTF16ToUTF8,
	UTF8ToUTF16String: UTF8ToUTF16String,
	UTF16StringToUTF8: UTF16StringToUTF8,
	IsValidUnicode: IsValidUnicode,
	IsValidUtf16: IsValidUtf16,
	IsValidUtf8: IsValidUtf8,
	AddNumber: AddNumber,
	AddNumberRef: AddNumberRef,
	RemoveNumber: RemoveNumber,
	GetNumberRef: GetNumberRef,
	RemoveNumberRef: RemoveNumberRef,
	AddString: AddString,
	AddStringRef: AddStringRef,
	RemoveString: RemoveString,
	GetStringRef: GetStringRef,
	RemoveStringRef: RemoveStringRef,
	AddBoolean: AddBoolean,
	AddBooleanRef: AddBooleanRef,
	RemoveBoolean: RemoveBoolean,
	GetBooleanRef: GetBooleanRef,
	RemoveDecimalRef: RemoveDecimalRef,
	CreateLinkedListString: CreateLinkedListString,
	LinkedListAddString: LinkedListAddString,
	LinkedListStringsToArray: LinkedListStringsToArray,
	LinkedListStringsLength: LinkedListStringsLength,
	FreeLinkedListString: FreeLinkedListString,
	CreateLinkedListNumbers: CreateLinkedListNumbers,
	CreateLinkedListNumbersArray: CreateLinkedListNumbersArray,
	LinkedListAddNumber: LinkedListAddNumber,
	LinkedListNumbersLength: LinkedListNumbersLength,
	LinkedListNumbersIndex: LinkedListNumbersIndex,
	LinkedListInsertNumber: LinkedListInsertNumber,
	LinkedListSet: LinkedListSet,
	LinkedListRemoveNumber: LinkedListRemoveNumber,
	FreeLinkedListNumbers: FreeLinkedListNumbers,
	FreeLinkedListNumbersArray: FreeLinkedListNumbersArray,
	LinkedListNumbersToArray: LinkedListNumbersToArray,
	ArrayToLinkedListNumbers: ArrayToLinkedListNumbers,
	LinkedListNumbersEqual: LinkedListNumbersEqual,
	CreateLinkedListCharacter: CreateLinkedListCharacter,
	LinkedListAddCharacter: LinkedListAddCharacter,
	LinkedListCharactersToArray: LinkedListCharactersToArray,
	LinkedListCharactersLength: LinkedListCharactersLength,
	FreeLinkedListCharacter: FreeLinkedListCharacter,
	CreateDynamicArrayNumbers: CreateDynamicArrayNumbers,
	CreateDynamicArrayNumbersWithInitialCapacity: CreateDynamicArrayNumbersWithInitialCapacity,
	DynamicArrayAddNumber: DynamicArrayAddNumber,
	DynamicArrayNumbersIncreaseSize: DynamicArrayNumbersIncreaseSize,
	DynamicArrayNumbersDecreaseSizeNecessary: DynamicArrayNumbersDecreaseSizeNecessary,
	DynamicArrayNumbersDecreaseSize: DynamicArrayNumbersDecreaseSize,
	DynamicArrayNumbersIndex: DynamicArrayNumbersIndex,
	DynamicArrayNumbersLength: DynamicArrayNumbersLength,
	DynamicArrayInsertNumber: DynamicArrayInsertNumber,
	DynamicArraySet: DynamicArraySet,
	DynamicArrayRemoveNumber: DynamicArrayRemoveNumber,
	FreeDynamicArrayNumbers: FreeDynamicArrayNumbers,
	DynamicArrayNumbersToArray: DynamicArrayNumbersToArray,
	ArrayToDynamicArrayNumbersWithOptimalSize: ArrayToDynamicArrayNumbersWithOptimalSize,
	ArrayToDynamicArrayNumbers: ArrayToDynamicArrayNumbers,
	DynamicArrayNumbersEqual: DynamicArrayNumbersEqual,
	DynamicArrayNumbersToLinkedList: DynamicArrayNumbersToLinkedList,
	LinkedListToDynamicArrayNumbers: LinkedListToDynamicArrayNumbers,
	AddCharacter: AddCharacter,
	AddCharacterRef: AddCharacterRef,
	RemoveCharacter: RemoveCharacter,
	GetCharacterRef: GetCharacterRef,
	RemoveCharacterRef: RemoveCharacterRef,
	charToLowerCase: charToLowerCase,
	charToUpperCase: charToUpperCase,
	charIsUpperCase: charIsUpperCase,
	charIsLowerCase: charIsLowerCase,
	charIsLetter: charIsLetter,
	charIsNumber: charIsNumber,
	charIsWhiteSpace: charIsWhiteSpace,
	charIsSymbol: charIsSymbol,
	charCharacterIsBefore: charCharacterIsBefore,
	test: test
};